@@ -4,8 +4,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 475645;
-var summary = 'Do not crash @ nanojit::LIns::isop';
+var BUGNUMBER = 355829;
+var summary = 'js_ValueToObject should return the original object if OBJ_DEFAULT_VALUE returns a primitive value';
 var actual = '';
 var expect = '';
 
@@ -20,28 +20,8 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  jit(true);
-
-  linkarr = new Array();
-  picarr = new Array();
-  textarr = new Array();
-  var f=161;
-  var t=27;
-  var pics = "";
-  var links = "";
-  var texts = "";
-  var s = f+t;
-  var d = "1";
-  picarr[2] = "2";
-  for(i=1;i<picarr.length;i++)
-  {
-    if(pics=="") pics = picarr[i];
-    else{
-      if(picarr[i].indexOf("jpg")==-1 && picarr[i].indexOf("JPG")==-1) picarr[i] = d;
-    }
-  }
-
-  jit(false);
+  var a = [ { valueOf: function() { return null; } } ];
+  a.toLocaleString();
 
   reportCompare(expect, actual, summary);
 
