@@ -4,10 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 424683;
-var summary = 'Throw too much recursion instead of script stack space quota';
-var actual = 'No Error';
-var expect = 'InternalError: too much recursion';
+var BUGNUMBER = 354499;
+var summary = 'Iterating over Array elements';
+var actual = '';
+var expect = '';
 
 
 //-----------------------------------------------------------------------------
@@ -19,17 +19,27 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  function f() { f(); }
 
-  try
+  expect = actual = 'No Crash';
+
+  var obj = {get a(){ return new Object(); }};
+
+  function setter(v)
   {
-    f();
-  }
-  catch(ex)
-  {
-    actual = ex + '';
+    // Push out obj.a from all temp roots
+    var tmp = { get toString() { return new Object(); }};
+    try { String(tmp); } catch (e) {  }
+    gc();
   }
+
+  Array.prototype.__defineGetter__(0, function() { });
+  Array.prototype.__defineSetter__(0, setter);
+
+  for (var i in Iterator(obj))
+    print(uneval(i));
+
+  delete Array.prototype[0];
+
   reportCompare(expect, actual, summary);
 
   exitFunc ('test');
