@@ -7,22 +7,20 @@
  * This test is actually testing the [[Delete]] internal method (8.12.8). Since the
  * language provides no way to directly exercise [[Delete]], the tests are placed here.
  *
- * @path ch11/11.4/11.4.1/11.4.1-4.a-2.js
- * @description delete operator returns true when deleting a configurable accessor property
+ * @path ch11/11.4/11.4.1/11.4.1-4.a-6.js
+ * @description delete operator returns true when deleting a property inside 'with'
  */
 
 
 function testcase() {
-  var o = {};
-
-  // define an accessor
-  // dummy getter
-  var getter = function () { return 1; }
-  var desc = { get: getter, configurable: true };
-  Object.defineProperty(o, "foo", desc);
-    
-  var d = delete o.foo;
-  if (d === true && o.hasOwnProperty("foo") === false) {
+  var o = new Object();
+  o.x = 1;
+  var d;
+  with(o)
+  {
+    d = delete x;
+  }
+  if (d === true && o.x === undefined) {
     return true;
   }
  }
