@@ -467,7 +467,6 @@ void CGameView::RegisterInit()
 	// previously done by CGameView::InitResources
 	RegMemFun(g_TexMan.GetSingletonPtr(), &CTerrainTextureManager::LoadTerrainTextures, L"LoadTerrainTextures", 60);
 	RegMemFun(g_Renderer.GetSingletonPtr(), &CRenderer::LoadAlphaMaps, L"LoadAlphaMaps", 5);
-	RegMemFun(g_Renderer.GetSingletonPtr()->GetWaterManager(), &WaterManager::LoadWaterTextures, L"LoadWaterTextures", 80);
 }
 
 
@@ -477,15 +476,6 @@ void CGameView::BeginFrame()
 	{
 		// Set up cull camera
 		m->CullCamera = m->ViewCamera;
-
-		// One way to fix shadows popping in at the edge of the screen is to widen the culling frustum so that
-		// objects aren't culled as early. The downside is that objects will get rendered even though they appear
-		// off screen, which is somewhat inefficient. A better solution would be to decouple shadow map rendering
-		// from model rendering; as it is now, a shadow map is only rendered if its associated model is to be
-		// rendered.
-		// (See http://trac.wildfiregames.com/ticket/504)
-		m->CullCamera.SetProjection(m->ViewNear, m->ViewFar, GetCullFOV());
-		m->CullCamera.UpdateFrustum();
 	}
 	g_Renderer.SetSceneCamera(m->ViewCamera, m->CullCamera);
 
@@ -523,51 +513,7 @@ void CGameView::EnumerateObjects(const CFrustum& frustum, SceneCollector* c)
 			}
 
 			if (!m->Culling || frustum.IsBoxVisible (CVector3D(0,0,0), bounds)) {
-				//c->Submit(patch);
-
-				// set the renderstate for this patch
-				patch->setDrawState(true);
-
-				// set the renderstate for the neighbors
-				CPatch *nPatch;
-
-				nPatch = pTerrain->GetPatch(i-1,j-1);
-				if(nPatch) nPatch->setDrawState(true);
-
-				nPatch = pTerrain->GetPatch(i,j-1);
-				if(nPatch) nPatch->setDrawState(true);
-
-				nPatch = pTerrain->GetPatch(i+1,j-1);
-				if(nPatch) nPatch->setDrawState(true);
-
-				nPatch = pTerrain->GetPatch(i-1,j);
-				if(nPatch) nPatch->setDrawState(true);
-
-				nPatch = pTerrain->GetPatch(i+1,j);
-				if(nPatch) nPatch->setDrawState(true);
-
-				nPatch = pTerrain->GetPatch(i-1,j+1);
-				if(nPatch) nPatch->setDrawState(true);
-
-				nPatch = pTerrain->GetPatch(i,j+1);
-				if(nPatch) nPatch->setDrawState(true);
-
-				nPatch = pTerrain->GetPatch(i+1,j+1);
-				if(nPatch) nPatch->setDrawState(true);
-			}
-		}
-	}
-
-	// draw the patches
-	for (ssize_t j=0; j<patchesPerSide; j++)
-	{
-		for (ssize_t i=0; i<patchesPerSide; i++)
-		{
-			CPatch* patch=pTerrain->GetPatch(i,j);	// can't fail
-			if(patch->getDrawState() == true)
-			{
 				c->Submit(patch);
-				patch->setDrawState(false);
 			}
 		}
 	}
@@ -775,7 +721,7 @@ void CGameView::Update(const float deltaRealTime)
 		{
 			// Get the most recent interpolated position
 			float frameOffset = m->Game->GetSimulation2()->GetLastFrameOffset();
-			CMatrix3D transform = cmpPosition->GetInterpolatedTransform(frameOffset, false);
+			CMatrix3D transform = cmpPosition->GetInterpolatedTransform(frameOffset);
 			CVector3D pos = transform.GetTranslation();
 
 			if (m->FollowFirstPerson)
@@ -1102,11 +1048,6 @@ float CGameView::GetFOV() const
 	return m->ViewFOV;
 }
 
-float CGameView::GetCullFOV() const
-{
-	return m->ViewFOV + DEGTORAD(6.0f);	//add 6 degrees to the default FOV for use with the culling frustum;
-}
-
 void CGameView::SetCameraProjection()
 {
 	m->ViewCamera.SetProjection(m->ViewNear, m->ViewFar, m->ViewFOV);
