@@ -131,9 +131,15 @@ void CMapReader::LoadMap(const VfsPath& pathname,  const CScriptValRooted& setti
 		RegMemFun(this, &CMapReader::UnpackMap, L"CMapReader::UnpackMap", 1200);
 
 	// read the corresponding XML file
-	RegMemFun(this, &CMapReader::ReadXML, L"CMapReader::ReadXML", 5800);
+	RegMemFun(this, &CMapReader::ReadXML, L"CMapReader::ReadXML", 50);
 
-	// apply data to the world
+	// apply terrain data to the world
+	RegMemFun(this, &CMapReader::ApplyTerrainData, L"CMapReader::ApplyTerrainData", 5);
+
+	// read entities
+	RegMemFun(this, &CMapReader::ReadXMLEntities, L"CMapReader::ReadXMLEntities", 5800);
+
+	// apply misc data to the world
 	RegMemFun(this, &CMapReader::ApplyData, L"CMapReader::ApplyData", 5);
 
 	// load map settings script (must be done after reading map)
@@ -189,10 +195,13 @@ void CMapReader::LoadRandomMap(const CStrW& scriptFile, const CScriptValRooted&
 	// parse RMS results into camera settings
 	RegMemFun(this, &CMapReader::ParseCamera, L"CMapReader::ParseCamera", 5);
 
+	// apply terrain data to the world
+	RegMemFun(this, &CMapReader::ApplyTerrainData, L"CMapReader::ApplyTerrainData", 5);
+
 	// parse RMS results into entities
 	RegMemFun(this, &CMapReader::ParseEntities, L"CMapReader::ParseEntities", 1000);
 
-	// apply data to the world
+	// apply misc data to the world
 	RegMemFun(this, &CMapReader::ApplyData, L"CMapReader::ApplyData", 5);
 
 	// load map settings script (must be done after reading map)
@@ -264,8 +273,7 @@ int CMapReader::UnpackTerrain()
 	return 0;
 }
 
-// ApplyData: take all the input data, and rebuild the scene from it
-int CMapReader::ApplyData()
+int CMapReader::ApplyTerrainData()
 {
 	if (m_PatchesPerSide == 0)
 	{
@@ -296,6 +304,16 @@ int CMapReader::ApplyData()
 		}
 	}
 
+	CmpPtr<ICmpTerrain> cmpTerrain(*pSimContext, SYSTEM_ENTITY);
+	if (cmpTerrain)
+		cmpTerrain->ReloadTerrain();
+
+	return 0;
+}
+
+// ApplyData: take all the input data, and rebuild the scene from it
+int CMapReader::ApplyData()
+{
 	// copy over the lighting parameters
 	if (pLightEnv)
 		*pLightEnv = m_LightEnv;
@@ -327,10 +345,6 @@ int CMapReader::ApplyData()
 		}
 	}
 
-	CmpPtr<ICmpTerrain> cmpTerrain(*pSimContext, SYSTEM_ENTITY);
-	if (cmpTerrain)
-		cmpTerrain->ReloadTerrain();
-
 	return 0;
 }
 
@@ -372,11 +386,18 @@ PSRETURN CMapSummaryReader::LoadMap(const VfsPath& pathname)
 
 CScriptValRooted CMapSummaryReader::GetMapSettings(ScriptInterface& scriptInterface)
 {
-	CScriptValRooted data;
-	scriptInterface.Eval("({})", data);
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+	
+	JS::RootedValue data(cx);
+	scriptInterface.Eval("({})", &data);
 	if (!m_ScriptSettings.empty())
-		scriptInterface.SetProperty(data.get(), "settings", scriptInterface.ParseJSON(m_ScriptSettings), false);
-	return data;
+	{
+		JS::RootedValue scriptSettingsVal(cx);
+		scriptInterface.ParseJSON(m_ScriptSettings, &scriptSettingsVal);
+		scriptInterface.SetProperty(data, "settings", scriptSettingsVal, false);
+	}
+	return CScriptValRooted(cx, data);
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -398,8 +419,11 @@ public:
 
 	CStr ReadScriptSettings();
 
+	// read everything except for entities
+	void ReadXML();
+
 	// return semantics: see Loader.cpp!LoadFunc.
-	int ProgressiveRead();
+	int ProgressiveReadEntities();
 
 private:
 	CXeromyces xmb_file;
@@ -582,13 +606,14 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 	EL(waterbody);
 	EL(type);
 	EL(colour);
+	EL(tint);
 	EL(height);
 	EL(shininess);	// for compatibility
 	EL(waviness);
 	EL(murkiness);
-	EL(tint);
-	EL(reflectiontint);
-	EL(reflectiontintstrength);
+	EL(windangle);
+	EL(reflectiontint);	// for compatibility
+	EL(reflectiontintstrength);	// for compatibility
 	EL(fog);
 	EL(fogcolour);
 	EL(fogfactor);
@@ -720,9 +745,12 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 					
 					if (element_name == el_type)
 					{
-						// TODO: implement this, when WaterManager supports it
+						if (waterelement.GetText() == "default")
+							m_MapReader.pWaterMan->m_WaterType = L"ocean";
+						else
+							m_MapReader.pWaterMan->m_WaterType =  waterelement.GetText().FromUTF8();
 					}
-					else if (element_name == el_shininess)
+					else if (element_name == el_shininess || element_name == el_reflectiontint || element_name == el_reflectiontintstrength)
 					{
 						// deprecated.
 					}
@@ -744,11 +772,10 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 					} \
 
 					READ_COLOUR(el_colour, m_MapReader.pWaterMan->m_WaterColor)
+					READ_COLOUR(el_tint, m_MapReader.pWaterMan->m_WaterTint)
 					READ_FLOAT(el_waviness, m_MapReader.pWaterMan->m_Waviness)
 					READ_FLOAT(el_murkiness, m_MapReader.pWaterMan->m_Murkiness)
-					READ_COLOUR(el_tint, m_MapReader.pWaterMan->m_WaterTint)
-					READ_COLOUR(el_reflectiontint, m_MapReader.pWaterMan->m_ReflectionTint)
-					READ_FLOAT(el_reflectiontintstrength, m_MapReader.pWaterMan->m_ReflectionTintStrength)
+					READ_FLOAT(el_windangle, m_MapReader.pWaterMan->m_WindAngle)
 
 #undef READ_FLOAT
 #undef READ_COLOUR
@@ -1074,17 +1101,11 @@ int CXMLReader::ReadEntities(XMBElement parent, double end_time)
 	return 0;
 }
 
-int CXMLReader::ProgressiveRead()
+void CXMLReader::ReadXML()
 {
-	// yield after this time is reached. balances increased progress bar
-	// smoothness vs. slowing down loading.
-	const double end_time = timer_Time() + 200e-3;
-
-	int ret;
-
-	while (node_idx < nodes.Count)
+	for (int i = 0; i < nodes.Count; ++i)
 	{
-		XMBElement node = nodes.Item(node_idx);
+		XMBElement node = nodes.Item(i);
 		CStr name = xmb_file.GetElementString(node.GetNodeName());
 		if (name == "Terrain")
 		{
@@ -1100,16 +1121,11 @@ int CXMLReader::ProgressiveRead()
 		}
 		else if (name == "ScriptSettings")
 		{
-			//Already loaded - this is to prevent an assertion
+			// Already loaded - this is to prevent an assertion
 		}
 		else if (name == "Entities")
 		{
-			if (!m_MapReader.m_SkipEntities)
-			{
-				ret = ReadEntities(node, end_time);
-				if (ret != 0)	// error or timed out
-					return ret;
-			}
+			// Handled by ProgressiveReadEntities instead
 		}
 		else if (name == "Paths")
 		{
@@ -1129,6 +1145,30 @@ int CXMLReader::ProgressiveRead()
 			debug_printf(L"Invalid XML element in map file: %hs\n", name.c_str());
 			debug_warn(L"Invalid map XML data");
 		}
+	}
+}
+
+int CXMLReader::ProgressiveReadEntities()
+{
+	// yield after this time is reached. balances increased progress bar
+	// smoothness vs. slowing down loading.
+	const double end_time = timer_Time() + 200e-3;
+
+	int ret;
+
+	while (node_idx < nodes.Count)
+	{
+		XMBElement node = nodes.Item(node_idx);
+		CStr name = xmb_file.GetElementString(node.GetNodeName());
+		if (name == "Entities")
+		{
+			if (!m_MapReader.m_SkipEntities)
+			{
+				ret = ReadEntities(node, end_time);
+				if (ret != 0)	// error or timed out
+					return ret;
+			}
+		}
 
 		node_idx++;
 	}
@@ -1170,13 +1210,23 @@ int CMapReader::LoadMapSettings()
 	return 0;
 }
 
-// progressive
 int CMapReader::ReadXML()
 {
 	if (!xml_reader)
 		xml_reader = new CXMLReader(filename_xml, *this);
 
-	int ret = xml_reader->ProgressiveRead();
+	xml_reader->ReadXML();
+
+	return 0;
+}
+
+// progressive
+int CMapReader::ReadXMLEntities()
+{
+	if (!xml_reader)
+		xml_reader = new CXMLReader(filename_xml, *this);
+
+	int ret = xml_reader->ProgressiveReadEntities();
 	// finished or failed
 	if (ret <= 0)
 	{
@@ -1210,6 +1260,9 @@ int CMapReader::LoadRMSettings()
 
 int CMapReader::GenerateMap()
 {
+	JSContext* cx = pSimulation2->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+	
 	if (!m_MapGen)
 	{
 		// Initialize map generator
@@ -1220,8 +1273,10 @@ int CMapReader::GenerateMap()
 		if (m_ScriptFile.length())
 			scriptPath = L"maps/random/"+m_ScriptFile;
 
+		// TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+		JS::RootedValue tmpScriptSettings(cx, m_ScriptSettings.get());
 		// Stringify settings to pass across threads
-		std::string scriptSettings = pSimulation2->GetScriptInterface().StringifyJSON(m_ScriptSettings.get());
+		std::string scriptSettings = pSimulation2->GetScriptInterface().StringifyJSON(&tmpScriptSettings);
 		
 		// Try to generate map
 		m_MapGen->GenerateMap(scriptPath, scriptSettings);
@@ -1240,15 +1295,17 @@ int CMapReader::GenerateMap()
 		shared_ptr<ScriptInterface::StructuredClone> results = m_MapGen->GetResults();
 
 		// Parse data into simulation context
-		CScriptValRooted data(pSimulation2->GetScriptInterface().GetContext(), pSimulation2->GetScriptInterface().ReadStructuredClone(results));
-		if (data.undefined())
+		JS::RootedValue data(cx); 
+		pSimulation2->GetScriptInterface().ReadStructuredClone(results, &data);
+		
+		if (data.isUndefined())
 		{
 			// RMS failed - return to main menu
 			throw PSERROR_Game_World_MapLoadFailed("Error generating random map.\nCheck application log for details.");
 		}
 		else
 		{
-			m_MapData = data;
+			m_MapData = CScriptValRooted(cx, data);
 		}
 	}
 	else
@@ -1268,7 +1325,9 @@ int CMapReader::GenerateMap()
 int CMapReader::ParseTerrain()
 {
 	TIMER(L"ParseTerrain");
-
+	JSContext* cx = pSimulation2->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+	
 	// parse terrain from map data
 	//	an error here should stop the loading process
 #define GET_TERRAIN_PROPERTY(val, prop, out)\
@@ -1276,17 +1335,18 @@ int CMapReader::ParseTerrain()
 		{	LOGERROR(L"CMapReader::ParseTerrain() failed to get '%hs' property", #prop);\
 			throw PSERROR_Game_World_MapLoadFailed("Error parsing terrain data.\nCheck application log for details"); }
 
+	JS::RootedValue tmpMapData(cx, m_MapData.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
 	u32 size;
-	GET_TERRAIN_PROPERTY(m_MapData.get(), size, size)
+	GET_TERRAIN_PROPERTY(tmpMapData, size, size)
 
 	m_PatchesPerSide = size / PATCH_SIZE;
 
 	// flat heightmap of u16 data
-	GET_TERRAIN_PROPERTY(m_MapData.get(), height, m_Heightmap)
+	GET_TERRAIN_PROPERTY(tmpMapData, height, m_Heightmap)
 
 	// load textures
 	std::vector<std::string> textureNames;
-	GET_TERRAIN_PROPERTY(m_MapData.get(), textureNames, textureNames)
+	GET_TERRAIN_PROPERTY(tmpMapData, textureNames, textureNames)
 	num_terrain_tex = textureNames.size();
 
 	while (cur_terrain_tex < num_terrain_tex)
@@ -1301,14 +1361,14 @@ int CMapReader::ParseTerrain()
 	// build tile data
 	m_Tiles.resize(SQR(size));
 
-	CScriptValRooted tileData;
-	GET_TERRAIN_PROPERTY(m_MapData.get(), tileData, tileData)
+	JS::RootedValue tileData(cx);
+	GET_TERRAIN_PROPERTY(tmpMapData, tileData, &tileData)
 
 	// parse tile data object into flat arrays
 	std::vector<u16> tileIndex;
 	std::vector<u16> tilePriority;
-	GET_TERRAIN_PROPERTY(tileData.get(), index, tileIndex);
-	GET_TERRAIN_PROPERTY(tileData.get(), priority, tilePriority);
+	GET_TERRAIN_PROPERTY(tileData, index, tileIndex);
+	GET_TERRAIN_PROPERTY(tileData, priority, tilePriority);
 
 	ENSURE(SQR(size) == tileIndex.size() && SQR(size) == tilePriority.size());
 
@@ -1342,11 +1402,15 @@ int CMapReader::ParseTerrain()
 int CMapReader::ParseEntities()
 {
 	TIMER(L"ParseEntities");
+	JSContext* cx = pSimulation2->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+	
+	JS::RootedValue tmpMapData(cx, m_MapData.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
 
 	// parse entities from map data
 	std::vector<Entity> entities;
 
-	if (!pSimulation2->GetScriptInterface().GetProperty(m_MapData.get(), "entities", entities))
+	if (!pSimulation2->GetScriptInterface().GetProperty(tmpMapData, "entities", entities))
 		LOGWARNING(L"CMapReader::ParseEntities() failed to get 'entities' property");
 
 	CSimulation2& sim = *pSimulation2;
@@ -1405,15 +1469,19 @@ int CMapReader::ParseEntities()
 int CMapReader::ParseEnvironment()
 {
 	// parse environment settings from map data
+	JSContext* cx = pSimulation2->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+	
+	JS::RootedValue tmpMapData(cx, m_MapData.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
 
 #define GET_ENVIRONMENT_PROPERTY(val, prop, out)\
 	if (!pSimulation2->GetScriptInterface().GetProperty(val, #prop, out))\
 		LOGWARNING(L"CMapReader::ParseEnvironment() failed to get '%hs' property", #prop);
 
-	CScriptValRooted envObj;
-	GET_ENVIRONMENT_PROPERTY(m_MapData.get(), Environment, envObj)
+	JS::RootedValue envObj(cx);
+	GET_ENVIRONMENT_PROPERTY(tmpMapData, Environment, &envObj)
 
-	if (envObj.undefined())
+	if (envObj.isUndefined())
 	{
 		LOGWARNING(L"CMapReader::ParseEnvironment(): Environment settings not found");
 		return 0;
@@ -1424,35 +1492,35 @@ int CMapReader::ParseEnvironment()
 		pPostproc->SetPostEffect(L"default");
 
 	std::wstring skySet;
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), SkySet, skySet)
+	GET_ENVIRONMENT_PROPERTY(envObj, SkySet, skySet)
 	if (pSkyMan)
 		pSkyMan->SetSkySet(skySet);
 
 	CColor sunColor;
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), SunColour, sunColor)
+	GET_ENVIRONMENT_PROPERTY(envObj, SunColour, sunColor)
 	m_LightEnv.m_SunColor = RGBColor(sunColor.r, sunColor.g, sunColor.b);
 
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), SunElevation, m_LightEnv.m_Elevation)
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), SunRotation, m_LightEnv.m_Rotation)
+	GET_ENVIRONMENT_PROPERTY(envObj, SunElevation, m_LightEnv.m_Elevation)
+	GET_ENVIRONMENT_PROPERTY(envObj, SunRotation, m_LightEnv.m_Rotation)
 	
 	CColor terrainAmbientColor;
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), TerrainAmbientColour, terrainAmbientColor)
+	GET_ENVIRONMENT_PROPERTY(envObj, TerrainAmbientColour, terrainAmbientColor)
 	m_LightEnv.m_TerrainAmbientColor = RGBColor(terrainAmbientColor.r, terrainAmbientColor.g, terrainAmbientColor.b);
 
 	CColor unitsAmbientColor;
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), UnitsAmbientColour, unitsAmbientColor)
+	GET_ENVIRONMENT_PROPERTY(envObj, UnitsAmbientColour, unitsAmbientColor)
 	m_LightEnv.m_UnitsAmbientColor = RGBColor(unitsAmbientColor.r, unitsAmbientColor.g, unitsAmbientColor.b);
 
 	// Water properties
-	CScriptValRooted waterObj;
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), Water, waterObj)
+	JS::RootedValue waterObj(cx);
+	GET_ENVIRONMENT_PROPERTY(envObj, Water, &waterObj)
 
-	CScriptValRooted waterBodyObj;
-	GET_ENVIRONMENT_PROPERTY(waterObj.get(), WaterBody, waterBodyObj)
+	JS::RootedValue waterBodyObj(cx);
+	GET_ENVIRONMENT_PROPERTY(waterObj, WaterBody, &waterBodyObj)
 
 	// Water level - necessary
 	float waterHeight;
-	GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Height, waterHeight)
+	GET_ENVIRONMENT_PROPERTY(waterBodyObj, Height, waterHeight)
 
 	CmpPtr<ICmpWaterManager> cmpWaterManager(*pSimulation2, SYSTEM_ENTITY);
 	ENSURE(cmpWaterManager);
@@ -1461,39 +1529,39 @@ int CMapReader::ParseEnvironment()
 	// If we have graphics, get rest of settings
 	if (pWaterMan)
 	{
-		// TODO: Water type not implemented
-
-		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Colour, pWaterMan->m_WaterColor)
-		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Waviness, pWaterMan->m_Waviness)
-		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Murkiness, pWaterMan->m_Murkiness)
-		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Tint, pWaterMan->m_WaterTint)
-		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), ReflectionTint, pWaterMan->m_ReflectionTint)
-		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), ReflectionTintStrength, pWaterMan->m_ReflectionTintStrength)
+		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Type, pWaterMan->m_WaterType)
+		if (pWaterMan->m_WaterType == L"default")
+			pWaterMan->m_WaterType = L"ocean";
+		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Colour, pWaterMan->m_WaterColor)
+		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Tint, pWaterMan->m_WaterTint)
+		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Waviness, pWaterMan->m_Waviness)
+		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Murkiness, pWaterMan->m_Murkiness)
+		GET_ENVIRONMENT_PROPERTY(waterBodyObj, WindAngle, pWaterMan->m_WindAngle)
 	}
 
-	CScriptValRooted fogObject;
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), Fog, fogObject);
+	JS::RootedValue fogObject(cx);
+	GET_ENVIRONMENT_PROPERTY(envObj, Fog, &fogObject);
 
-	GET_ENVIRONMENT_PROPERTY(fogObject.get(), FogFactor, m_LightEnv.m_FogFactor);
-	GET_ENVIRONMENT_PROPERTY(fogObject.get(), FogThickness, m_LightEnv.m_FogMax);
+	GET_ENVIRONMENT_PROPERTY(fogObject, FogFactor, m_LightEnv.m_FogFactor);
+	GET_ENVIRONMENT_PROPERTY(fogObject, FogThickness, m_LightEnv.m_FogMax);
 
 	CColor fogColor;
-	GET_ENVIRONMENT_PROPERTY(fogObject.get(), FogColor, fogColor);
+	GET_ENVIRONMENT_PROPERTY(fogObject, FogColor, fogColor);
 	m_LightEnv.m_FogColor = RGBColor(fogColor.r, fogColor.g, fogColor.b);
 
-	CScriptValRooted postprocObject;
-	GET_ENVIRONMENT_PROPERTY(envObj.get(), Postproc, postprocObject);
+	JS::RootedValue postprocObject(cx);
+	GET_ENVIRONMENT_PROPERTY(envObj, Postproc, &postprocObject);
 
 	std::wstring postProcEffect;
-	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), PostprocEffect, postProcEffect);
+	GET_ENVIRONMENT_PROPERTY(postprocObject, PostprocEffect, postProcEffect);
 
 	if (pPostproc)
 		pPostproc->SetPostEffect(postProcEffect);
 
-	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), Brightness, m_LightEnv.m_Brightness);
-	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), Contrast, m_LightEnv.m_Contrast);
-	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), Saturation, m_LightEnv.m_Saturation);
-	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), Bloom, m_LightEnv.m_Bloom);
+	GET_ENVIRONMENT_PROPERTY(postprocObject, Brightness, m_LightEnv.m_Brightness);
+	GET_ENVIRONMENT_PROPERTY(postprocObject, Contrast, m_LightEnv.m_Contrast);
+	GET_ENVIRONMENT_PROPERTY(postprocObject, Saturation, m_LightEnv.m_Saturation);
+	GET_ENVIRONMENT_PROPERTY(postprocObject, Bloom, m_LightEnv.m_Bloom);
 
 	m_LightEnv.CalculateSunDirection();
 
@@ -1504,6 +1572,8 @@ int CMapReader::ParseEnvironment()
 
 int CMapReader::ParseCamera()
 {
+	JSContext* cx = pSimulation2->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
 	// parse camera settings from map data
 	// defaults if we don't find player starting camera
 	float declination = DEGTORAD(30.f), rotation = DEGTORAD(-45.f);
@@ -1513,17 +1583,18 @@ int CMapReader::ParseCamera()
 	if (!pSimulation2->GetScriptInterface().GetProperty(val, #prop, out))\
 		LOGWARNING(L"CMapReader::ParseCamera() failed to get '%hs' property", #prop);
 
-	CScriptValRooted cameraObj;
-	GET_CAMERA_PROPERTY(m_MapData.get(), Camera, cameraObj)
+	JS::RootedValue tmpMapData(cx, m_MapData.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+	JS::RootedValue cameraObj(cx);
+	GET_CAMERA_PROPERTY(tmpMapData, Camera, &cameraObj)
 
-	if (!cameraObj.undefined())
+	if (!cameraObj.isUndefined())
 	{	// If camera property exists, read values
 		CFixedVector3D pos;
-		GET_CAMERA_PROPERTY(cameraObj.get(), Position, pos)
+		GET_CAMERA_PROPERTY(cameraObj, Position, pos)
 		translation = pos;
 
-		GET_CAMERA_PROPERTY(cameraObj.get(), Rotation, rotation)
-		GET_CAMERA_PROPERTY(cameraObj.get(), Declination, declination)
+		GET_CAMERA_PROPERTY(cameraObj, Rotation, rotation)
+		GET_CAMERA_PROPERTY(cameraObj, Declination, declination)
 	}
 #undef GET_CAMERA_PROPERTY
 
