@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -53,7 +53,7 @@ public:
 	~CMapReader();
 
 	// LoadMap: try to load the map from given file; reinitialise the scene to new data if successful
-	void LoadMap(const VfsPath& pathname, CTerrain*, WaterManager*, SkyManager*, CLightEnv*, CGameView*,
+	void LoadMap(const VfsPath& pathname, const CScriptValRooted& settings, CTerrain*, WaterManager*, SkyManager*, CLightEnv*, CGameView*,
 		CCinemaManager*, CTriggerManager*, CPostprocManager* pPostproc, CSimulation2*, const CSimContext*, 
 	        int playerID, bool skipEntities);
 
@@ -79,10 +79,14 @@ private:
 
 	// ApplyData: take all the input data, and rebuild the scene from it
 	int ApplyData();
+	int ApplyTerrainData();
 
-	// ReadXML: read some other data (entities, etc) in XML format
+	// read some misc data from the XML file
 	int ReadXML();
 
+	// read entity data from the XML file
+	int ReadXMLEntities();
+
 	// clean up everything used during delayed load
 	int DelayLoadFinished();
 	
