@@ -52,6 +52,7 @@
 #include "ps/SavedGame.h"
 #include "ps/scripting/JSInterface_ConfigDB.h"
 #include "ps/scripting/JSInterface_Console.h"
+#include "ps/scripting/JSInterface_Mod.h"
 #include "ps/scripting/JSInterface_VFS.h"
 #include "ps/UserReport.h"
 #include "ps/GameSetup/Atlas.h"
@@ -86,13 +87,23 @@ namespace {
 // Note that the initData argument may only contain clonable data.
 // Functions aren't supported for example!
 // TODO: Use LOGERROR to print a friendly error message when the requirements aren't met instead of failing with debug_warn when cloning.
-void PushGuiPage(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name, CScriptVal initData)
+void PushGuiPage(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name, CScriptVal initData1)
 {
-	g_GUI->PushPage(name, pCxPrivate->pScriptInterface->WriteStructuredClone(initData.get()));
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	// TODO: Get Handle parameter directly with SpiderMonkey 31
+	JS::RootedValue initData(cx, initData1.get());
+	
+	g_GUI->PushPage(name, pCxPrivate->pScriptInterface->WriteStructuredClone(initData));
 }
 
-void SwitchGuiPage(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name, CScriptVal initData)
+void SwitchGuiPage(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name, CScriptVal initData1)
 {
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	// TODO: Get Handle parameter directly with SpiderMonkey 31
+	JS::RootedValue initData(cx, initData1.get());
+	
 	g_GUI->SwitchPage(name, pCxPrivate->pScriptInterface, initData);
 }
 
@@ -104,32 +115,43 @@ void PopGuiPage(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 // Note that the args argument may only contain clonable data.
 // Functions aren't supported for example!
 // TODO: Use LOGERROR to print a friendly error message when the requirements aren't met instead of failing with debug_warn when cloning.
-void PopGuiPageCB(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal args)
+void PopGuiPageCB(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal args1)
 {
-	g_GUI->PopPageCB(pCxPrivate->pScriptInterface->WriteStructuredClone(args.get()));
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	// TODO: Get Handle parameter directly with SpiderMonkey 31
+	JS::RootedValue args(cx, args1.get());
+	
+	g_GUI->PopPageCB(pCxPrivate->pScriptInterface->WriteStructuredClone(args));
 }
 
-CScriptVal GuiInterfaceCall(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name, CScriptVal data)
+CScriptVal GuiInterfaceCall(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name, CScriptVal data1)
 {
+	// TODO: With ESR31 we should be able to take JS::HandleValue directly 
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	JS::RootedValue data(cx, data1.get());
+	
 	if (!g_Game)
-		return JSVAL_VOID;
+		return JS::UndefinedValue();
 	CSimulation2* sim = g_Game->GetSimulation2();
 	ENSURE(sim);
 
 	CmpPtr<ICmpGuiInterface> cmpGuiInterface(*sim, SYSTEM_ENTITY);
 	if (!cmpGuiInterface)
-		return JSVAL_VOID;
+		return JS::UndefinedValue();
 
-	int player = -1;
-	if (g_Game)
-		player = g_Game->GetPlayerID();
+	int player = g_Game->GetPlayerID();
 
-	CScriptValRooted arg (sim->GetScriptInterface().GetContext(), sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), data.get()));
-	CScriptVal ret (cmpGuiInterface->ScriptCall(player, name, arg.get()));
-	return pCxPrivate->pScriptInterface->CloneValueFromOtherContext(sim->GetScriptInterface(), ret.get());
+	JSContext* cxSim = sim->GetScriptInterface().GetContext();
+	JSAutoRequest rqSim(cxSim);
+	JS::RootedValue arg(cxSim, sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), data));
+	JS::RootedValue ret(cxSim, cmpGuiInterface->ScriptCall(player, name, CScriptVal(arg)).get());
+
+	return pCxPrivate->pScriptInterface->CloneValueFromOtherContext(sim->GetScriptInterface(), ret);
 }
 
-void PostNetworkCommand(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal cmd)
+void PostNetworkCommand(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal cmd1)
 {
 	if (!g_Game)
 		return;
@@ -139,10 +161,17 @@ void PostNetworkCommand(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal cmd)
 	CmpPtr<ICmpCommandQueue> cmpCommandQueue(*sim, SYSTEM_ENTITY);
 	if (!cmpCommandQueue)
 		return;
+	
+	// TODO: With ESR31 we should be able to take JS::HandleValue directly 
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	JS::RootedValue cmd(cx, cmd1.get());
 
-	jsval cmd2 = sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), cmd.get());
+	JSContext* cxSim = sim->GetScriptInterface().GetContext();
+	JSAutoRequest rqSim(cxSim);
+	JS::RootedValue cmd2(cxSim, sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), cmd));
 
-	cmpCommandQueue->PostNetworkCommand(cmd2);
+	cmpCommandQueue->PostNetworkCommand(CScriptVal(cmd2));
 }
 
 std::vector<entity_id_t> PickEntitiesAtPoint(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int x, int y, int range)
@@ -202,76 +231,108 @@ void StartNetworkGame(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 	g_NetServer->StartGame();
 }
 
-void StartGame(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal attribs, int playerID)
+void StartGame(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal attribs1, int playerID)
 {
 	ENSURE(!g_NetServer);
 	ENSURE(!g_NetClient);
 
 	ENSURE(!g_Game);
 	g_Game = new CGame();
+	
+	// TODO: With ESR31 we should be able to take JS::HandleValue directly 
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	JS::RootedValue attribs(cx, attribs1.get());
 
 	// Convert from GUI script context to sim script context
 	CSimulation2* sim = g_Game->GetSimulation2();
-	CScriptValRooted gameAttribs (sim->GetScriptInterface().GetContext(),
-			sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), attribs.get()));
+	JSContext* cxSim = sim->GetScriptInterface().GetContext();
+	JSAutoRequest rqSim(cxSim);
+	
+	JS::RootedValue gameAttribs(cxSim, 
+		sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), attribs));
 
 	g_Game->SetPlayerID(playerID);
-	g_Game->StartGame(gameAttribs, "");
+	g_Game->StartGame(CScriptValRooted(cxSim, gameAttribs), "");
 }
 
 CScriptVal StartSavedGame(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name)
 {
+	// We need to be careful with different compartments and contexts.
+	// The GUI calls this function from the GUI context and expects the return value in the same context.
+	// The game we start from here creates another context and expects data in this context.
+	
+	JSContext* cxGui = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cxGui);
+
 	ENSURE(!g_NetServer);
 	ENSURE(!g_NetClient);
 
 	ENSURE(!g_Game);
 
 	// Load the saved game data from disk
-	CScriptValRooted metadata;
+	JS::RootedValue guiContextMetadata(cxGui);
 	std::string savedState;
-	Status err = SavedGames::Load(name, *(pCxPrivate->pScriptInterface), metadata, savedState);
+	Status err = SavedGames::Load(name, *(pCxPrivate->pScriptInterface), &guiContextMetadata, savedState);
 	if (err < 0)
-		return CScriptVal();
+		return JS::UndefinedValue();
 
 	g_Game = new CGame();
+	
+	{
+		CSimulation2* sim = g_Game->GetSimulation2();
+		JSContext* cxGame = sim->GetScriptInterface().GetContext();
+		JSAutoRequest rq(cxGame);
+		
+		JS::RootedValue gameContextMetadata(cxGame, 
+			sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), guiContextMetadata));
+		JS::RootedValue gameInitAttributes(cxGame);
+		sim->GetScriptInterface().GetProperty(gameContextMetadata, "initAttributes", &gameInitAttributes);
 
-	// Convert from GUI script context to sim script context
-	CSimulation2* sim = g_Game->GetSimulation2();
-	CScriptValRooted gameMetadata (sim->GetScriptInterface().GetContext(),
-		sim->GetScriptInterface().CloneValueFromOtherContext(*(pCxPrivate->pScriptInterface), metadata.get()));
+		int playerID;
+		sim->GetScriptInterface().GetProperty(gameContextMetadata, "player", playerID);
 
-	CScriptValRooted gameInitAttributes;
-	sim->GetScriptInterface().GetProperty(gameMetadata.get(), "initAttributes", gameInitAttributes);
+		// Start the game
+		g_Game->SetPlayerID(playerID);
+		g_Game->StartGame(CScriptValRooted(cxGame, gameInitAttributes), savedState);
+	}
 
-	int playerID;
-	sim->GetScriptInterface().GetProperty(gameMetadata.get(), "player", playerID);
-
-	// Start the game
-	g_Game->SetPlayerID(playerID);
-	g_Game->StartGame(gameInitAttributes, savedState);
-
-	return metadata.get();
+	return guiContextMetadata.get();
 }
 
-void SaveGame(ScriptInterface::CxPrivate* pCxPrivate, std::wstring filename, std::wstring description, CScriptVal GUIMetadata)
+void SaveGame(ScriptInterface::CxPrivate* pCxPrivate, std::wstring filename, std::wstring description, CScriptVal GUIMetadata1)
 {
-	shared_ptr<ScriptInterface::StructuredClone> GUIMetadataClone = pCxPrivate->pScriptInterface->WriteStructuredClone(GUIMetadata.get());
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	// TODO: Get Handle parameter directly with SpiderMonkey 31
+	JS::RootedValue GUIMetadata(cx, GUIMetadata1.get());
+	
+	shared_ptr<ScriptInterface::StructuredClone> GUIMetadataClone = pCxPrivate->pScriptInterface->WriteStructuredClone(GUIMetadata);
 	if (SavedGames::Save(filename, description, *g_Game->GetSimulation2(), GUIMetadataClone, g_Game->GetPlayerID()) < 0)
 		LOGERROR(L"Failed to save game");
 }
 
-void SaveGamePrefix(ScriptInterface::CxPrivate* pCxPrivate, std::wstring prefix, std::wstring description, CScriptVal GUIMetadata)
+void SaveGamePrefix(ScriptInterface::CxPrivate* pCxPrivate, std::wstring prefix, std::wstring description, CScriptVal GUIMetadata1)
 {
-	shared_ptr<ScriptInterface::StructuredClone> GUIMetadataClone = pCxPrivate->pScriptInterface->WriteStructuredClone(GUIMetadata.get());
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	// TODO: Get Handle parameter directly with SpiderMonkey 31
+	JS::RootedValue GUIMetadata(cx, GUIMetadata1.get());
+	
+	shared_ptr<ScriptInterface::StructuredClone> GUIMetadataClone = pCxPrivate->pScriptInterface->WriteStructuredClone(GUIMetadata);
 	if (SavedGames::SavePrefix(prefix, description, *g_Game->GetSimulation2(), GUIMetadataClone, g_Game->GetPlayerID()) < 0)
 		LOGERROR(L"Failed to save game");
 }
 
-void SetNetworkGameAttributes(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal attribs)
+void SetNetworkGameAttributes(ScriptInterface::CxPrivate* pCxPrivate, CScriptVal attribs1)
 {
 	ENSURE(g_NetServer);
+	// TODO: Get Handle parameter directly with SpiderMonkey 31
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	JS::RootedValue attribs(cx, attribs1.get());
 
-	g_NetServer->UpdateGameAttributes(attribs, *(pCxPrivate->pScriptInterface));
+	g_NetServer->UpdateGameAttributes(&attribs, *(pCxPrivate->pScriptInterface));
 }
 
 void StartNetworkHost(ScriptInterface::CxPrivate* pCxPrivate, std::wstring playerName)
@@ -329,12 +390,13 @@ void DisconnectNetworkGame(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 CScriptVal PollNetworkClient(ScriptInterface::CxPrivate* pCxPrivate)
 {
 	if (!g_NetClient)
-		return CScriptVal();
-
-	CScriptValRooted poll = g_NetClient->GuiPoll();
+		return JS::UndefinedValue();
 
 	// Convert from net client context to GUI script context
-	return pCxPrivate->pScriptInterface->CloneValueFromOtherContext(g_NetClient->GetScriptInterface(), poll.get());
+	JSContext* cxNet = g_NetClient->GetScriptInterface().GetContext();
+	JSAutoRequest rqNet(cxNet);
+	JS::RootedValue pollNet(cxNet, g_NetClient->GuiPoll().get());
+	return pCxPrivate->pScriptInterface->CloneValueFromOtherContext(g_NetClient->GetScriptInterface(), pollNet);
 }
 
 void AssignNetworkPlayer(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), int playerID, std::string guid)
@@ -425,11 +487,15 @@ CScriptVal LoadMapSettings(ScriptInterface::CxPrivate* pCxPrivate, VfsPath pathn
 CScriptVal GetMapSettings(ScriptInterface::CxPrivate* pCxPrivate)
 {
 	if (!g_Game)
-		return CScriptVal();
+		return JS::UndefinedValue();
 
+	JSContext* cx = g_Game->GetSimulation2()->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+	
+	JS::RootedValue mapSettings(cx, g_Game->GetSimulation2()->GetMapSettings().get());
 	return pCxPrivate->pScriptInterface->CloneValueFromOtherContext(
 		g_Game->GetSimulation2()->GetScriptInterface(),
-		g_Game->GetSimulation2()->GetMapSettings().get());
+		mapSettings);
 }
 
 /**
@@ -697,11 +763,11 @@ int GetFps(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
 
 CScriptVal GetGUIObjectByName(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), CStr name)
 {
-		IGUIObject* guiObj = g_GUI->FindObjectByName(name);
-		if (guiObj)
-			return OBJECT_TO_JSVAL(guiObj->GetJSObject());
-		else
-			return JSVAL_VOID;
+	IGUIObject* guiObj = g_GUI->FindObjectByName(name);
+	if (guiObj)
+		return OBJECT_TO_JSVAL(guiObj->GetJSObject());
+	else
+		return JSVAL_VOID;
 }
 
 // Return the date/time at which the current executable was compiled.
@@ -844,6 +910,7 @@ void GuiScriptingInit(ScriptInterface& scriptInterface)
 	JSI_Renderer::RegisterScriptFunctions(scriptInterface);
 	JSI_Console::RegisterScriptFunctions(scriptInterface);
 	JSI_ConfigDB::RegisterScriptFunctions(scriptInterface);
+	JSI_Mod::RegisterScriptFunctions(scriptInterface);
 	JSI_Sound::RegisterScriptFunctions(scriptInterface);
 	JSI_L10n::RegisterScriptFunctions(scriptInterface);
  
@@ -960,6 +1027,7 @@ void GuiScriptingInit(ScriptInterface& scriptInterface)
 	scriptInterface.RegisterFunction<void, &JSI_Lobby::SendGetGameList>("SendGetGameList");
 	scriptInterface.RegisterFunction<void, &JSI_Lobby::SendGetBoardList>("SendGetBoardList");
 	scriptInterface.RegisterFunction<void, &JSI_Lobby::SendGetRatingList>("SendGetRatingList");
+	scriptInterface.RegisterFunction<void, std::wstring, &JSI_Lobby::SendGetProfile>("SendGetProfile");
 	scriptInterface.RegisterFunction<void, CScriptVal, &JSI_Lobby::SendRegisterGame>("SendRegisterGame");
 	scriptInterface.RegisterFunction<void, CScriptVal, &JSI_Lobby::SendGameReport>("SendGameReport");
 	scriptInterface.RegisterFunction<void, &JSI_Lobby::SendUnregisterGame>("SendUnregisterGame");
@@ -967,6 +1035,7 @@ void GuiScriptingInit(ScriptInterface& scriptInterface)
 	scriptInterface.RegisterFunction<CScriptVal, &JSI_Lobby::GetPlayerList>("GetPlayerList");
 	scriptInterface.RegisterFunction<CScriptVal, &JSI_Lobby::GetGameList>("GetGameList");
 	scriptInterface.RegisterFunction<CScriptVal, &JSI_Lobby::GetBoardList>("GetBoardList");
+	scriptInterface.RegisterFunction<CScriptVal, &JSI_Lobby::GetProfile>("GetProfile");
 	scriptInterface.RegisterFunction<CScriptVal, &JSI_Lobby::LobbyGuiPollMessage>("LobbyGuiPollMessage");
 	scriptInterface.RegisterFunction<void, std::wstring, &JSI_Lobby::LobbySendMessage>("LobbySendMessage");
 	scriptInterface.RegisterFunction<void, std::wstring, &JSI_Lobby::LobbySetPlayerPresence>("LobbySetPlayerPresence");
