@@ -1,4 +1,4 @@
-/* Copyright (c) 2012 Wildfire Games
+/* Copyright (c) 2014 Wildfire Games
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -27,14 +27,15 @@
 #include "precompiled.h"
 #include "cursor.h"
 
-#include <cstring>
 #include <cstdio>
+#include <cstring>
 #include <sstream>
 
+#include "lib/external_libraries/libsdl.h"
 #include "lib/ogl.h"
+#include "lib/res/h_mgr.h"
 #include "lib/sysdep/cursor.h"
 #include "ogl_tex.h"
-#include "lib/res/h_mgr.h"
 
 // On Windows, allow runtime choice between system cursors and OpenGL
 // cursors (Windows = more responsive, OpenGL = more consistent with what
@@ -45,10 +46,10 @@
 # define ALLOW_SYS_CURSOR 0
 #endif
 
-
+#if !SDL_VERSION_ATLEAST(2,0,0)
 static Status load_sys_cursor(const PIVFS& vfs, const VfsPath& pathname, int hx, int hy, sys_cursor* cursor)
 {
-#if !ALLOW_SYS_CURSOR
+# if !ALLOW_SYS_CURSOR
 	UNUSED2(vfs);
 	UNUSED2(pathname);
 	UNUSED2(hx);
@@ -56,7 +57,7 @@ static Status load_sys_cursor(const PIVFS& vfs, const VfsPath& pathname, int hx,
 	UNUSED2(cursor);
 
 	return ERR::FAIL;
-#else
+# else
 	shared_ptr<u8> file; size_t fileSize;
 	RETURN_STATUS_IF_ERR(vfs->LoadFile(pathname, file, fileSize));
 
@@ -72,9 +73,54 @@ static Status load_sys_cursor(const PIVFS& vfs, const VfsPath& pathname, int hx,
 
 	RETURN_STATUS_IF_ERR(sys_cursor_create((int)t.m_Width, (int)t.m_Height, bgra_img, hx, hy, cursor));
 	return INFO::OK;
-#endif
+# endif // ALLOW_SYS_CURSOR
 }
 
+#else // SDL_VERSION_ATLEAST(2,0,0)
+
+class SDLCursor
+{
+	SDL_Surface* surface;
+	SDL_Cursor* cursor;
+
+public:
+	Status create(const PIVFS& vfs, const VfsPath& pathname, int hotspotx_, int hotspoty_)
+	{
+		shared_ptr<u8> file; size_t fileSize;
+		RETURN_STATUS_IF_ERR(vfs->LoadFile(pathname, file, fileSize));
+
+		Tex t;
+		RETURN_STATUS_IF_ERR(t.decode(file, fileSize));
+
+		// convert to required BGRA format.
+		const size_t flags = (t.m_Flags | TEX_BGR) & ~TEX_DXT;
+		RETURN_STATUS_IF_ERR(t.transform_to(flags));
+		void* bgra_img = t.get_data();
+		if(!bgra_img)
+			WARN_RETURN(ERR::FAIL);
+
+		surface = SDL_CreateRGBSurfaceFrom(bgra_img, (int)t.m_Width, (int)t.m_Height, 32, (int)t.m_Width*4, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
+		if (!surface)
+			return ERR::FAIL;
+		cursor = SDL_CreateColorCursor(surface, hotspotx_, hotspoty_);
+		if (!cursor)
+			return ERR::FAIL;
+
+		return INFO::OK;
+	}
+
+	void set()
+	{
+		SDL_SetCursor(cursor);
+	}
+
+	void destroy()
+	{
+		SDL_FreeCursor(cursor);
+		SDL_FreeSurface(surface);
+	}
+};
+#endif // SDL_VERSION_ATLEAST(2,0,0)
 
 // no init is necessary because this is stored in struct Cursor, which
 // is 0-initialized by h_mgr.
@@ -150,7 +196,11 @@ public:
 enum CursorKind
 {
 	CK_Default,
+#if !SDL_VERSION_ATLEAST(2,0,0)
 	CK_System,
+#else
+	CK_SDL,
+#endif
 	CK_OpenGL
 };
 
@@ -161,12 +211,19 @@ struct Cursor
 
 	CursorKind kind;
 
+#if !SDL_VERSION_ATLEAST(2,0,0)
 	// valid iff kind == CK_System
 	sys_cursor system_cursor;
+#else
+	// valid iff kind == CK_SDL
+	SDLCursor sdl_cursor;
+#endif
 
 	// valid iff kind == CK_OpenGL
 	GLCursor gl_cursor;
+#if !SDL_VERSION_ATLEAST(2,0,0)
 	sys_cursor gl_empty_system_cursor;
+#endif
 };
 
 H_TYPE_DEFINE(Cursor);
@@ -183,13 +240,21 @@ static void Cursor_dtor(Cursor* c)
 	case CK_Default:
 		break;	// nothing to do
 
+#if !SDL_VERSION_ATLEAST(2,0,0)
 	case CK_System:
 		sys_cursor_free(c->system_cursor);
 		break;
+#else
+	case CK_SDL:
+		c->sdl_cursor.destroy();
+		break;
+#endif
 
 	case CK_OpenGL:
 		c->gl_cursor.destroy();
+#if !SDL_VERSION_ATLEAST(2, 0, 0)
 		sys_cursor_free(c->gl_empty_system_cursor);
+#endif
 		break;
 
 	default:
@@ -215,15 +280,23 @@ static Status Cursor_reload(Cursor* c, const PIVFS& vfs, const VfsPath& name, Ha
 
 	const VfsPath pathnameImage = pathname.ChangeExtension(L".png");
 
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+	// try loading as SDL2 cursor
+	if (!c->forceGL && c->sdl_cursor.create(vfs, pathnameImage, hotspotx, hotspoty) == INFO::OK)
+		c->kind = CK_SDL;
+#else
 	// try loading as system cursor (2d, hardware accelerated)
-	if(!c->forceGL && load_sys_cursor(vfs, pathnameImage, hotspotx, hotspoty, &c->system_cursor) == INFO::OK)
+	if (!c->forceGL && load_sys_cursor(vfs, pathnameImage, hotspotx, hotspoty, &c->system_cursor) == INFO::OK)
 		c->kind = CK_System;
+#endif
 	// fall back to GLCursor (system cursor code is disabled or failed)
 	else if(c->gl_cursor.create(vfs, pathnameImage, hotspotx, hotspoty) == INFO::OK)
 	{
 		c->kind = CK_OpenGL;
+#if !SDL_VERSION_ATLEAST(2, 0, 0)
 		// (we need to hide the system cursor when using a OpenGL cursor)
 		sys_cursor_create_empty(&c->gl_empty_system_cursor);
+#endif
 	}
 	// everything failed, leave cursor unchanged
 	else
@@ -239,10 +312,15 @@ static Status Cursor_validate(const Cursor* c)
 	case CK_Default:
 		break;	// nothing to do
 
+#if !SDL_VERSION_ATLEAST(2,0,0)
 	case CK_System:
 		if(c->system_cursor == 0)
 			WARN_RETURN(ERR::_1);
 		break;
+#else
+	case CK_SDL:
+		break;	// nothing to do
+#endif
 
 	case CK_OpenGL:
 		RETURN_STATUS_IF_ERR(c->gl_cursor.validate());
@@ -265,9 +343,15 @@ static Status Cursor_to_string(const Cursor* c, wchar_t* buf)
 		type = L"default";
 		break;
 
+#if !SDL_VERSION_ATLEAST(2,0,0)
 	case CK_System:
 		type = L"sys";
 		break;
+#else
+	case CK_SDL:
+		type = L"sdl";
+		break;
+#endif
 
 	case CK_OpenGL:
 		type = L"gl";
@@ -311,7 +395,11 @@ Status cursor_draw(const PIVFS& vfs, const wchar_t* name, int x, int y, bool for
 	// hide the cursor
 	if(!name)
 	{
+#if !SDL_VERSION_ATLEAST(2, 0, 0)
 		sys_cursor_set(0);
+#else
+		SDL_ShowCursor(SDL_DISABLE);
+#endif
 		return INFO::OK;
 	}
 
@@ -328,18 +416,29 @@ Status cursor_draw(const PIVFS& vfs, const wchar_t* name, int x, int y, bool for
 	case CK_Default:
 		break;
 
+#if !SDL_VERSION_ATLEAST(2,0,0)
 	case CK_System:
 		sys_cursor_set(c->system_cursor);
 		break;
+#else
+	case CK_SDL:
+		c->sdl_cursor.set();
+		SDL_ShowCursor(SDL_ENABLE);
+		break;
+#endif
 
 	case CK_OpenGL:
 		c->gl_cursor.draw(x, y);
+#if !SDL_VERSION_ATLEAST(2, 0, 0)
 		// note: gl_empty_system_cursor can be 0 if sys_cursor_create_empty
 		// failed; in that case we don't want to sys_cursor_set because that
 		// would restore the default cursor (which is exactly what we're
 		// trying to avoid here)
 		if(c->gl_empty_system_cursor)
 			sys_cursor_set(c->gl_empty_system_cursor);
+#else
+		SDL_ShowCursor(SDL_DISABLE);
+#endif
 		break;
 
 	default:
