@@ -657,7 +657,7 @@ u32 Tex::get_average_colour() const
 	Tex basetex = *this;
 	uint8_t *data = new uint8_t[last_level_size];
 	memcpy(data, m_Data.get() + m_Ofs + size - last_level_size, last_level_size);
-	boost::shared_ptr<uint8_t> sdata(data);
+	boost::shared_ptr<uint8_t> sdata(data, ArrayDeleter());
 	basetex.wrap(1, 1, m_Bpp, m_Flags, sdata, 0);
 
 	// convert to BGRA
