@@ -102,19 +102,22 @@ XmppClient::XmppClient(const std::string& sUsername, const std::string& sPasswor
 	const int mechs = gloox::SaslMechAll ^ gloox::SaslMechPlain;
 	m_client->setSASLMechanisms(mechs);
 
-	m_client->registerConnectionListener( this );
+	m_client->registerConnectionListener(this);
 	m_client->setPresence(gloox::Presence::Available, -1);
-	m_client->disco()->setVersion( "Pyrogenesis", "0.0.15" );
-	m_client->disco()->setIdentity( "client", "bot" );
+	m_client->disco()->setVersion("Pyrogenesis", "0.0.17");
+	m_client->disco()->setIdentity("client", "bot");
 	m_client->setCompression(false);
 
-	m_client->registerStanzaExtension( new GameListQuery() );
-	m_client->registerIqHandler( this, ExtGameListQuery);
+	m_client->registerStanzaExtension(new GameListQuery());
+	m_client->registerIqHandler(this, ExtGameListQuery);
 
-	m_client->registerStanzaExtension( new BoardListQuery() );
-	m_client->registerIqHandler( this, ExtBoardListQuery);
+	m_client->registerStanzaExtension(new BoardListQuery());
+	m_client->registerIqHandler(this, ExtBoardListQuery);
 
-	m_client->registerMessageHandler( this );
+	m_client->registerStanzaExtension(new ProfileQuery());
+	m_client->registerIqHandler(this, ExtProfileQuery);
+
+	m_client->registerMessageHandler(this);
 
 	// Uncomment to see the raw stanzas
 	//m_client->getWrapped()->logInstance().registerLogHandler( gloox::LogLevelDebug, gloox::LogAreaAll, this );
@@ -152,6 +155,8 @@ XmppClient::~XmppClient()
 		glooxwrapper::Tag::free(*it);
 	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_BoardList.begin(); it != m_BoardList.end(); ++it)
 		glooxwrapper::Tag::free(*it);
+	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_Profile.begin(); it != m_Profile.end(); ++it)
+		glooxwrapper::Tag::free(*it);
 }
 
 /// Network
@@ -212,9 +217,12 @@ void XmppClient::onDisconnect(gloox::ConnectionError error)
 		glooxwrapper::Tag::free(*it);
 	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_BoardList.begin(); it != m_BoardList.end(); ++it)
 		glooxwrapper::Tag::free(*it);
+	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_Profile.begin(); it != m_Profile.end(); ++it)
+		glooxwrapper::Tag::free(*it);
 	m_BoardList.clear();
 	m_GameList.clear();
 	m_PlayerMap.clear();
+	m_Profile.clear();
 
 	if(error == gloox::ConnAuthenticationFailed)
 		CreateSimpleMessage("system", "authentication failed", "error");
@@ -283,6 +291,22 @@ void XmppClient::SendIqGetBoardList()
 	m_client->send(iq);
 }
 
+/**
+ * Request the profile data from the server.
+ */
+void XmppClient::SendIqGetProfile(const std::string& player)
+{
+	glooxwrapper::JID xpartamuppJid(m_xpartamuppId);
+
+	// Send IQ
+	ProfileQuery* b = new ProfileQuery();
+	b->m_Command = player;
+	glooxwrapper::IQ iq(gloox::IQ::Get, xpartamuppJid);
+	iq.addExtension(b);
+	DbgXMPP("SendIqGetProfile [" << tag_xml(iq) << "]");
+	m_client->send(iq);
+}
+
 /**
  * Request the rating data from the server.
  */
@@ -479,22 +503,25 @@ void XmppClient::handleOOB(const glooxwrapper::JID&, const glooxwrapper::OOB&)
  */
 CScriptValRooted XmppClient::GUIGetPlayerList(ScriptInterface& scriptInterface)
 {
-	CScriptValRooted playerList;
-	scriptInterface.Eval("([])", playerList);
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+	
+	JS::RootedValue playerList(cx);
+	scriptInterface.Eval("([])", &playerList);
 
 	// Convert the internal data structure to a Javascript object.
 	for (std::map<std::string, std::vector<std::string> >::const_iterator it = m_PlayerMap.begin(); it != m_PlayerMap.end(); ++it)
 	{
-		CScriptValRooted player;
-		scriptInterface.Eval("({})", player);
-		scriptInterface.SetProperty(player.get(), "name", wstring_from_utf8(it->first));
-		scriptInterface.SetProperty(player.get(), "presence", wstring_from_utf8(it->second[0]));
-		scriptInterface.SetProperty(player.get(), "rating", wstring_from_utf8(it->second[1]));
-		scriptInterface.SetProperty(player.get(), "role", wstring_from_utf8(it->second[2]));
-		scriptInterface.CallFunctionVoid(playerList.get(), "push", player);
+		JS::RootedValue player(cx);
+		scriptInterface.Eval("({})", &player);
+		scriptInterface.SetProperty(player, "name", wstring_from_utf8(it->first));
+		scriptInterface.SetProperty(player, "presence", wstring_from_utf8(it->second[0]));
+		scriptInterface.SetProperty(player, "rating", wstring_from_utf8(it->second[1]));
+		scriptInterface.SetProperty(player, "role", wstring_from_utf8(it->second[2]));
+		scriptInterface.CallFunctionVoid(playerList, "push", player);
 	}
 
-	return playerList;
+	return CScriptValRooted(cx, playerList);
 }
 
 /**
@@ -504,22 +531,24 @@ CScriptValRooted XmppClient::GUIGetPlayerList(ScriptInterface& scriptInterface)
  */
 CScriptValRooted XmppClient::GUIGetGameList(ScriptInterface& scriptInterface)
 {
-	CScriptValRooted gameList;
-	scriptInterface.Eval("([])", gameList);
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+	
+	JS::RootedValue gameList(cx);
+	scriptInterface.Eval("([])", &gameList);
 	for(std::vector<const glooxwrapper::Tag*>::const_iterator it = m_GameList.begin(); it != m_GameList.end(); ++it)
 	{
-		CScriptValRooted game;
-		scriptInterface.Eval("({})", game);
+		JS::RootedValue game(cx);
+		scriptInterface.Eval("({})", &game);
 
 		const char* stats[] = { "name", "ip", "state", "nbp", "tnbp", "players", "mapName", "niceMapName", "mapSize", "mapType", "victoryCondition" };
-		short stats_length = 11;
-		for (short i = 0; i < stats_length; i++)
-			scriptInterface.SetProperty(game.get(), stats[i], wstring_from_utf8((*it)->findAttribute(stats[i]).to_string()));
+		for (size_t i = 0; i < ARRAY_SIZE(stats); ++i)
+			scriptInterface.SetProperty(game, stats[i], wstring_from_utf8((*it)->findAttribute(stats[i]).to_string()));
 
-		scriptInterface.CallFunctionVoid(gameList.get(), "push", game);
+		scriptInterface.CallFunctionVoid(gameList, "push", game);
 	}
 
-	return gameList;
+	return CScriptValRooted(cx, gameList);;
 }
 
 /**
@@ -529,22 +558,51 @@ CScriptValRooted XmppClient::GUIGetGameList(ScriptInterface& scriptInterface)
  */
 CScriptValRooted XmppClient::GUIGetBoardList(ScriptInterface& scriptInterface)
 {
-	CScriptValRooted boardList;
-	scriptInterface.Eval("([])", boardList);
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+
+	JS::RootedValue boardList(cx);
+	scriptInterface.Eval("([])", &boardList);
 	for(std::vector<const glooxwrapper::Tag*>::const_iterator it = m_BoardList.begin(); it != m_BoardList.end(); ++it)
 	{
-		CScriptValRooted board;
-		scriptInterface.Eval("({})", board);
+		JS::RootedValue board(cx);
+		scriptInterface.Eval("({})", &board);
 
 		const char* attributes[] = { "name", "rank", "rating" };
-		short attributes_length = 3;
-		for (short i = 0; i < attributes_length; i++)
-			scriptInterface.SetProperty(board.get(), attributes[i], wstring_from_utf8((*it)->findAttribute(attributes[i]).to_string()));
+		for (size_t i = 0; i < ARRAY_SIZE(attributes); ++i)
+			scriptInterface.SetProperty(board, attributes[i], wstring_from_utf8((*it)->findAttribute(attributes[i]).to_string()));
 
-		scriptInterface.CallFunctionVoid(boardList.get(), "push", board);
+		scriptInterface.CallFunctionVoid(boardList, "push", board);
 	}
 
-	return boardList;
+	return CScriptValRooted(cx, boardList);
+}
+
+/**
+ * Handle requests from the GUI for profile data.
+ *
+ * @return A JS array containing the specific user's profile data
+ */
+CScriptValRooted XmppClient::GUIGetProfile(ScriptInterface& scriptInterface)
+{
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+
+	JS::RootedValue profileFetch(cx);
+	scriptInterface.Eval("([])", &profileFetch);
+	const char* stats[] = { "player", "rating", "totalGamesPlayed", "highestRating", "wins", "losses", "rank" };
+	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_Profile.begin(); it != m_Profile.end(); ++it)
+	{
+		JS::RootedValue profile(cx);
+		scriptInterface.Eval("({})", &profile);
+
+		for (size_t i = 0; i < ARRAY_SIZE(stats); ++i)
+			scriptInterface.SetProperty(profile, stats[i], wstring_from_utf8((*it)->findAttribute(stats[i]).to_string()));
+
+		scriptInterface.CallFunctionVoid(profileFetch, "push", profile);
+	}
+
+	return CScriptValRooted(cx, profileFetch);
 }
 
 /*****************************************************
@@ -560,23 +618,26 @@ CScriptValRooted XmppClient::GuiPollMessage(ScriptInterface& scriptInterface)
 		return CScriptValRooted();
 
 	GUIMessage message = m_GuiMessageQueue.front();
-	CScriptValRooted messageVal;
-
-	scriptInterface.Eval("({})", messageVal);
-	scriptInterface.SetProperty(messageVal.get(), "type", message.type);
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+	
+	JS::RootedValue messageVal(cx);
+ 
+	scriptInterface.Eval("({})", &messageVal);
+	scriptInterface.SetProperty(messageVal, "type", message.type);
 	if (!message.from.empty())
-		scriptInterface.SetProperty(messageVal.get(), "from", message.from);
+		scriptInterface.SetProperty(messageVal, "from", message.from);
 	if (!message.text.empty())
-		scriptInterface.SetProperty(messageVal.get(), "text", message.text);
+		scriptInterface.SetProperty(messageVal, "text", message.text);
 	if (!message.level.empty())
-		scriptInterface.SetProperty(messageVal.get(), "level", message.level);
+		scriptInterface.SetProperty(messageVal, "level", message.level);
 	if (!message.message.empty())
-		scriptInterface.SetProperty(messageVal.get(), "message", message.message);
+		scriptInterface.SetProperty(messageVal, "message", message.message);
 	if (!message.data.empty())
-		scriptInterface.SetProperty(messageVal.get(), "data", message.data);
+		scriptInterface.SetProperty(messageVal, "data", message.data);
 	
 	m_GuiMessageQueue.pop_front();
-	return messageVal;
+	return CScriptValRooted(cx, messageVal);
 }
 
 /**
@@ -636,6 +697,7 @@ bool XmppClient::handleIq(const glooxwrapper::IQ& iq)
 	{
 		const GameListQuery* gq = iq.findExtension<GameListQuery>( ExtGameListQuery );
 		const BoardListQuery* bq = iq.findExtension<BoardListQuery>( ExtBoardListQuery );
+		const ProfileQuery* pq = iq.findExtension<ProfileQuery>( ExtProfileQuery );
 		if(gq)
 		{
 			for(std::vector<const glooxwrapper::Tag*>::const_iterator it = m_GameList.begin(); it != m_GameList.end(); ++it )
@@ -672,8 +734,19 @@ bool XmppClient::handleIq(const glooxwrapper::IQ& iq)
 				CreateSimpleMessage("system", "ratinglist updated", "internal");
 			}
 		}
+		if (pq)
+		{
+			for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_Profile.begin(); it != m_Profile.end(); ++it)
+				glooxwrapper::Tag::free(*it);
+			m_Profile.clear();
+
+			for (std::vector<const glooxwrapper::Tag*>::const_iterator it = pq->m_StanzaProfile.begin(); it != pq->m_StanzaProfile.end(); ++it)
+				m_Profile.push_back((*it)->clone());
+
+			CreateSimpleMessage("system", "profile updated", "internal");
+		}
 	}
-	else if(iq.subtype() == gloox::IQ::Error)
+	else if (iq.subtype() == gloox::IQ::Error)
 	{
 		gloox::StanzaError err = iq.error_error();
 		std::string msg = StanzaErrorToString(err);
