@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -32,7 +32,6 @@
 #include "ps/ConfigDB.h"
 
 #if CONFIG2_MINIUPNPC
-// Next four files are for UPnP port forwarding.
 #include <miniupnpc/miniwget.h>
 #include <miniupnpc/miniupnpc.h>
 #include <miniupnpc/upnpcommands.h>
@@ -218,7 +217,7 @@ void* CNetServerWorker::SetupUPnP(void*)
 	// Intermediate variables.
 	struct UPNPUrls urls;
 	struct IGDdatas data;
-	struct UPNPDev* devlist = 0;
+	struct UPNPDev* devlist = NULL;
 
 	// Cached root descriptor URL.
 	std::string rootDescURL;
@@ -226,12 +225,22 @@ void* CNetServerWorker::SetupUPnP(void*)
 	if (!rootDescURL.empty())
 		LOGMESSAGE(L"Net server: attempting to use cached root descriptor URL: %hs", rootDescURL.c_str());
 
-	// Init the return variable for UPNP_GetValidIGD to 1 so things behave when using cached URLs.
-	int ret = 1;
+	int ret = 0;
+	bool allocatedUrls = false;
 
-	// If we have a cached URL, try that first, otherwise try getting a valid UPnP device for 10 seconds. We also get our LAN address here.
-	if (!((!rootDescURL.empty() && UPNP_GetIGDFromUrl(rootDescURL.c_str(), &urls, &data, internalIPAddress, sizeof(internalIPAddress)))
-	  || ((devlist = upnpDiscover(10000, 0, 0, 0, 0, 0)) != NULL && (ret = UPNP_GetValidIGD(devlist, &urls, &data, internalIPAddress, sizeof(internalIPAddress))) != 0)))
+	// Try a cached URL first
+	if (!rootDescURL.empty() && UPNP_GetIGDFromUrl(rootDescURL.c_str(), &urls, &data, internalIPAddress, sizeof(internalIPAddress)))
+	{
+		LOGMESSAGE(L"Net server: using cached IGD = %hs", urls.controlURL);
+		ret = 1;
+	}
+	// No cached URL, or it did not respond. Try getting a valid UPnP device for 10 seconds.
+	else if ((devlist = upnpDiscover(10000, 0, 0, 0, 0, 0)) != NULL)
+	{
+		ret = UPNP_GetValidIGD(devlist, &urls, &data, internalIPAddress, sizeof(internalIPAddress));
+		allocatedUrls = ret != 0; // urls is allocated on non-zero return values
+	}
+	else
 	{
 		LOGMESSAGE(L"Net server: upnpDiscover failed and no working cached URL.");
 		return NULL;
@@ -239,6 +248,9 @@ void* CNetServerWorker::SetupUPnP(void*)
 
 	switch (ret)
 	{
+	case 0:
+		LOGMESSAGE(L"Net server: No IGD found");
+		break;
 	case 1:
 		LOGMESSAGE(L"Net server: found valid IGD = %hs", urls.controlURL);
 		break;
@@ -275,6 +287,9 @@ void* CNetServerWorker::SetupUPnP(void*)
 	ret = UPNP_GetSpecificPortMappingEntry(urls.controlURL,
 									 data.first.servicetype,
 									 psPort, protocall,
+#if MINIUPNPC_API_VERSION >= 10
+									 NULL/*remoteHost*/,
+#endif
 									 intClient, intPort, NULL/*desc*/,
 									 NULL/*enabled*/, duration);
 
@@ -293,7 +308,9 @@ void* CNetServerWorker::SetupUPnP(void*)
 	LOGMESSAGE(L"Net server: cached UPnP root descriptor URL as %hs", urls.controlURL);
 
 	// Make sure everything is properly freed.
-	FreeUPNPUrls(&urls);
+	if (allocatedUrls)
+		FreeUPNPUrls(&urls);
+    
 	freeUPNPDevlist(devlist);
 
 	return NULL;
@@ -373,6 +390,11 @@ bool CNetServerWorker::RunStep()
 	// Check for messages from the game thread.
 	// (Do as little work as possible while the mutex is held open,
 	// to avoid performance problems and deadlocks.)
+	
+	m_ScriptInterface->MaybeIncrementalRuntimeGC(0.5f);
+	
+	JSContext* cx = m_ScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
 
 	std::vector<std::pair<int, CStr> > newAssignPlayer;
 	std::vector<bool> newStartGame;
@@ -405,7 +427,11 @@ bool CNetServerWorker::RunStep()
 		ClearAllPlayerReady();
 
 	if (!newGameAttributes.empty())
-		UpdateGameAttributes(GetScriptInterface().ParseJSON(newGameAttributes.back()));
+	{
+		JS::RootedValue gameAttributesVal(cx);
+		GetScriptInterface().ParseJSON(newGameAttributes.back(), &gameAttributesVal);
+		UpdateGameAttributes(&gameAttributesVal);
+	}
 
 	if (!newTurnLength.empty())
 		SetTurnLength(newTurnLength.back());
@@ -561,6 +587,7 @@ void CNetServerWorker::SetupSession(CNetServerSession* session)
 	session->AddTransition(NSS_PREGAME, (uint)NMT_READY, NSS_PREGAME, (void*)&OnReady, context);
 	session->AddTransition(NSS_PREGAME, (uint)NMT_LOADED_GAME, NSS_INGAME, (void*)&OnLoadedGame, context);
 
+	session->AddTransition(NSS_JOIN_SYNCING, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED, (void*)&OnDisconnect, context);
 	session->AddTransition(NSS_JOIN_SYNCING, (uint)NMT_LOADED_GAME, NSS_INGAME, (void*)&OnJoinSyncingLoadedGame, context);
 
 	session->AddTransition(NSS_INGAME, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED, (void*)&OnDisconnect, context);
@@ -599,7 +626,7 @@ void CNetServerWorker::OnUserLeave(CNetServerSession* session)
 {
 	RemovePlayer(session->GetGUID());
 
-	if (m_ServerTurnManager)
+	if (m_ServerTurnManager && session->GetCurrState() != NSS_JOIN_SYNCING)
 		m_ServerTurnManager->UninitialiseClient(session->GetHostID()); // TODO: only for non-observers
 
 	// TODO: ought to switch the player controlled by that client
@@ -618,7 +645,6 @@ void CNetServerWorker::AddPlayer(const CStr& guid, const CStrW& name)
 	// back their old player ID
 
 	i32 playerID = -1;
-	bool foundPlayerID = false;
 
 	// Try to match GUID first
 	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
@@ -626,36 +652,25 @@ void CNetServerWorker::AddPlayer(const CStr& guid, const CStrW& name)
 		if (!it->second.m_Enabled && it->first == guid && usedIDs.find(it->second.m_PlayerID) == usedIDs.end())
 		{
 			playerID = it->second.m_PlayerID;
-			foundPlayerID = true;
 			m_PlayerAssignments.erase(it); // delete the old mapping, since we've got a new one now
-			break;
+			goto found;
 		}
 	}
 
 	// Try to match username next
-	if (!foundPlayerID)
+	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
 	{
-		for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
+		if (!it->second.m_Enabled && it->second.m_Name == name && usedIDs.find(it->second.m_PlayerID) == usedIDs.end())
 		{
-			if (!it->second.m_Enabled && it->second.m_Name == name && usedIDs.find(it->second.m_PlayerID) == usedIDs.end())
-			{
-				playerID = it->second.m_PlayerID;
-				foundPlayerID = true;
-				m_PlayerAssignments.erase(it); // delete the old mapping, since we've got a new one now
-				break;
-			}
+			playerID = it->second.m_PlayerID;
+			m_PlayerAssignments.erase(it); // delete the old mapping, since we've got a new one now
+			goto found;
 		}
 	}
 
-	// Otherwise pick the first free player ID
-	if (!foundPlayerID)
-	{
-		for (playerID = 1; usedIDs.find(playerID) != usedIDs.end(); ++playerID)
-		{
-			// (do nothing)
-		}
-	}
+	// Otherwise leave the player ID as -1 (observer) and let gamesetup change it as needed.
 
+found:
 	PlayerAssignment assignment;
 	assignment.m_Enabled = true;
 	assignment.m_Name = name;
@@ -1006,17 +1021,21 @@ void CNetServerWorker::StartGame()
 
 	m_State = SERVER_STATE_LOADING;
 
+	JSContext* cx = GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+	// TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+	JS::RootedValue tmpGameAttributes(cx, m_GameAttributes.get());
 	// Send the final setup state to all clients
-	UpdateGameAttributes(m_GameAttributes);
+	UpdateGameAttributes(&tmpGameAttributes);
 	SendPlayerAssignments();
 
 	CGameStartMessage gameStart;
 	Broadcast(&gameStart);
 }
 
-void CNetServerWorker::UpdateGameAttributes(const CScriptValRooted& attrs)
+void CNetServerWorker::UpdateGameAttributes(JS::MutableHandleValue attrs)
 {
-	m_GameAttributes = attrs;
+	m_GameAttributes = CScriptValRooted(GetScriptInterface().GetContext(), attrs);
 
 	if (!m_Host)
 		return;
@@ -1115,11 +1134,11 @@ void CNetServer::StartGame()
 	m_Worker->m_StartGameQueue.push_back(true);
 }
 
-void CNetServer::UpdateGameAttributes(const CScriptVal& attrs, ScriptInterface& scriptInterface)
+void CNetServer::UpdateGameAttributes(JS::MutableHandleValue attrs, ScriptInterface& scriptInterface)
 {
 	// Pass the attributes as JSON, since that's the easiest safe
 	// cross-thread way of passing script data
-	std::string attrsJSON = scriptInterface.StringifyJSON(attrs.get(), false);
+	std::string attrsJSON = scriptInterface.StringifyJSON(attrs, false);
 
 	CScopeLock lock(m_Worker->m_WorkerMutex);
 	m_Worker->m_GameAttributesQueue.push_back(attrsJSON);
