@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 #include "lib/ogl.h"
 #include "lib/sysdep/clipboard.h"
 #include "lib/timer.h"
+#include "lib/utf8.h"
 #include "maths/MathUtil.h"
 #include "network/NetClient.h"
 #include "network/NetServer.h"
@@ -91,6 +92,14 @@ void CConsole::ToggleVisible()
 {
 	m_bToggle = true;
 	m_bVisible = !m_bVisible;
+
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+	// TODO: this should be based on input focus, not visibility
+	if (m_bVisible)
+		SDL_StartTextInput();
+	else
+		SDL_StopTextInput();
+#endif
 }
 
 void CConsole::SetVisible(bool visible)
@@ -206,11 +215,11 @@ void CConsole::Render()
 	const float DeltaY = (1.0f - m_fVisibleFrac) * m_fHeight;
 	transform.PostTranslate(m_fX, m_fY - DeltaY, 0.0f); // move to window position
 	solidShader->Uniform(str_transform, transform);
-	
+
 	DrawWindow(solidShader);
 
 	solidTech->EndPass();
-	
+
 	CShaderTechniquePtr textTech = g_Renderer.GetShaderManager().LoadEffect(str_gui_text);
 	textTech->BeginPass();
 	CTextRenderer textRenderer(textTech->GetShader());
@@ -593,6 +602,10 @@ void CConsole::UseHistoryFile(const VfsPath& filename, int max_history_lines)
 
 void CConsole::ProcessBuffer(const wchar_t* szLine)
 {
+	shared_ptr<ScriptInterface> pScriptInterface = g_GUI->GetActiveGUI()->GetScriptInterface();
+	JSContext* cx = pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
+	
 	if (szLine == NULL) return;
 	if (wcslen(szLine) <= 0) return;
 
@@ -604,10 +617,10 @@ void CConsole::ProcessBuffer(const wchar_t* szLine)
 
 	// Process it as JavaScript
 	
-	CScriptVal rval;
-	g_GUI->GetActiveGUI()->GetScriptInterface()->Eval(szLine, rval);
-	if (!rval.undefined())
-		InsertMessage(L"%ls", g_GUI->GetActiveGUI()->GetScriptInterface()->ToString(rval.get()).c_str());
+	JS::RootedValue rval(cx);
+	pScriptInterface->Eval(szLine, &rval);
+	if (!rval.isUndefined())
+		InsertMessageRaw(pScriptInterface->ToString(&rval));
 }
 
 void CConsole::LoadHistory()
@@ -669,9 +682,11 @@ static bool isUnprintableChar(SDL_Keysym key)
 static bool isUnprintableChar(SDL_keysym key)
 #endif
 {
+#if !SDL_VERSION_ATLEAST(2, 0, 0)
 	// U+0000 to U+001F are control characters
 	if (key.unicode < 0x20)
 	{
+#endif
 		switch (key.sym)
 		{
 			// We want to allow some, which are handled specially
@@ -687,9 +702,10 @@ static bool isUnprintableChar(SDL_keysym key)
 		default:
 			return true;
 		}
+#if !SDL_VERSION_ATLEAST(2, 0, 0)
 	}
-
 	return false;
+#endif
 }
 
 InReaction conInputHandler(const SDL_Event_* ev)
@@ -725,6 +741,20 @@ InReaction conInputHandler(const SDL_Event_* ev)
 	if (!g_Console->IsActive())
 		return IN_PASS;
 
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+	// In SDL2, we no longer get Unicode wchars via SDL_Keysym
+	// we use text input events instead and they provide UTF-8 chars
+	if (ev->ev.type == SDL_TEXTINPUT && !HotkeyIsPressed("console.toggle"))
+	{
+		// TODO: this could be more efficient with an interface to insert UTF-8 strings directly
+		std::wstring wstr = wstring_from_utf8(ev->ev.text.text);
+		for (size_t i = 0; i < wstr.length(); ++i)
+			g_Console->InsertChar(0, wstr[i]);
+		return IN_HANDLED;
+	}
+	// TODO: text editing events for IME support
+#endif
+
 	if (ev->ev.type != SDL_KEYDOWN)
 		return IN_PASS;
 
@@ -735,7 +765,11 @@ InReaction conInputHandler(const SDL_Event_* ev)
 	if (!isUnprintableChar(ev->ev.key.keysym) &&
 		!HotkeyIsPressed("console.toggle"))
 	{
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+		g_Console->InsertChar(sym, 0);
+#else
 		g_Console->InsertChar(sym, (wchar_t)ev->ev.key.keysym.unicode);
+#endif
 		return IN_HANDLED;
 	}
 
