@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -172,6 +172,27 @@ void LoadHotkeys()
 	}
 }
 
+void UnloadHotkeys()
+{
+	g_HotkeyMap.clear();
+	g_HotkeyStatus.clear();
+}
+
+bool isNegated(const SKey& key)
+{
+	// Normal keycodes are below EXTRA_KEYS_BASE
+	if ((int)key.code < EXTRA_KEYS_BASE && g_keys[key.code] == key.negated)
+		return false;
+	// Mouse 'keycodes' are after the modifier keys
+	else if ((int)key.code > UNIFIED_LAST && g_mouse_buttons[key.code - UNIFIED_LAST] == key.negated)
+		return false;
+	// Modifier keycodes are between the normal keys and the mouse 'keys'
+	else if ((int)key.code < UNIFIED_LAST && (int)key.code > CUSTOM_SDL_KEYCODE && unified[key.code - UNIFIED_SHIFT] == key.negated)
+		return false;
+	else
+		return true;
+}
+
 InReaction HotkeyInputHandler( const SDL_Event_* ev )
 {
 	int keycode = 0;
@@ -186,17 +207,14 @@ InReaction HotkeyInputHandler( const SDL_Event_* ev )
 	case SDL_MOUSEBUTTONDOWN:
 	case SDL_MOUSEBUTTONUP:
 #if SDL_VERSION_ATLEAST(2, 0, 0)
-		if ((int)ev->ev.button.button <= SDL_BUTTON_RIGHT)
-#elif SDL_VERSION_ATLEAST(1, 2, 13)
-		if ((int)ev->ev.button.button <= SDL_BUTTON_X2)
-#else
-		if ((int)ev->ev.button.button <= SDL_BUTTON_WHEELDOWN)
+		// Mousewheel events are no longer buttons, but we want to maintain the order
+		// expected by g_mouse_buttons for compatibility
+		if (ev->ev.button.button >= SDL_BUTTON_X1)
+			keycode = MOUSE_BASE + (int)ev->ev.button.button + 2;
+		else
 #endif
-		{
-			keycode = CUSTOM_SDL_KEYCODE + (int)ev->ev.button.button;
-			break;
-		}
-		return IN_PASS;
+		keycode = MOUSE_BASE + (int)ev->ev.button.button;
+		break;
 
 #if SDL_VERSION_ATLEAST(2, 0, 0)
 	case SDL_MOUSEWHEEL:
@@ -210,17 +228,37 @@ InReaction HotkeyInputHandler( const SDL_Event_* ev )
 			keycode = MOUSE_WHEELDOWN;
 			break;
 		}
+		else if (ev->ev.wheel.x > 0)
+		{
+			keycode = MOUSE_X2;
+			break;
+		}
+		else if (ev->ev.wheel.x < 0)
+		{
+			keycode = MOUSE_X1;
+			break;
+		}
 		return IN_PASS;
 #endif
 
+	case SDL_HOTKEYDOWN:
+		g_HotkeyStatus[static_cast<const char*>(ev->ev.user.data1)] = true;
+		return IN_PASS;
+
+	case SDL_HOTKEYUP:
+		g_HotkeyStatus[static_cast<const char*>(ev->ev.user.data1)] = false;
+		return IN_PASS;
+
 	default:
 		return IN_PASS;
 	}
 
+#if !SDL_VERSION_ATLEAST(2, 0, 0)
 	// Rather ugly hack to make the '"' key work better on a MacBook Pro on Windows so it doesn't
 	// always close the console. (Maybe this would be better handled in wsdl or something?)
 	if (keycode == SDLK_BACKQUOTE && (ev->ev.key.keysym.unicode == '\'' || ev->ev.key.keysym.unicode == '"'))
 		keycode = ev->ev.key.keysym.unicode;
+#endif
 
 	// Somewhat hackish:
 	// Create phantom 'unified-modifier' events when left- or right- modifier keys are pressed
@@ -284,52 +322,31 @@ InReaction HotkeyInputHandler( const SDL_Event_* ev )
 #else
 	bool typeKeyDown = ( ev->ev.type == SDL_KEYDOWN ) || ( ev->ev.type == SDL_MOUSEBUTTONDOWN );
 #endif
-	
+
 	// -- KEYDOWN SECTION -- 
 
 	std::vector<const char*> closestMapNames;
 	size_t closestMapMatch = 0;
 
-	for( std::vector<SHotkeyMapping>::iterator it = g_HotkeyMap[keycode].begin(); it < g_HotkeyMap[keycode].end(); ++it )
+	for (std::vector<SHotkeyMapping>::iterator it = g_HotkeyMap[keycode].begin(); it < g_HotkeyMap[keycode].end(); ++it)
 	{
 		// If a key has been pressed, and this event triggers on its release, skip it.
 		// Similarly, if the key's been released and the event triggers on a keypress, skip it.
-		if( it->negated == typeKeyDown )
+		if (it->negated == typeKeyDown)
 			continue;
 
-		// Check to see if all auxiliary keys are down
-		
+		// Check for no unpermitted keys
 		bool accept = true;
+		for (std::vector<SKey>::iterator itKey = it->requires.begin(); itKey != it->requires.end() && accept; ++itKey)
+			accept = isNegated(*itKey);
 
-		for( std::vector<SKey>::iterator itKey = it->requires.begin(); itKey != it->requires.end(); ++itKey )
+		if (accept && !(consoleCapture && it->name != "console.toggle"))
 		{
-			bool rqdState = !itKey->negated;
-
-			if( (int)itKey->code < CUSTOM_SDL_KEYCODE )
-			{
-				if( g_keys[itKey->code] != rqdState ) accept = false;
-			}
-			else if( (int)itKey->code < UNIFIED_SHIFT )
-			{
-				if( g_mouse_buttons[itKey->code - CUSTOM_SDL_KEYCODE] != rqdState ) accept = false;
-			}
-			else if( (int)itKey->code < UNIFIED_LAST )
-			{
-				if( unified[itKey->code - UNIFIED_SHIFT] != rqdState ) accept = false;
-			}
-		}
-
-		if( accept && !( consoleCapture && it->name != "console.toggle" ) )
-		{
-			// Tentatively set status to un-pressed, since it may be overridden by
-			// a closer match. (The closest matches will be set to pressed later.)
-			g_HotkeyStatus[it->name] = false;
-
 			// Check if this is an equally precise or more precise match
-			if( it->requires.size() + 1 >= closestMapMatch )
+			if (it->requires.size() + 1 >= closestMapMatch)
 			{
 				// Check if more precise
-				if( it->requires.size() + 1 > closestMapMatch )
+				if (it->requires.size() + 1 > closestMapMatch)
 				{
 					// Throw away the old less-precise matches
 					closestMapNames.clear();
@@ -343,58 +360,38 @@ InReaction HotkeyInputHandler( const SDL_Event_* ev )
 
 	for (size_t i = 0; i < closestMapNames.size(); ++i)
 	{
-		g_HotkeyStatus[closestMapNames[i]] = true;
-
-		SDL_Event hotkeyNotification;
-		hotkeyNotification.type = SDL_HOTKEYDOWN;
-		hotkeyNotification.user.data1 = const_cast<char*>(closestMapNames[i]);
-		SDL_PushEvent(&hotkeyNotification);
+		SDL_Event_ hotkeyNotification;
+		hotkeyNotification.ev.type = SDL_HOTKEYDOWN;
+		hotkeyNotification.ev.user.data1 = const_cast<char*>(closestMapNames[i]);
+		in_push_priority_event(&hotkeyNotification);
 	}
 
 	// -- KEYUP SECTION --
 
-	for( std::vector<SHotkeyMapping>::iterator it = g_HotkeyMap[keycode].begin(); it < g_HotkeyMap[keycode].end(); ++it )
+	for (std::vector<SHotkeyMapping>::iterator it = g_HotkeyMap[keycode].begin(); it < g_HotkeyMap[keycode].end(); ++it)
 	{
 		// If it's a keydown event, won't cause HotKeyUps in anything that doesn't
 		// use this key negated => skip them
 		// If it's a keyup event, won't cause HotKeyUps in anything that does use
 		// this key negated => skip them too.
-		if( it->negated != typeKeyDown )
+		if (it->negated != typeKeyDown)
 			continue;
 
-		// Check to see if all auxiliary keys are down
-
+		// Check for no unpermitted keys
 		bool accept = true;
+		for (std::vector<SKey>::iterator itKey = it->requires.begin(); itKey != it->requires.end() && accept; ++itKey)
+			accept = isNegated(*itKey);
 
-		for( std::vector<SKey>::iterator itKey = it->requires.begin(); itKey != it->requires.end(); ++itKey )
+		if (accept)
 		{
-			bool rqdState = !itKey->negated;
-
-			if( (int)itKey->code < CUSTOM_SDL_KEYCODE )
-			{
-				if( g_keys[itKey->code] != rqdState ) accept = false;
-			}
-			else if( (int)itKey->code < UNIFIED_SHIFT )
-			{
-				if( g_mouse_buttons[itKey->code - CUSTOM_SDL_KEYCODE] != rqdState ) accept = false;
-			}
-			else if( (int)itKey->code < UNIFIED_LAST )
-			{
-				if( unified[itKey->code - UNIFIED_SHIFT] != rqdState ) accept = false;
-			}
-		}
-
-		if( accept )
-		{
-			g_HotkeyStatus[it->name] = false;
-			SDL_Event hotkeyNotification;
-			hotkeyNotification.type = SDL_HOTKEYUP;
-			hotkeyNotification.user.data1 = const_cast<char*>(it->name.c_str());
-			SDL_PushEvent( &hotkeyNotification );
+			SDL_Event_ hotkeyNotification;
+			hotkeyNotification.ev.type = SDL_HOTKEYUP;
+			hotkeyNotification.ev.user.data1 = const_cast<char*>(it->name.c_str());
+			in_push_priority_event(&hotkeyNotification);
 		}
 	}
 
-	return( IN_PASS );
+	return IN_PASS;
 }
 
 bool HotkeyIsPressed(const CStr& keyname)
