@@ -20,12 +20,13 @@
 
 #include "graphics/Camera.h"
 #include "graphics/RenderableObject.h"
-#include "graphics/ShaderProgram.h"
-#include "renderer/ShadowMap.h"
+#include "graphics/ShaderProgramPtr.h"
 #include "renderer/VertexArray.h"
 
 class CModelDecal;
+class CShaderDefines;
 class CSimulation2;
+class ShadowMap;
 
 class CDecalRData : public CRenderData
 {
