@@ -64,6 +64,7 @@ CPatchRData::CPatchRData(CPatch* patch, CSimulation2* simulation) :
 	m_VBBase(0), m_VBBaseIndices(0),
 	m_VBBlends(0), m_VBBlendIndices(0),
 	m_VBWater(0), m_VBWaterIndices(0),
+	m_VBWaterShore(0), m_VBWaterIndicesShore(0),
 	m_Simulation(simulation)
 {
 	ENSURE(patch);
@@ -82,6 +83,8 @@ CPatchRData::~CPatchRData()
 	if (m_VBBlendIndices) g_VBMan.Release(m_VBBlendIndices);
 	if (m_VBWater) g_VBMan.Release(m_VBWater);
 	if (m_VBWaterIndices) g_VBMan.Release(m_VBWaterIndices);
+	if (m_VBWaterShore) g_VBMan.Release(m_VBWaterShore);
+	if (m_VBWaterIndicesShore) g_VBMan.Release(m_VBWaterIndicesShore);
 }
 
 /**
@@ -1288,6 +1291,16 @@ void CPatchRData::BuildWater()
 		g_VBMan.Release(m_VBWaterIndices);
 		m_VBWaterIndices = 0;
 	}
+	if (m_VBWaterShore)
+	{
+		g_VBMan.Release(m_VBWaterShore);
+		m_VBWaterShore = 0;
+	}
+	if (m_VBWaterIndicesShore)
+	{
+		g_VBMan.Release(m_VBWaterIndicesShore);
+		m_VBWaterIndicesShore = 0;
+	}
 	m_WaterBounds.SetEmpty();
 
 	// We need to use this to access the water manager or we may not have the
@@ -1302,153 +1315,191 @@ void CPatchRData::BuildWater()
 	u16 water_index_map[PATCH_SIZE+1][PATCH_SIZE+1];
 	memset(water_index_map, 0xFF, sizeof(water_index_map));
 
-	// TODO: This is not (yet) exported via the ICmp interface so... we stick to these values which can be compiled in defaults
+	// Build data for shore
+	std::vector<SWaterVertex> water_vertex_data_shore;
+	std::vector<GLushort> water_indices_shore;
+	u16 water_shore_index_map[PATCH_SIZE+1][PATCH_SIZE+1];
+	memset(water_shore_index_map, 0xFF, sizeof(water_shore_index_map));
+	
 	WaterManager* WaterMgr = g_Renderer.GetWaterManager();
 
-	if (WaterMgr->m_NeedInfoUpdate)
-	{
-		WaterMgr->m_NeedInfoUpdate = false;
-		WaterMgr->CreateSuperfancyInfo(m_Simulation);
-	}
 	CPatch* patch = m_Patch;
 	CTerrain* terrain = patch->m_Parent;
 
 	ssize_t mapSize = (size_t)terrain->GetVerticesPerSide();
-
+	
+	//Top-left coordinates of our patch.
 	ssize_t x1 = m_Patch->m_X*PATCH_SIZE;
 	ssize_t z1 = m_Patch->m_Z*PATCH_SIZE;
 
+	// to whoever implements different water heights, this is a TODO: water height)
+	float waterHeight = cmpWaterManager->GetExactWaterLevel(0.0f,0.0f);
+
+	// The 4 points making a water tile.
+	int moves[4][2] = { {0,0}, {water_cell_size,0}, {0,water_cell_size}, {water_cell_size,water_cell_size} };
+	// Where to look for when checking for water for shore tiles.
+	int check[10][2] = { {0,0},{water_cell_size,0},{water_cell_size*2,0},{0,water_cell_size},{0,water_cell_size*2},{water_cell_size,water_cell_size},{water_cell_size*2,water_cell_size*2}, {-water_cell_size,0}, {0,-water_cell_size}, {-water_cell_size,-water_cell_size} };
+	
 	// build vertices, uv, and shader varying
 	for (ssize_t z = 0; z < PATCH_SIZE; z += water_cell_size)
 	{
-		for (ssize_t x = 0; x <= PATCH_SIZE; x += water_cell_size)
+		for (ssize_t x = 0; x < PATCH_SIZE; x += water_cell_size)
 		{
-			// Check that the edge at x is partially underwater
-			float startTerrainHeight[2] = { terrain->GetVertexGroundLevel(x+x1, z+z1), terrain->GetVertexGroundLevel(x+x1, z+z1 + water_cell_size) };
-			float startWaterHeight[2] = { cmpWaterManager->GetExactWaterLevel(x+x1, z+z1), cmpWaterManager->GetExactWaterLevel(x+x1, z+z1 + water_cell_size) };
-			if (startTerrainHeight[0] >= startWaterHeight[0] && startTerrainHeight[1] >= startWaterHeight[1])
+			
+			// Check that this tile is close to water
+			bool nearWat = false;
+			for (size_t test = 0; test < 10; ++test)
+				if (terrain->GetVertexGroundLevel(x+x1+check[test][0], z+z1+check[test][1]) < waterHeight)
+					nearWat = true;
+			if (!nearWat)
 				continue;
-
-			// Move x back one cell (unless at start of patch), then scan rightwards
-			bool belowWater = true;
-			ssize_t stripStart;
-			for (stripStart = x = std::max(x-water_cell_size, (ssize_t)0); x <= PATCH_SIZE; x += water_cell_size)
+			
+			// This is actually lying and I should call CcmpTerrain
+			/*if (!terrain->IsOnMap(x+x1, z+z1)
+			 && !terrain->IsOnMap(x+x1, z+z1 + water_cell_size)
+			 && !terrain->IsOnMap(x+x1 + water_cell_size, z+z1)
+			 && !terrain->IsOnMap(x+x1 + water_cell_size, z+z1 + water_cell_size))
+			 continue;*/
+						
+			for (int i = 0; i < 4; ++i)
 			{
-				// If this edge is not underwater, and neither is the previous edge
-				// (i.e. belowWater == false), then stop this strip since we've reached
-				// a cell that's entirely above water
-				float terrainHeight[2] = { terrain->GetVertexGroundLevel(x+x1, z+z1), terrain->GetVertexGroundLevel(x+x1, z+z1 + water_cell_size) };
-				float waterHeight[2] = { cmpWaterManager->GetExactWaterLevel(x+x1, z+z1), cmpWaterManager->GetExactWaterLevel(x+x1, z+z1 + water_cell_size) };
-				if (terrainHeight[0] >= waterHeight[0] && terrainHeight[1] >= waterHeight[1])
-				{
-					if (!belowWater)
-						break;
-					belowWater = false;
-				}
-				else
-					belowWater = true;
-
-				// Edge (x,z)-(x,z+1) is at least partially underwater, so extend the water plane strip across it
-
-				// Compute vertex data for the 2 points on the edge
-				for (int j = 0; j < 2; j++)
-				{
-					// Check if we already computed this vertex from an earlier strip
-					if (water_index_map[z+j*water_cell_size][x] != 0xFFFF)
-						continue;
-
-					SWaterVertex vertex;
-
-					terrain->CalcPosition(x+x1, z+z1 + j*water_cell_size, vertex.m_Position);
-					float depth = waterHeight[j] - vertex.m_Position.Y;
-					vertex.m_Position.Y = waterHeight[j];
-					m_WaterBounds += vertex.m_Position;
-
-					// NB: Usually this factor is view dependent, but for performance reasons
-					// we do not take it into account with basic non-shader based water.
-					// Average constant Fresnel effect for non-fancy water
-					float alpha = clamp(depth / WaterMgr->m_WaterFullDepth + WaterMgr->m_WaterAlphaOffset, WaterMgr->m_WaterAlphaOffset, WaterMgr->m_WaterMaxAlpha);
-
-					// Split the depth data across 24 bits, so the fancy-water shader can reconstruct
-					// the depth value while the simple-water can just use the precomputed alpha
-					float depthInt = floor(depth);
-					float depthFrac = depth - depthInt;
-					vertex.m_DepthData = SColor4ub(
-						u8(clamp(depthInt, 0.0f, 255.0f)),
-						u8(clamp(-depthInt, 0.0f, 255.0f)),
-						u8(clamp(depthFrac*255.0f, 0.0f, 255.0f)),
-						u8(clamp(alpha*255.0f, 0.0f, 255.0f)));
-
-					int tx = x+x1;
-					int ty = z+z1 + j*water_cell_size;
-
-					vertex.m_WaterData = CVector4D(WaterMgr->m_WaveX[tx + ty*mapSize],
-												   WaterMgr->m_WaveZ[tx + ty*mapSize],
-												   WaterMgr->m_DistanceToShore[tx + ty*mapSize],
-												   WaterMgr->m_FoamFactor[tx + ty*mapSize]);
-
-					water_index_map[z+j*water_cell_size][x] = water_vertex_data.size();
-					water_vertex_data.push_back(vertex);
-				}
-
-				// If this was not the first x in the strip, then add a quad
-				// using the computed vertex data
-
-				if (x <= stripStart)
+				if (water_index_map[z+moves[i][1]][x+moves[i][0]] != 0xFFFF)
 					continue;
-
-				water_indices.push_back(water_index_map[z + water_cell_size][x - water_cell_size]);
-				water_indices.push_back(water_index_map[z][x - water_cell_size]);
-				water_indices.push_back(water_index_map[z + water_cell_size][x]);
-
-				water_indices.push_back(water_index_map[z + water_cell_size][x]);
-				water_indices.push_back(water_index_map[z][x - water_cell_size]);
-				water_indices.push_back(water_index_map[z][x]);
+				
+				ssize_t zz = z+z1+moves[i][1];
+				ssize_t xx = x+x1+moves[i][0];
+				
+				SWaterVertex vertex;
+				terrain->CalcPosition(xx,zz, vertex.m_Position);
+				float depth = waterHeight - vertex.m_Position.Y;
+				
+				vertex.m_Position.Y = waterHeight;
+				
+				m_WaterBounds += vertex.m_Position;
+				
+				vertex.m_WaterData = CVector2D(WaterMgr->m_WindStrength[xx + zz*mapSize], depth);
+				
+				water_index_map[z+moves[i][1]][x+moves[i][0]] = water_vertex_data.size();
+				water_vertex_data.push_back(vertex);
+			}
+			water_indices.push_back(water_index_map[z + moves[2][1]][x + moves[2][0]]);
+			water_indices.push_back(water_index_map[z + moves[0][1]][x + moves[0][0]]);
+			water_indices.push_back(water_index_map[z + moves[1][1]][x + moves[1][0]]);
+			water_indices.push_back(water_index_map[z + moves[1][1]][x + moves[1][0]]);
+			water_indices.push_back(water_index_map[z + moves[3][1]][x + moves[3][0]]);
+			water_indices.push_back(water_index_map[z + moves[2][1]][x + moves[2][0]]);
+			
+			// Check id this tile is partly over land.
+			// If so add a square over the terrain. This is necessary to render waves that go on shore.
+			if (terrain->GetVertexGroundLevel(x+x1, z+z1) < waterHeight
+				&& terrain->GetVertexGroundLevel(x+x1 + water_cell_size, z+z1) < waterHeight
+				&& terrain->GetVertexGroundLevel(x+x1, z+z1+water_cell_size) < waterHeight
+				&& terrain->GetVertexGroundLevel(x+x1 + water_cell_size, z+z1+water_cell_size) < waterHeight)
+				continue;
+			
+			for (int i = 0; i < 4; ++i)
+			{
+				if (water_shore_index_map[z+moves[i][1]][x+moves[i][0]] != 0xFFFF)
+					continue;
+				ssize_t zz = z+z1+moves[i][1];
+				ssize_t xx = x+x1+moves[i][0];
+				
+				SWaterVertex vertex;
+				terrain->CalcPosition(xx,zz, vertex.m_Position);
+				
+				vertex.m_Position.Y += 0.02f;
+				m_WaterBounds += vertex.m_Position;
+				
+				vertex.m_WaterData = CVector2D(0.0f, -5.0f);
+				
+				water_shore_index_map[z+moves[i][1]][x+moves[i][0]] = water_vertex_data_shore.size();
+				water_vertex_data_shore.push_back(vertex);
+			}
+			if (terrain->GetTriangulationDir(x+x1,z+z1))
+			{
+				water_indices_shore.push_back(water_shore_index_map[z + moves[2][1]][x + moves[2][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[0][1]][x + moves[0][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[1][1]][x + moves[1][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[1][1]][x + moves[1][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[3][1]][x + moves[3][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[2][1]][x + moves[2][0]]);
+			}
+			else
+			{
+				water_indices_shore.push_back(water_shore_index_map[z + moves[3][1]][x + moves[3][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[2][1]][x + moves[2][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[0][1]][x + moves[0][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[3][1]][x + moves[3][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[0][1]][x + moves[0][0]]);
+				water_indices_shore.push_back(water_shore_index_map[z + moves[1][1]][x + moves[1][0]]);
 			}
 		}
 	}
 
 	// no vertex buffers if no data generated
-	if (water_indices.size() == 0)
-		return;
+	if (water_indices.size() != 0)
+	{
+		m_VBWater = g_VBMan.Allocate(sizeof(SWaterVertex), water_vertex_data.size(), GL_STATIC_DRAW, GL_ARRAY_BUFFER);
+		m_VBWater->m_Owner->UpdateChunkVertices(m_VBWater, &water_vertex_data[0]);
+		
+		m_VBWaterIndices = g_VBMan.Allocate(sizeof(GLushort), water_indices.size(), GL_STATIC_DRAW, GL_ELEMENT_ARRAY_BUFFER);
+		m_VBWaterIndices->m_Owner->UpdateChunkVertices(m_VBWaterIndices, &water_indices[0]);
+	}
 
-	// allocate vertex buffer
-	m_VBWater = g_VBMan.Allocate(sizeof(SWaterVertex), water_vertex_data.size(), GL_STATIC_DRAW, GL_ARRAY_BUFFER);
-	m_VBWater->m_Owner->UpdateChunkVertices(m_VBWater, &water_vertex_data[0]);
+	if (water_indices_shore.size() != 0)
+	{
+		m_VBWaterShore = g_VBMan.Allocate(sizeof(SWaterVertex), water_vertex_data_shore.size(), GL_STATIC_DRAW, GL_ARRAY_BUFFER);
+		m_VBWaterShore->m_Owner->UpdateChunkVertices(m_VBWaterShore, &water_vertex_data_shore[0]);
+		
+		// Construct indices buffer
+		m_VBWaterIndicesShore = g_VBMan.Allocate(sizeof(GLushort), water_indices_shore.size(), GL_STATIC_DRAW, GL_ELEMENT_ARRAY_BUFFER);
+		m_VBWaterIndicesShore->m_Owner->UpdateChunkVertices(m_VBWaterIndicesShore, &water_indices_shore[0]);
+	}}
 
-	// Construct indices buffer
-	m_VBWaterIndices = g_VBMan.Allocate(sizeof(GLushort), water_indices.size(), GL_STATIC_DRAW, GL_ELEMENT_ARRAY_BUFFER);
-	m_VBWaterIndices->m_Owner->UpdateChunkVertices(m_VBWaterIndices, &water_indices[0]);
-}
-
-void CPatchRData::RenderWater(CShaderProgramPtr& shader)
+void CPatchRData::RenderWater(CShaderProgramPtr& shader, bool onlyShore, bool fixedPipeline)
 {
 	ASSERT(m_UpdateFlags==0);
 
-	if (!m_VBWater)
+	if (g_Renderer.m_SkipSubmit || (!m_VBWater && !m_VBWaterShore))
 		return;
 
-	SWaterVertex *base=(SWaterVertex *)m_VBWater->m_Owner->Bind();
-
-	// setup data pointers
-	GLsizei stride = sizeof(SWaterVertex);
-	shader->ColorPointer(4, GL_UNSIGNED_BYTE, stride, &base[m_VBWater->m_Index].m_DepthData);
-	shader->VertexPointer(3, GL_FLOAT, stride, &base[m_VBWater->m_Index].m_Position);
-	shader->TexCoordPointer(GL_TEXTURE0, 4, GL_FLOAT, stride, &base[m_VBWater->m_Index].m_WaterData);
-
-	shader->AssertPointersBound();
-
-	// render
-	if (!g_Renderer.m_SkipSubmit)
+	if (m_VBWater != 0x0 && !onlyShore)
 	{
+		SWaterVertex *base=(SWaterVertex *)m_VBWater->m_Owner->Bind();
+		
+		// setup data pointers
+		GLsizei stride = sizeof(SWaterVertex);
+		shader->VertexPointer(3, GL_FLOAT, stride, &base[m_VBWater->m_Index].m_Position);
+		if (!fixedPipeline)
+			shader->VertexAttribPointer(str_a_waterInfo, 2, GL_FLOAT, false, stride, &base[m_VBWater->m_Index].m_WaterData);
+		
+		shader->AssertPointersBound();
+		
 		u8* indexBase = m_VBWaterIndices->m_Owner->Bind();
 		glDrawElements(GL_TRIANGLES, (GLsizei) m_VBWaterIndices->m_Count,
-			GL_UNSIGNED_SHORT, indexBase + sizeof(u16)*(m_VBWaterIndices->m_Index));
+					   GL_UNSIGNED_SHORT, indexBase + sizeof(u16)*(m_VBWaterIndices->m_Index));
+		
+		g_Renderer.m_Stats.m_DrawCalls++;
+		g_Renderer.m_Stats.m_WaterTris += m_VBWaterIndices->m_Count / 3;
+	}
+	if (m_VBWaterShore != 0x0 && g_Renderer.GetWaterManager()->m_WaterFancyEffects && !g_Renderer.GetWaterManager()->m_WaterUgly)
+	{
+		SWaterVertex *base=(SWaterVertex *)m_VBWaterShore->m_Owner->Bind();
+		
+		GLsizei stride = sizeof(SWaterVertex);
+		shader->VertexPointer(3, GL_FLOAT, stride, &base[m_VBWaterShore->m_Index].m_Position);
+		if (!fixedPipeline)
+			shader->VertexAttribPointer(str_a_waterInfo, 2, GL_FLOAT, false, stride, &base[m_VBWaterShore->m_Index].m_WaterData);
+		
+		shader->AssertPointersBound();
+		
+		u8* indexBase = m_VBWaterIndicesShore->m_Owner->Bind();
+		glDrawElements(GL_TRIANGLES, (GLsizei) m_VBWaterIndicesShore->m_Count,
+					   GL_UNSIGNED_SHORT, indexBase + sizeof(u16)*(m_VBWaterIndicesShore->m_Index));
+		
+		g_Renderer.m_Stats.m_DrawCalls++;
+		g_Renderer.m_Stats.m_WaterTris += m_VBWaterIndicesShore->m_Count / 3;
 	}
-
-	// bump stats
-	g_Renderer.m_Stats.m_DrawCalls++;
-	g_Renderer.m_Stats.m_WaterTris += m_VBWaterIndices->m_Count / 2;
 
 	CVertexBuffer::Unbind();
 }
