@@ -21,15 +21,16 @@
 #include <vector>
 #include "graphics/SColor.h"
 #include "maths/Vector3D.h"
+#include "maths/Vector2D.h"
 #include "graphics/RenderableObject.h"
-#include "graphics/ShaderProgram.h"
-#include "renderer/ShadowMap.h"
+#include "graphics/ShaderProgramPtr.h"
 #include "VertexBufferManager.h"
 
 class CPatch;
 class CSimulation2;
 class CTerrainTextureEntry;
 class CTextRenderer;
+class ShadowMap;
 
 //////////////////////////////////////////////////////////////////////////////////////////////////
 // CPatchRData: class encapsulating logic for rendering terrain patches; holds per
@@ -45,7 +46,7 @@ public:
 	void RenderSides(CShaderProgramPtr& shader);
 	void RenderPriorities(CTextRenderer& textRenderer);
 
-	void RenderWater(CShaderProgramPtr& shader);
+	void RenderWater(CShaderProgramPtr& shader, bool onlyShore = false, bool fixedPipeline = false);
 
 	static void RenderBases(const std::vector<CPatchRData*>& patches, const CShaderDefines& context, 
 			      ShadowMap* shadow, bool isDummyShader=false, const CShaderProgramPtr& dummy=CShaderProgramPtr());
@@ -105,12 +106,9 @@ private:
 	struct SWaterVertex {
 		// vertex position
 		CVector3D m_Position;
-		// (p,q,r, a) where
-		//   p*255 + q*-255 + r = depth of water
-		//   a = depth-dependent alpha
-		SColor4ub m_DepthData;
+		CVector2D m_WaterData;
 	};
-	cassert(sizeof(SWaterVertex) == 16);
+	cassert(sizeof(SWaterVertex) == 20);
 
 	// build this renderdata object
 	void Build();
@@ -154,9 +152,11 @@ private:
 
 	// Water vertex buffer
 	CVertexBuffer::VBChunk* m_VBWater;
+	CVertexBuffer::VBChunk* m_VBWaterShore;
 
 	// Water indices buffer
 	CVertexBuffer::VBChunk* m_VBWaterIndices;
+	CVertexBuffer::VBChunk* m_VBWaterIndicesShore;
 
 	CSimulation2* m_Simulation;
 
