@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -34,9 +34,10 @@
 #include "renderer/PostprocManager.h"
 #include "renderer/Renderer.h"
 
+#if !CONFIG2_GLES
 
 CPostprocManager::CPostprocManager()
-	: m_IsInitialised(false), m_PingFbo(0), m_PongFbo(0), m_PostProcEffect(L"default"), m_ColourTex1(0), m_ColourTex2(0), 
+	: m_IsInitialized(false), m_PingFbo(0), m_PongFbo(0), m_PostProcEffect(L"default"), m_ColourTex1(0), m_ColourTex2(0), 
 	  m_DepthTex(0), m_BloomFbo(0), m_BlurTex2a(0), m_BlurTex2b(0), m_BlurTex4a(0), m_BlurTex4b(0),
 	  m_BlurTex8a(0), m_BlurTex8b(0), m_WhichBuffer(true)
 {
@@ -44,53 +45,60 @@ CPostprocManager::CPostprocManager()
 
 CPostprocManager::~CPostprocManager()
 {
-	Cleanup();
-}
-
-
-void CPostprocManager::Initialize()
-{
-	RecreateBuffers();
-	m_IsInitialised = true;
-	SetPostEffect(L"default");
+	if (m_IsInitialized)
+		Cleanup();
 }
 
 void CPostprocManager::Cleanup()
 {
-	if (m_IsInitialised)
-	{
-		if (m_PingFbo) pglDeleteFramebuffersEXT(1, &m_PingFbo);
-		if (m_PongFbo) pglDeleteFramebuffersEXT(1, &m_PongFbo);
-		if (m_BloomFbo) pglDeleteFramebuffersEXT(1, &m_BloomFbo);
-		m_PingFbo = m_PongFbo = m_BloomFbo = 0;
-		
-		if (m_ColourTex1) glDeleteTextures(1, &m_ColourTex1);
-		if (m_ColourTex2) glDeleteTextures(1, &m_ColourTex2);
-		if (m_DepthTex) glDeleteTextures(1, &m_DepthTex);
-		m_ColourTex1 = m_ColourTex2 = m_DepthTex = 0;
-		
-		if (m_BlurTex2a) glDeleteTextures(1, &m_BlurTex2a);
-		if (m_BlurTex2b) glDeleteTextures(1, &m_BlurTex2b);
-		if (m_BlurTex4a) glDeleteTextures(1, &m_BlurTex4a);
-		if (m_BlurTex4b) glDeleteTextures(1, &m_BlurTex4b);
-		if (m_BlurTex8a) glDeleteTextures(1, &m_BlurTex8a);
-		if (m_BlurTex8b) glDeleteTextures(1, &m_BlurTex8b);
-		m_BlurTex2a = m_BlurTex2b = m_BlurTex4a = m_BlurTex4b = m_BlurTex8a = m_BlurTex8b = 0;
-	}
+	if (m_PingFbo) pglDeleteFramebuffersEXT(1, &m_PingFbo);
+	if (m_PongFbo) pglDeleteFramebuffersEXT(1, &m_PongFbo);
+	if (m_BloomFbo) pglDeleteFramebuffersEXT(1, &m_BloomFbo);
+	m_PingFbo = m_PongFbo = m_BloomFbo = 0;
+
+	if (m_ColourTex1) glDeleteTextures(1, &m_ColourTex1);
+	if (m_ColourTex2) glDeleteTextures(1, &m_ColourTex2);
+	if (m_DepthTex) glDeleteTextures(1, &m_DepthTex);
+	m_ColourTex1 = m_ColourTex2 = m_DepthTex = 0;
+
+	if (m_BlurTex2a) glDeleteTextures(1, &m_BlurTex2a);
+	if (m_BlurTex2b) glDeleteTextures(1, &m_BlurTex2b);
+	if (m_BlurTex4a) glDeleteTextures(1, &m_BlurTex4a);
+	if (m_BlurTex4b) glDeleteTextures(1, &m_BlurTex4b);
+	if (m_BlurTex8a) glDeleteTextures(1, &m_BlurTex8a);
+	if (m_BlurTex8b) glDeleteTextures(1, &m_BlurTex8b);
+	m_BlurTex2a = m_BlurTex2b = m_BlurTex4a = m_BlurTex4b = m_BlurTex8a = m_BlurTex8b = 0;
+}
+
+void CPostprocManager::Initialize()
+{
+	// The screen size starts out correct and then must be updated with Resize()
+	m_Width = g_Renderer.GetWidth();
+	m_Height = g_Renderer.GetHeight();
+
+	RecreateBuffers();
+	m_IsInitialized = true;
+}
+
+void CPostprocManager::Resize()
+{
+	m_Width = g_Renderer.GetWidth();
+	m_Height = g_Renderer.GetHeight();
+
+	// If the buffers were intialized, recreate them to the new size.
+	if (m_IsInitialized)
+		RecreateBuffers();
 }
 
 void CPostprocManager::RecreateBuffers()
 {
-	Cleanup();
-	
-	m_Width = g_Renderer.GetWidth();
-	m_Height = g_Renderer.GetHeight();
-	
+	if (m_IsInitialized) // Only cleanup if previously used
+		Cleanup();
+
 	#define GEN_BUFFER_RGBA(name, w, h) \
 		glGenTextures(1, (GLuint*)&name); \
 		glBindTexture(GL_TEXTURE_2D, name); \
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, \
-			GL_UNSIGNED_BYTE, 0); \
+		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0); \
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); \
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); \
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE); \
@@ -118,10 +126,11 @@ void CPostprocManager::RecreateBuffers()
 	// Allocate the Depth/Stencil texture.
 	glGenTextures(1, (GLuint*)&m_DepthTex);
 	glBindTexture(GL_TEXTURE_2D, m_DepthTex);
+
 	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH24_STENCIL8_EXT, m_Width, m_Height,
-		      0, GL_DEPTH_STENCIL_EXT, GL_UNSIGNED_INT_24_8_EXT, 0);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE,
-			GL_NONE);
+				 0, GL_DEPTH_STENCIL_EXT, GL_UNSIGNED_INT_24_8_EXT, 0);
+
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_NONE);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
@@ -133,8 +142,12 @@ void CPostprocManager::RecreateBuffers()
 	
 	pglGenFramebuffersEXT(1, &m_PingFbo);
 	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_PingFbo);
-	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_ColourTex1, 0);
-	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_STENCIL_ATTACHMENT, GL_TEXTURE_2D, m_DepthTex, 0);
+
+	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
+							   GL_TEXTURE_2D, m_ColourTex1, 0);
+
+	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_STENCIL_ATTACHMENT,
+							   GL_TEXTURE_2D, m_DepthTex, 0);
 	
 	GLenum status = pglCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
 	if (status != GL_FRAMEBUFFER_COMPLETE_EXT)
@@ -144,8 +157,12 @@ void CPostprocManager::RecreateBuffers()
 	
 	pglGenFramebuffersEXT(1, &m_PongFbo);
 	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_PongFbo);
-	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_ColourTex2, 0);
-	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_STENCIL_ATTACHMENT, GL_TEXTURE_2D, m_DepthTex, 0);
+
+	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
+							   GL_TEXTURE_2D, m_ColourTex2, 0);
+
+	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_STENCIL_ATTACHMENT,
+							   GL_TEXTURE_2D, m_DepthTex, 0);
 	
 	status = pglCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
 	if (status != GL_FRAMEBUFFER_COMPLETE_EXT)
@@ -155,13 +172,17 @@ void CPostprocManager::RecreateBuffers()
 	
 	pglGenFramebuffersEXT(1, &m_BloomFbo);
 	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_BloomFbo);
-	/*pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_BloomTex1, 0);
+
+	/*
+	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
+							   GL_TEXTURE_2D, m_BloomTex1, 0);
 	
 	status = pglCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
 	if (status != GL_FRAMEBUFFER_COMPLETE_EXT)
 	{
 		LOGWARNING(L"Framebuffer object incomplete (B): 0x%04X", status);
-	}*/
+	}
+	*/
 
 	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
 }
@@ -175,8 +196,8 @@ void CPostprocManager::ApplyBlurDownscale2x(GLuint inTex, GLuint outTex, int inW
 	
 	// Get bloom shader with instructions to simply copy texels.
 	CShaderDefines defines;
-	defines.Add("BLOOM_NOP", "1");
-	CShaderTechniquePtr tech = g_Renderer.GetShaderManager().LoadEffect(CStrIntern("bloom"),
+	defines.Add(str_BLOOM_NOP, str_1);
+	CShaderTechniquePtr tech = g_Renderer.GetShaderManager().LoadEffect(str_bloom,
 			g_Renderer.GetSystemShaderDefines(), defines);
 	
 	tech->BeginPass();
@@ -192,20 +213,37 @@ void CPostprocManager::ApplyBlurDownscale2x(GLuint inTex, GLuint outTex, int inW
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glBindTexture(GL_TEXTURE_2D, 0);
 	
-	shader->BindTexture("renderedTex", renderedTex);
+	shader->BindTexture(str_renderedTex, renderedTex);
 	
-	glPushAttrib(GL_VIEWPORT_BIT); 
-	glViewport(0, 0, inWidth / 2, inHeight / 2);
+	const SViewPort oldVp = g_Renderer.GetViewport();
+	const SViewPort vp = { 0, 0, inWidth / 2, inHeight / 2 };
+	g_Renderer.SetViewport(vp);
 	
-	glBegin(GL_QUADS);
-	    glColor4f(1.f, 1.f, 1.f, 1.f);
-	    glTexCoord2f(1.0, 1.0);	glVertex2f(1,1);
-	    glTexCoord2f(0.0, 1.0);	glVertex2f(-1,1);	    
-	    glTexCoord2f(0.0, 0.0);	glVertex2f(-1,-1);
-	    glTexCoord2f(1.0, 0.0);	glVertex2f(1,-1);
-	glEnd();
+	float quadVerts[] = {
+		1.0f, 1.0f,
+		-1.0f, 1.0f,
+		-1.0f, -1.0f,
+
+		-1.0f, -1.0f,
+		1.0f, -1.0f,
+		1.0f, 1.0f
+	};
+	float quadTex[] = {
+		1.0f, 1.0f,
+		0.0f, 1.0f,
+		0.0f, 0.0f,
+
+		0.0f, 0.0f,
+		1.0f, 0.0f,
+		1.0f, 1.0f
+	};
+	shader->TexCoordPointer(GL_TEXTURE0, 2, GL_FLOAT, 0, quadTex);
+	shader->VertexPointer(2, GL_FLOAT, 0, quadVerts);
+	shader->AssertPointersBound();
+	glDrawArrays(GL_TRIANGLES, 0, 6);
+
+	g_Renderer.SetViewport(oldVp);
 	
-	glPopAttrib(); 
 	tech->EndPass();
 }
 
@@ -217,27 +255,44 @@ void CPostprocManager::ApplyBlurGauss(GLuint inOutTex, GLuint tempTex, int inWid
 	
 	// Get bloom shader, for a horizontal Gaussian blur pass.
 	CShaderDefines defines2;
-	defines2.Add("BLOOM_PASS_H", "1");
-	CShaderTechniquePtr tech = g_Renderer.GetShaderManager().LoadEffect(CStrIntern("bloom"),
+	defines2.Add(str_BLOOM_PASS_H, str_1);
+	CShaderTechniquePtr tech = g_Renderer.GetShaderManager().LoadEffect(str_bloom,
 			g_Renderer.GetSystemShaderDefines(), defines2);
 	
 	tech->BeginPass();
 	CShaderProgramPtr shader = tech->GetShader();
-	shader->BindTexture("renderedTex", inOutTex);
-	shader->Uniform("texSize", inWidth, inHeight, 0.0f, 0.0f);
+	shader->BindTexture(str_renderedTex, inOutTex);
+	shader->Uniform(str_texSize, inWidth, inHeight, 0.0f, 0.0f);
 	
-	glPushAttrib(GL_VIEWPORT_BIT); 
-	glViewport(0, 0, inWidth, inHeight);
+	const SViewPort oldVp = g_Renderer.GetViewport();
+	const SViewPort vp = { 0, 0, inWidth, inHeight };
+	g_Renderer.SetViewport(vp);
 	
-	glBegin(GL_QUADS);
-	    glColor4f(1.f, 1.f, 1.f, 1.f);
-	    glTexCoord2f(1.0, 1.0);	glVertex2f(1,1);
-	    glTexCoord2f(0.0, 1.0);	glVertex2f(-1,1);
-	    glTexCoord2f(0.0, 0.0);	glVertex2f(-1,-1);
-	    glTexCoord2f(1.0, 0.0);	glVertex2f(1,-1);
-	glEnd();
-	
-	glPopAttrib(); 
+	float quadVerts[] = {
+		1.0f, 1.0f,
+		-1.0f, 1.0f,
+		-1.0f, -1.0f,
+
+		-1.0f, -1.0f,
+		1.0f, -1.0f,
+		1.0f, 1.0f
+	};
+	float quadTex[] = {
+		1.0f, 1.0f,
+		0.0f, 1.0f,
+		0.0f, 0.0f,
+
+		0.0f, 0.0f,
+		1.0f, 0.0f,
+		1.0f, 1.0f
+	};
+	shader->TexCoordPointer(GL_TEXTURE0, 2, GL_FLOAT, 0, quadTex);
+	shader->VertexPointer(2, GL_FLOAT, 0, quadVerts);
+	shader->AssertPointersBound();
+	glDrawArrays(GL_TRIANGLES, 0, 6);
+
+	g_Renderer.SetViewport(oldVp);
+
 	tech->EndPass();
 	
 	// Set result texture as our render target.
@@ -246,29 +301,26 @@ void CPostprocManager::ApplyBlurGauss(GLuint inOutTex, GLuint tempTex, int inWid
 	
 	// Get bloom shader, for a vertical Gaussian blur pass.
 	CShaderDefines defines3;
-	defines3.Add("BLOOM_PASS_V", "1");
-	tech = g_Renderer.GetShaderManager().LoadEffect(CStrIntern("bloom"),
+	defines3.Add(str_BLOOM_PASS_V, str_1);
+	tech = g_Renderer.GetShaderManager().LoadEffect(str_bloom,
 			g_Renderer.GetSystemShaderDefines(), defines3);
 	
 	tech->BeginPass();
 	shader = tech->GetShader();
 	
 	// Our input texture to the shader is the output of the horizontal pass.
-	shader->BindTexture("renderedTex", tempTex);
-	shader->Uniform("texSize", inWidth, inHeight, 0.0f, 0.0f);
+	shader->BindTexture(str_renderedTex, tempTex);
+	shader->Uniform(str_texSize, inWidth, inHeight, 0.0f, 0.0f);
 	
-	glPushAttrib(GL_VIEWPORT_BIT); 
-	glViewport(0, 0, inWidth, inHeight);
+	g_Renderer.SetViewport(vp);
 	
-	glBegin(GL_QUADS);
-	    glColor4f(1.f, 1.f, 1.f, 1.f);
-	    glTexCoord2f(1.0, 1.0);	glVertex2f(1,1);
-	    glTexCoord2f(0.0, 1.0);	glVertex2f(-1,1);
-	    glTexCoord2f(0.0, 0.0);	glVertex2f(-1,-1);
-	    glTexCoord2f(1.0, 0.0);	glVertex2f(1,-1);
-	glEnd();
-	
-	glPopAttrib(); 
+	shader->TexCoordPointer(GL_TEXTURE0, 2, GL_FLOAT, 0, quadTex);
+	shader->VertexPointer(2, GL_FLOAT, 0, quadVerts);
+	shader->AssertPointersBound();
+	glDrawArrays(GL_TRIANGLES, 0, 6);
+
+	g_Renderer.SetViewport(oldVp);
+
 	tech->EndPass();
 }
 
@@ -300,6 +352,9 @@ void CPostprocManager::ApplyBlur()
 
 void CPostprocManager::CaptureRenderOutput()
 {
+	if (!m_IsInitialized)
+		Initialize();
+
 	// clear both FBOs and leave m_PingFbo selected for rendering; 
 	// m_WhichBuffer stays true at this point
 	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_PongFbo);	
@@ -318,6 +373,9 @@ void CPostprocManager::CaptureRenderOutput()
 
 void CPostprocManager::ReleaseRenderOutput()
 {
+	if (!m_IsInitialized)
+		Initialize();
+
 	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
 	
@@ -338,13 +396,13 @@ void CPostprocManager::ReleaseRenderOutput()
 
 void CPostprocManager::LoadEffect(CStrW &name)
 {
-	if (!m_IsInitialised) 
-		return;
-	
+	if (!m_IsInitialized)
+		Initialize();
+
 	if (name != L"default")
 	{
 		CStrW n = L"postproc/" + name;
-		m_PostProcTech = g_Renderer.GetShaderManager().LoadEffect(n.ToUTF8().c_str());
+		m_PostProcTech = g_Renderer.GetShaderManager().LoadEffect(CStrIntern(n.ToUTF8()));
 	}
 	
 	m_PostProcEffect = name;
@@ -371,33 +429,48 @@ void CPostprocManager::ApplyEffect(CShaderTechniquePtr &shaderTech1, int pass)
 	// We also bind a bunch of other textures and parameters, but since
 	// this only happens once per frame the overhead is negligible.
 	if (m_WhichBuffer)
-		shader->BindTexture("renderedTex", m_ColourTex1);
+		shader->BindTexture(str_renderedTex, m_ColourTex1);
 	else
-		shader->BindTexture("renderedTex", m_ColourTex2);
+		shader->BindTexture(str_renderedTex, m_ColourTex2);
 	
-	shader->BindTexture("depthTex", m_DepthTex);
+	shader->BindTexture(str_depthTex, m_DepthTex);
 	
-	shader->BindTexture("blurTex2", m_BlurTex2a);
-	shader->BindTexture("blurTex4", m_BlurTex4a);
-	shader->BindTexture("blurTex8", m_BlurTex8a);
+	shader->BindTexture(str_blurTex2, m_BlurTex2a);
+	shader->BindTexture(str_blurTex4, m_BlurTex4a);
+	shader->BindTexture(str_blurTex8, m_BlurTex8a);
 	
-	shader->Uniform("width", m_Width);
-	shader->Uniform("height", m_Height);
-	shader->Uniform("zNear", g_Game->GetView()->GetNear());
-	shader->Uniform("zFar", g_Game->GetView()->GetFar());
+	shader->Uniform(str_width, m_Width);
+	shader->Uniform(str_height, m_Height);
+	shader->Uniform(str_zNear, g_Game->GetView()->GetNear());
+	shader->Uniform(str_zFar, g_Game->GetView()->GetFar());
 	
-	shader->Uniform("brightness", g_LightEnv.m_Brightness);
-	shader->Uniform("hdr", g_LightEnv.m_Contrast);
-	shader->Uniform("saturation", g_LightEnv.m_Saturation);
-	shader->Uniform("bloom", g_LightEnv.m_Bloom);
+	shader->Uniform(str_brightness, g_LightEnv.m_Brightness);
+	shader->Uniform(str_hdr, g_LightEnv.m_Contrast);
+	shader->Uniform(str_saturation, g_LightEnv.m_Saturation);
+	shader->Uniform(str_bloom, g_LightEnv.m_Bloom);
 	
-	glBegin(GL_QUADS);
-	    glColor4f(1.f, 1.f, 1.f, 1.f);
-	    glTexCoord2f(1.0, 1.0);	glVertex2f(1,1);
-	    glTexCoord2f(0.0, 1.0);	glVertex2f(-1,1);	    
-	    glTexCoord2f(0.0, 0.0);	glVertex2f(-1,-1);
-	    glTexCoord2f(1.0, 0.0);	glVertex2f(1,-1);
-	glEnd();
+	float quadVerts[] = {
+		1.0f, 1.0f,
+		-1.0f, 1.0f,
+		-1.0f, -1.0f,
+
+		-1.0f, -1.0f,
+		1.0f, -1.0f,
+		1.0f, 1.0f
+	};
+	float quadTex[] = {
+		1.0f, 1.0f,
+		0.0f, 1.0f,
+		0.0f, 0.0f,
+
+		0.0f, 0.0f,
+		1.0f, 0.0f,
+		1.0f, 1.0f
+	};
+	shader->TexCoordPointer(GL_TEXTURE0, 2, GL_FLOAT, 0, quadTex);
+	shader->VertexPointer(2, GL_FLOAT, 0, quadVerts);
+	shader->AssertPointersBound();
+	glDrawArrays(GL_TRIANGLES, 0, 6);
 	
 	shader->Unbind();
 	
@@ -411,14 +484,12 @@ void CPostprocManager::ApplyEffect(CShaderTechniquePtr &shaderTech1, int pass)
 
 void CPostprocManager::ApplyPostproc()
 {
-	if (!m_IsInitialised) 
-		return;
-	
+	if (!m_IsInitialized)
+		Initialize();
+
 	// Don't do anything if we are using the default effect.
 	if (m_PostProcEffect == L"default")
-	{
 		return;
-	}
 	
 	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_PongFbo);
 	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT1_EXT, GL_TEXTURE_2D, 0, 0);
@@ -441,9 +512,7 @@ void CPostprocManager::ApplyPostproc()
 	ApplyBlur(); 
 
 	for (int pass = 0; pass < m_PostProcTech->GetNumPasses(); ++pass)
-	{
 		ApplyEffect(m_PostProcTech, pass);
-	}
 	
 	pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, m_PongFbo);
 	pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_STENCIL_ATTACHMENT, GL_TEXTURE_2D, m_DepthTex, 0);
@@ -454,7 +523,7 @@ void CPostprocManager::ApplyPostproc()
 
 
 // Generate list of available effect-sets
-std::vector<CStrW> CPostprocManager::GetPostEffects() const
+std::vector<CStrW> CPostprocManager::GetPostEffects()
 {
 	std::vector<CStrW> effects;
 	
@@ -462,10 +531,8 @@ std::vector<CStrW> CPostprocManager::GetPostEffects() const
 	
 	VfsPaths pathnames;
 	if(vfs::GetPathnames(g_VFS, path, 0, pathnames) < 0)
-	{
 		LOGERROR(L"Error finding Post effects in '%ls'", path.string().c_str());
-	}
-	
+
 	for(size_t i = 0; i < pathnames.size(); i++)
 	{
 		if (pathnames[i].Extension() != L".xml")
@@ -484,5 +551,71 @@ std::vector<CStrW> CPostprocManager::GetPostEffects() const
 
 void CPostprocManager::SetPostEffect(CStrW name)
 {
+	if (!m_IsInitialized)
+		Initialize();
+
 	LoadEffect(name);
 }
+
+#else
+
+#warning TODO: implement PostprocManager for GLES
+
+void ApplyBlurDownscale2x(GLuint UNUSED(inTex), GLuint UNUSED(outTex), int UNUSED(inWidth), int UNUSED(inHeight))
+{
+}
+
+void CPostprocManager::ApplyBlurGauss(GLuint UNUSED(inOutTex), GLuint UNUSED(tempTex), int UNUSED(inWidth), int UNUSED(inHeight))
+{
+}
+
+void CPostprocManager::ApplyEffect(CShaderTechniquePtr &UNUSED(shaderTech1), int UNUSED(pass))
+{
+}
+	
+CPostprocManager::CPostprocManager()
+{
+}
+
+CPostprocManager::~CPostprocManager()
+{
+}
+
+void CPostprocManager::Initialize()
+{
+}
+
+void CPostprocManager::Cleanup()
+{
+}
+
+void CPostprocManager::RecreateBuffers()
+{
+}
+
+void CPostprocManager::LoadEffect(CStrW &UNUSED(name))
+{
+}
+
+std::vector<CStrW> CPostprocManager::GetPostEffects() const
+{
+	return std::vector<CStrW>();
+}
+
+void CPostprocManager::SetPostEffect(CStrW UNUSED(name))
+{
+}
+
+void CPostprocManager::CaptureRenderOutput()
+{
+}
+	
+void CPostprocManager::ApplyPostproc()
+{
+}
+
+void CPostprocManager::ReleaseRenderOutput()
+{
+}
+
+#endif
