@@ -78,16 +78,13 @@ struct TerrainRendererInternals
 	Phase phase;
 
 	/// Patches that were submitted for this frame
-	std::vector<CPatchRData*> visiblePatches;
-	std::vector<CPatchRData*> filteredPatches;
+	std::vector<CPatchRData*> visiblePatches[CRenderer::CULL_MAX];
 
 	/// Decals that were submitted for this frame
-	std::vector<CDecalRData*> visibleDecals;
-	std::vector<CDecalRData*> filteredDecals;
+	std::vector<CDecalRData*> visibleDecals[CRenderer::CULL_MAX];
 
 	/// Fancy water shader
 	CShaderProgramPtr fancyWaterShader;
-	CShaderProgramPtr wavesShader;
 
 	CSimulation2* simulation;
 };
@@ -114,7 +111,7 @@ void TerrainRenderer::SetSimulation(CSimulation2* simulation)
 
 ///////////////////////////////////////////////////////////////////
 // Submit a patch for rendering
-void TerrainRenderer::Submit(CPatch* patch)
+void TerrainRenderer::Submit(int cullGroup, CPatch* patch)
 {
 	ENSURE(m->phase == Phase_Submit);
 
@@ -127,12 +124,12 @@ void TerrainRenderer::Submit(CPatch* patch)
 	}
 	data->Update(m->simulation);
 
-	m->visiblePatches.push_back(data);
+	m->visiblePatches[cullGroup].push_back(data);
 }
 
 ///////////////////////////////////////////////////////////////////
 // Submit a decal for rendering
-void TerrainRenderer::Submit(CModelDecal* decal)
+void TerrainRenderer::Submit(int cullGroup, CModelDecal* decal)
 {
 	ENSURE(m->phase == Phase_Submit);
 
@@ -145,7 +142,7 @@ void TerrainRenderer::Submit(CModelDecal* decal)
 	}
 	data->Update(m->simulation);
 
-	m->visibleDecals.push_back(data);
+	m->visibleDecals[cullGroup].push_back(data);
 }
 
 ///////////////////////////////////////////////////////////////////
@@ -163,45 +160,27 @@ void TerrainRenderer::EndFrame()
 {
 	ENSURE(m->phase == Phase_Render || m->phase == Phase_Submit);
 
-	m->visiblePatches.clear();
-	m->visibleDecals.clear();
+	for (int i = 0; i < CRenderer::CULL_MAX; ++i)
+	{
+		m->visiblePatches[i].clear();
+		m->visibleDecals[i].clear();
+	}
 
 	m->phase = Phase_Submit;
 }
 
 
-///////////////////////////////////////////////////////////////////
-// Culls patches and decals against a frustum.
-bool TerrainRenderer::CullPatches(const CFrustum* frustum)
-{
-	m->filteredPatches.clear();
-	for (std::vector<CPatchRData*>::iterator it = m->visiblePatches.begin(); it != m->visiblePatches.end(); ++it)
-	{
-		if (frustum->IsBoxVisible(CVector3D(0, 0, 0), (*it)->GetPatch()->GetWorldBounds()))
-			m->filteredPatches.push_back(*it);
-	}
-
-	m->filteredDecals.clear();
-	for (std::vector<CDecalRData*>::iterator it = m->visibleDecals.begin(); it != m->visibleDecals.end(); ++it)
-	{
-		if (frustum->IsBoxVisible(CVector3D(0, 0, 0), (*it)->GetDecal()->GetWorldBounds()))
-			m->filteredDecals.push_back(*it);
-	}
-
-	return !m->filteredPatches.empty() || !m->filteredDecals.empty();
-}
-
 ///////////////////////////////////////////////////////////////////
 // Full-featured terrain rendering with blending and everything
-void TerrainRenderer::RenderTerrain(bool filtered)
+void TerrainRenderer::RenderTerrain(int cullGroup)
 {
 #if CONFIG2_GLES
-	UNUSED2(filtered);
+	UNUSED2(cullGroup);
 #else
 	ENSURE(m->phase == Phase_Render);
 
-	std::vector<CPatchRData*>& visiblePatches = filtered ? m->filteredPatches : m->visiblePatches;
-	std::vector<CDecalRData*>& visibleDecals = filtered ? m->filteredDecals : m->visibleDecals;
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
+	std::vector<CDecalRData*>& visibleDecals = m->visibleDecals[cullGroup];
 	if (visiblePatches.empty() && visibleDecals.empty())
 		return;
 
@@ -396,15 +375,16 @@ void TerrainRenderer::RenderTerrain(bool filtered)
 #endif
 }
 
-void TerrainRenderer::RenderTerrainOverlayTexture(CMatrix3D& textureMatrix)
+void TerrainRenderer::RenderTerrainOverlayTexture(int cullGroup, CMatrix3D& textureMatrix)
 {
 #if CONFIG2_GLES
 #warning TODO: implement TerrainRenderer::RenderTerrainOverlayTexture for GLES
+	UNUSED2(cullGroup);
 	UNUSED2(textureMatrix);
 #else
 	ENSURE(m->phase == Phase_Render);
 
-	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches;
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
 
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
@@ -428,9 +408,9 @@ void TerrainRenderer::RenderTerrainOverlayTexture(CMatrix3D& textureMatrix)
 	// To make the overlay visible over water, render an additional map-sized
 	// water-height patch
 	CBoundingBoxAligned waterBounds;
-	for (size_t i = 0; i < m->visiblePatches.size(); ++i)
+	for (size_t i = 0; i < visiblePatches.size(); ++i)
 	{
-		CPatchRData* data = m->visiblePatches[i];
+		CPatchRData* data = visiblePatches[i];
 		waterBounds += data->GetWaterBounds();
 	}
 	if (!waterBounds.IsEmpty())
@@ -493,12 +473,12 @@ void TerrainRenderer::PrepareShader(const CShaderProgramPtr& shader, ShadowMap*
 	shader->Uniform(str_fogParams, lightEnv.m_FogFactor, lightEnv.m_FogMax, 0.f, 0.f);
 }
 
-void TerrainRenderer::RenderTerrainShader(const CShaderDefines& context, ShadowMap* shadow, bool filtered)
+void TerrainRenderer::RenderTerrainShader(const CShaderDefines& context, int cullGroup, ShadowMap* shadow)
 {
 	ENSURE(m->phase == Phase_Render);
 
-	std::vector<CPatchRData*>& visiblePatches = filtered ? m->filteredPatches : m->visiblePatches;
-	std::vector<CDecalRData*>& visibleDecals = filtered ? m->filteredDecals : m->visibleDecals;
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
+	std::vector<CDecalRData*>& visibleDecals = m->visibleDecals[cullGroup];
 	if (visiblePatches.empty() && visibleDecals.empty())
 		return;
 
@@ -545,11 +525,11 @@ void TerrainRenderer::RenderTerrainShader(const CShaderDefines& context, ShadowM
 
 ///////////////////////////////////////////////////////////////////
 // Render un-textured patches as polygons
-void TerrainRenderer::RenderPatches(bool filtered)
+void TerrainRenderer::RenderPatches(int cullGroup)
 {
 	ENSURE(m->phase == Phase_Render);
 
-	std::vector<CPatchRData*>& visiblePatches = filtered ? m->filteredPatches : m->visiblePatches;
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
 	if (visiblePatches.empty())
 		return;
 
@@ -570,11 +550,11 @@ void TerrainRenderer::RenderPatches(bool filtered)
 
 ///////////////////////////////////////////////////////////////////
 // Render outlines of submitted patches as lines
-void TerrainRenderer::RenderOutlines(bool filtered)
+void TerrainRenderer::RenderOutlines(int cullGroup)
 {
 	ENSURE(m->phase == Phase_Render);
 
-	std::vector<CPatchRData*>& visiblePatches = filtered ? m->filteredPatches : m->visiblePatches;
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
 	if (visiblePatches.empty())
 		return;
 
@@ -591,12 +571,14 @@ void TerrainRenderer::RenderOutlines(bool filtered)
 
 ///////////////////////////////////////////////////////////////////
 // Scissor rectangle of water patches
-CBoundingBoxAligned TerrainRenderer::ScissorWater(const CMatrix3D &viewproj)
+CBoundingBoxAligned TerrainRenderer::ScissorWater(int cullGroup, const CMatrix3D &viewproj)
 {
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
+
 	CBoundingBoxAligned scissor;
-	for (size_t i = 0; i < m->visiblePatches.size(); ++i)
+	for (size_t i = 0; i < visiblePatches.size(); ++i)
 	{
-		CPatchRData* data = m->visiblePatches[i];
+		CPatchRData* data = visiblePatches[i];
 		const CBoundingBoxAligned& waterBounds = data->GetWaterBounds();
 		if (waterBounds.IsEmpty())
 			continue;
@@ -642,40 +624,26 @@ CBoundingBoxAligned TerrainRenderer::ScissorWater(const CMatrix3D &viewproj)
 }
 
 // Render fancy water
-bool TerrainRenderer::RenderFancyWater(const CShaderDefines& context, ShadowMap* shadow)
+bool TerrainRenderer::RenderFancyWater(const CShaderDefines& context, int cullGroup, ShadowMap* shadow)
 {
 	PROFILE3_GPU("fancy water");
 	
 	WaterManager* WaterMgr = g_Renderer.GetWaterManager();
 	CShaderDefines defines = context;
 	
-	WaterMgr->UpdateQuality();
-
 	// If we're using fancy water, make sure its shader is loaded
 	if (!m->fancyWaterShader || WaterMgr->m_NeedsReloading)
 	{
-		if (WaterMgr->m_WaterNormal)
-			defines.Add(str_USE_NORMALS, str_1);
 		if (WaterMgr->m_WaterRealDepth)
 			defines.Add(str_USE_REAL_DEPTH, str_1);
-		if (WaterMgr->m_WaterFoam)
-			defines.Add(str_USE_FOAM, str_1);
-		if (WaterMgr->m_WaterCoastalWaves && false)
-			defines.Add(str_USE_WAVES, str_1);
+		if (WaterMgr->m_WaterFancyEffects)
+			defines.Add(str_USE_FANCY_EFFECTS, str_1);
 		if (WaterMgr->m_WaterRefraction)
 			defines.Add(str_USE_REFRACTION, str_1);
 		if (WaterMgr->m_WaterReflection)
 			defines.Add(str_USE_REFLECTION, str_1);
 		if (shadow && WaterMgr->m_WaterShadows)
-			defines.Add(str_USE_SHADOWS, str_1);
-
-		m->wavesShader = g_Renderer.GetShaderManager().LoadProgram("glsl/waves", defines);
-		if (!m->wavesShader)
-		{
-			LOGERROR(L"Failed to load waves shader. Deactivating waves.\n");
-			g_Renderer.SetOptionBool(CRenderer::OPT_WATERCOASTALWAVES, false);
-			defines.Add(str_USE_WAVES, str_0);
-		}
+			defines.Add(str_USE_SHADOWS_ON_WATER, str_1);
 		
 		// haven't updated the ARB shader yet so I'll always load the GLSL
 		/*if (!g_Renderer.m_Options.m_PreferGLSL && !superFancy)
@@ -685,7 +653,7 @@ bool TerrainRenderer::RenderFancyWater(const CShaderDefines& context, ShadowMap*
 		
 		if (!m->fancyWaterShader)
 		{
-			LOGERROR(L"Failed to load water shader. Falling back to non-fancy water.\n");
+			LOGERROR(L"Failed to load water shader. Falling back to fixed pipeline water.\n");
 			WaterMgr->m_RenderWater = false;
 			return false;
 		}
@@ -694,37 +662,26 @@ bool TerrainRenderer::RenderFancyWater(const CShaderDefines& context, ShadowMap*
 	
 	CLOSTexture& losTexture = g_Renderer.GetScene().GetLOSTexture();
 
-	GLuint depthTex;
 	// creating the real depth texture using the depth buffer.
 	if (WaterMgr->m_WaterRealDepth)
 	{
 		if (WaterMgr->m_depthTT == 0)
 		{
+			GLuint depthTex;
 			glGenTextures(1, (GLuint*)&depthTex);
 			WaterMgr->m_depthTT = depthTex;
 			glBindTexture(GL_TEXTURE_2D, WaterMgr->m_depthTT);
-			
-#if CONFIG2_GLES
-			GLenum format = GL_DEPTH_COMPONENT;
-#else
-			GLenum format = GL_DEPTH_COMPONENT32;
-#endif
-			
-			// TODO: use POT texture
-			glTexImage2D(GL_TEXTURE_2D, 0, format, g_Renderer.GetWidth(), g_Renderer.GetHeight(),
-						 0, GL_DEPTH_COMPONENT, GL_UNSIGNED_BYTE,NULL);
+			glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, g_Renderer.GetWidth(), g_Renderer.GetHeight(), 0, GL_DEPTH_COMPONENT, GL_UNSIGNED_BYTE,NULL);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+		}
+		else
+		{
+			glBindTexture(GL_TEXTURE_2D, WaterMgr->m_depthTT);
+			glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, 0, 0, g_Renderer.GetWidth(), g_Renderer.GetHeight(), 0);
 		}
-		glBindTexture(GL_TEXTURE_2D, WaterMgr->m_depthTT);
-#if !CONFIG2_GLES
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_NONE);
-#endif
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-		
-		glCopyTexImage2D(GL_TEXTURE_2D,0,GL_DEPTH_COMPONENT, 0, 0, g_Renderer.GetWidth(), g_Renderer.GetHeight(), 0);
-		
 		glBindTexture(GL_TEXTURE_2D, 0);
 	}
 	// Calculating the advanced informations about Foam and all if the quality calls for it.
@@ -734,120 +691,94 @@ bool TerrainRenderer::RenderFancyWater(const CShaderDefines& context, ShadowMap*
 		WaterMgr->CreateSuperfancyInfo();
 	}*/
 	
-	glEnable(GL_BLEND);
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	glEnable(GL_DEPTH_TEST);
-	glDepthFunc(GL_LEQUAL);
-
 	double time = WaterMgr->m_WaterTexTimer;
 	double period = 8;
 	int curTex = (int)(time*60/period) % 60;
 	int nexTex = (curTex + 1) % 60;
-
-	GLuint FramebufferName = 0;
-
-	// rendering waves to a framebuffer
-	// TODO: reactivate this with something that looks good.
-	if (false && WaterMgr->m_WaterCoastalWaves && WaterMgr->m_VBWaves && !g_AtlasGameLoop->running)
+	
+	float repeatPeriod = WaterMgr->m_RepeatPeriod;
+	
+	// Render normals and foam to a framebuffer if we're in fancy effects
+	if (WaterMgr->m_WaterFancyEffects)
 	{
 		// Save the post-processing framebuffer.
 		GLint fbo;
 		glGetIntegerv(GL_FRAMEBUFFER_BINDING_EXT, &fbo);
-
-		pglGenFramebuffersEXT(1, &FramebufferName);
-		pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, FramebufferName);
 		
-		GLuint renderedTexture;
-		if (WaterMgr->m_waveTT == 0)
+		pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, WaterMgr->m_FancyEffectsFBO);
+
+		glDisable(GL_BLEND);
+		glEnable(GL_DEPTH_TEST);
+		glDepthFunc(GL_LEQUAL);
+		
+		glDisable(GL_CULL_FACE);
+		// Overwrite waves that would be behind the ground.
+		CShaderProgramPtr dummyShader = g_Renderer.GetShaderManager().LoadProgram("glsl/gui_solid", CShaderDefines());
+		dummyShader->Bind();
+		
+		dummyShader->Uniform(str_transform, g_Renderer.GetViewCamera().GetViewProjection());
+		dummyShader->Uniform(str_color, 0.0f, 0.0f, 0.0f, 0.0f);
+		std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
+		for (size_t i = 0; i < visiblePatches.size(); ++i)
 		{
-			glGenTextures(1, &renderedTexture);
-			WaterMgr->m_waveTT = renderedTexture;
-			
-			glBindTexture(GL_TEXTURE_2D, WaterMgr->m_waveTT);
-			// TODO: use POT texture
-			glTexImage2D(GL_TEXTURE_2D, 0,GL_RGBA, (float)g_Renderer.GetWidth(), (float)g_Renderer.GetHeight(), 0,GL_RGBA, GL_UNSIGNED_BYTE, 0);
-
+			CPatchRData* data = visiblePatches[i];
+			data->RenderWater(dummyShader, true, true);
 		}
-		glBindTexture(GL_TEXTURE_2D, WaterMgr->m_waveTT);
+		dummyShader->Unbind();
 		
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-		
-		pglFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, WaterMgr->m_waveTT, 0);
-		
-		glClearColor(0.5f,0.5f,1.0f,0.0f);
-		glClear(GL_COLOR_BUFFER_BIT);
-		
-		// rendering
-		m->wavesShader->Bind();
-		m->wavesShader->BindTexture(str_waveTex, WaterMgr->m_Wave);
-		m->wavesShader->Uniform(str_time, (float)time);
-		m->wavesShader->Uniform(str_waviness, WaterMgr->m_Waviness);
-		m->wavesShader->Uniform(str_mapSize, (float)(WaterMgr->m_TexSize));
-		
-		SWavesVertex *base=(SWavesVertex *)WaterMgr->m_VBWaves->m_Owner->Bind();
-		GLsizei stride = sizeof(SWavesVertex);
-		m->wavesShader->VertexPointer(3, GL_FLOAT, stride, &base[WaterMgr->m_VBWaves->m_Index].m_Position);
-		m->wavesShader->TexCoordPointer(GL_TEXTURE0,2,GL_BYTE, stride,&base[WaterMgr->m_VBWaves->m_Index].m_UV);
-		m->wavesShader->AssertPointersBound();
-
-		u8* indexBase = WaterMgr->m_VBWavesIndices->m_Owner->Bind();
-		glDrawElements(GL_TRIANGLES, (GLsizei) WaterMgr->m_VBWavesIndices->m_Count, GL_UNSIGNED_SHORT, indexBase + sizeof(u16)*(WaterMgr->m_VBWavesIndices->m_Index));
-
-		g_Renderer.m_Stats.m_DrawCalls++;
-		CVertexBuffer::Unbind();
-		m->wavesShader->Unbind();
-		
-		// rebind post-processing frambuffer.
+		glEnable(GL_CULL_FACE);
 		pglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo);
-		glBindTexture(GL_TEXTURE_2D, 0);
-		
 	}
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_DEPTH_TEST);
+	glDepthFunc(GL_LEQUAL);
 	
 	m->fancyWaterShader->Bind();
-
-	
-	// Shift the texture coordinates by these amounts to make the water "flow"
-	float tx = -fmod(time, 81.0 / (WaterMgr->m_Waviness/20.0 + 0.8) )/(81.0/ (WaterMgr->m_Waviness/20.0 + 0.8) );
-	float ty = -fmod(time, 34.0 / (WaterMgr->m_Waviness/20.0 + 0.8) )/(34.0/ (WaterMgr->m_Waviness/20.0 + 0.8) );
-	
-	float repeatPeriod = WaterMgr->m_RepeatPeriod;
-	
+		
 	const CCamera& camera = g_Renderer.GetViewCamera();
 	CVector3D camPos = camera.m_Orientation.GetTranslation();
 
 	m->fancyWaterShader->BindTexture(str_normalMap, WaterMgr->m_NormalMap[curTex]);
 	m->fancyWaterShader->BindTexture(str_normalMap2, WaterMgr->m_NormalMap[nexTex]);
 	
-	if (WaterMgr->m_WaterFoam || WaterMgr->m_WaterCoastalWaves)
+	if (WaterMgr->m_WaterFancyEffects)
 	{
-		m->fancyWaterShader->BindTexture(str_Foam, WaterMgr->m_Foam);
-		m->fancyWaterShader->Uniform(str_mapSize, (float)(WaterMgr->m_TexSize));
+		m->fancyWaterShader->BindTexture(str_waterEffectsTexNorm, WaterMgr->m_FancyTextureNormal);
+		m->fancyWaterShader->BindTexture(str_waterEffectsTexOther, WaterMgr->m_FancyTextureOther);
 	}
+	
 	if (WaterMgr->m_WaterRealDepth)
 		m->fancyWaterShader->BindTexture(str_depthTex, WaterMgr->m_depthTT);
-	if (WaterMgr->m_WaterCoastalWaves)
-		m->fancyWaterShader->BindTexture(str_waveTex, WaterMgr->m_waveTT);
-	if (WaterMgr->m_WaterReflection)
-	m->fancyWaterShader->BindTexture(str_reflectionMap, WaterMgr->m_ReflectionTexture);
+	
 	if (WaterMgr->m_WaterRefraction)
-	m->fancyWaterShader->BindTexture(str_refractionMap, WaterMgr->m_RefractionTexture);
+		m->fancyWaterShader->BindTexture(str_refractionMap, WaterMgr->m_RefractionTexture);
+	if (WaterMgr->m_WaterReflection)
+		m->fancyWaterShader->BindTexture(str_skyCube, g_Renderer.GetSkyManager()->GetSkyCube());
 
+	m->fancyWaterShader->BindTexture(str_reflectionMap, WaterMgr->m_ReflectionTexture);
 	m->fancyWaterShader->BindTexture(str_losMap, losTexture.GetTextureSmooth());
 
 	const CLightEnv& lightEnv = g_Renderer.GetLightEnv();
 
-	// TODO: only bind what's really needed for that.
+	m->fancyWaterShader->Uniform(str_transform, g_Renderer.GetViewCamera().GetViewProjection());
+
+	//TODO: bind only what's needed
+	if (WaterMgr->m_WaterReflection)
+	{
+		// TODO: check that this rotates in the right direction.
+		CMatrix3D skyBoxRotation;
+		skyBoxRotation.SetIdentity();
+		skyBoxRotation.RotateY(M_PI - 0.3f + lightEnv.GetRotation());
+		m->fancyWaterShader->Uniform(str_skyBoxRot, skyBoxRotation);
+	}
 	m->fancyWaterShader->Uniform(str_sunDir, lightEnv.GetSunDir());
-	m->fancyWaterShader->Uniform(str_sunColor, lightEnv.m_SunColor.X);
+	m->fancyWaterShader->Uniform(str_sunColor, lightEnv.m_SunColor);
 	m->fancyWaterShader->Uniform(str_color, WaterMgr->m_WaterColor);
-	m->fancyWaterShader->Uniform(str_specularStrength, WaterMgr->m_SpecularStrength);
+	m->fancyWaterShader->Uniform(str_tint, WaterMgr->m_WaterTint);
 	m->fancyWaterShader->Uniform(str_waviness, WaterMgr->m_Waviness);
 	m->fancyWaterShader->Uniform(str_murkiness, WaterMgr->m_Murkiness);
-	m->fancyWaterShader->Uniform(str_tint, WaterMgr->m_WaterTint);
-	m->fancyWaterShader->Uniform(str_reflectionTintStrength, WaterMgr->m_ReflectionTintStrength);
-	m->fancyWaterShader->Uniform(str_reflectionTint, WaterMgr->m_ReflectionTint);
-	m->fancyWaterShader->Uniform(str_translation, tx, ty);
+	m->fancyWaterShader->Uniform(str_windAngle, WaterMgr->m_WindAngle);
 	m->fancyWaterShader->Uniform(str_repeatScale, 1.0f / repeatPeriod);
 	m->fancyWaterShader->Uniform(str_reflectionMatrix, WaterMgr->m_ReflectionMatrix);
 	m->fancyWaterShader->Uniform(str_refractionMatrix, WaterMgr->m_RefractionMatrix);
@@ -857,8 +788,23 @@ bool TerrainRenderer::RenderFancyWater(const CShaderDefines& context, ShadowMap*
 	m->fancyWaterShader->Uniform(str_fogParams, lightEnv.m_FogFactor, lightEnv.m_FogMax, 0.f, 0.f);
 	m->fancyWaterShader->Uniform(str_time, (float)time);
 	m->fancyWaterShader->Uniform(str_screenSize, (float)g_Renderer.GetWidth(), (float)g_Renderer.GetHeight(), 0.0f, 0.0f);
-	m->fancyWaterShader->BindTexture(str_skyCube, g_Renderer.GetSkyManager()->GetSkyCube());
-
+	
+	if (WaterMgr->m_WaterType == L"clap")
+	{
+		m->fancyWaterShader->Uniform(str_waveParams1, 30.0f,1.5f,20.0f,0.03f);
+		m->fancyWaterShader->Uniform(str_waveParams2, 0.5f,0.0f,0.0f,0.0f);
+	}
+	else if (WaterMgr->m_WaterType == L"lake")
+	{
+		m->fancyWaterShader->Uniform(str_waveParams1, 8.5f,1.5f,15.0f,0.03f);
+		m->fancyWaterShader->Uniform(str_waveParams2, 0.2f,0.0f,0.0f,0.07f);
+	}
+	else
+	{
+		m->fancyWaterShader->Uniform(str_waveParams1, 15.0f,0.8f,10.0f,0.1f);
+		m->fancyWaterShader->Uniform(str_waveParams2, 0.3f,0.0f,0.1f,0.3f);
+	}
+	
 	if (shadow && WaterMgr->m_WaterShadows)
 	{
 		m->fancyWaterShader->BindTexture(str_shadowTex, shadow->GetTexture());
@@ -868,32 +814,30 @@ bool TerrainRenderer::RenderFancyWater(const CShaderDefines& context, ShadowMap*
 		m->fancyWaterShader->Uniform(str_shadowScale, width, height, 1.0f / width, 1.0f / height);
 	}
 
-	for (size_t i = 0; i < m->visiblePatches.size(); ++i)
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
+	for (size_t i = 0; i < visiblePatches.size(); ++i)
 	{
-		CPatchRData* data = m->visiblePatches[i];
+		CPatchRData* data = visiblePatches[i];
 		data->RenderWater(m->fancyWaterShader);
 	}
-
 	m->fancyWaterShader->Unbind();
 
-	pglActiveTextureARB(GL_TEXTURE0);
-	pglDeleteFramebuffersEXT(1, &FramebufferName);
-
+	glDepthFunc(GL_LEQUAL);
 	glDisable(GL_BLEND);
 
 	return true;
 }
 
-void TerrainRenderer::RenderSimpleWater()
+void TerrainRenderer::RenderSimpleWater(int cullGroup)
 {
-#if !CONFIG2_GLES
+#if CONFIG2_GLES
+	UNUSED2(cullGroup);
+#else
 	PROFILE3_GPU("simple water");
 
 	WaterManager* WaterMgr = g_Renderer.GetWaterManager();
 	CLOSTexture& losTexture = g_Game->GetView()->GetLOSTexture();
 
-	glEnable(GL_BLEND);
-	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
 	glEnable(GL_DEPTH_TEST);
 	glDepthFunc(GL_LEQUAL);
 
@@ -918,7 +862,7 @@ void TerrainRenderer::RenderSimpleWater()
 	glEnable(GL_TEXTURE_GEN_S);
 	glEnable(GL_TEXTURE_GEN_T);
 
-	// Set up texture environment to multiply vertex RGB by texture RGB and use vertex alpha
+	// Set up texture environment to multiply vertex RGB by texture RGB.
 	GLfloat waterColor[4] = { WaterMgr->m_WaterColor.r, WaterMgr->m_WaterColor.g, WaterMgr->m_WaterColor.b, 1.0f };
 	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, waterColor);
 	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
@@ -927,9 +871,7 @@ void TerrainRenderer::RenderSimpleWater()
 	glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_ARB, GL_SRC_COLOR);
 	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB_ARB, GL_CONSTANT);
 	glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB_ARB, GL_SRC_COLOR);
-	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_ARB, GL_REPLACE);
-	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_ARB, GL_PRIMARY_COLOR_ARB);
-	glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_ARB, GL_SRC_ALPHA);
+
 
 	// Multiply by LOS texture
 	losTexture.BindTexture(1);
@@ -949,23 +891,19 @@ void TerrainRenderer::RenderSimpleWater()
 	glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_ARB, GL_SRC_COLOR);
 	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB_ARB, GL_TEXTURE);
 	glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB_ARB, GL_SRC_ALPHA);
-	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_ARB, GL_REPLACE);
-	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_ARB, GL_PREVIOUS);
-	glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_ARB, GL_SRC_ALPHA);
 
 	CShaderProgramPtr dummyShader = g_Renderer.GetShaderManager().LoadProgram("fixed:dummy", CShaderDefines());
 	dummyShader->Bind();
 
 	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_COLOR_ARRAY);
 
-	for (size_t i = 0; i < m->visiblePatches.size(); ++i)
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
+	for (size_t i = 0; i < visiblePatches.size(); ++i)
 	{
-		CPatchRData* data = m->visiblePatches[i];
-		data->RenderWater(dummyShader);
+		CPatchRData* data = visiblePatches[i];
+		data->RenderWater(dummyShader, false, true);
 	}
 
-	glDisableClientState(GL_COLOR_ARRAY);
 	glDisableClientState(GL_VERTEX_ARRAY);
 
 	dummyShader->Unbind();
@@ -983,24 +921,25 @@ void TerrainRenderer::RenderSimpleWater()
 	glDisable(GL_TEXTURE_GEN_T);
 	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
-	glDisable(GL_BLEND);
 	glDisable(GL_TEXTURE_2D);
 #endif
 }
 
 ///////////////////////////////////////////////////////////////////
 // Render water that is part of the terrain
-void TerrainRenderer::RenderWater(const CShaderDefines& context, ShadowMap* shadow)
+void TerrainRenderer::RenderWater(const CShaderDefines& context, int cullGroup, ShadowMap* shadow)
 {
 	WaterManager* WaterMgr = g_Renderer.GetWaterManager();
 
+	WaterMgr->UpdateQuality();
+
 	if (!WaterMgr->WillRenderFancyWater())
-		RenderSimpleWater();
+		RenderSimpleWater(cullGroup);
 	else
-		RenderFancyWater(context, shadow);
+		RenderFancyWater(context, cullGroup, shadow);
 }
 
-void TerrainRenderer::RenderPriorities()
+void TerrainRenderer::RenderPriorities(int cullGroup)
 {
 	PROFILE("priorities");
 
@@ -1013,8 +952,9 @@ void TerrainRenderer::RenderPriorities()
 	textRenderer.Font(CStrIntern("mono-stroke-10"));
 	textRenderer.Color(1.0f, 1.0f, 0.0f);
 
-	for (size_t i = 0; i < m->visiblePatches.size(); ++i)
-		m->visiblePatches[i]->RenderPriorities(textRenderer);
+	std::vector<CPatchRData*>& visiblePatches = m->visiblePatches[cullGroup];
+	for (size_t i = 0; i < visiblePatches.size(); ++i)
+		visiblePatches[i]->RenderPriorities(textRenderer);
 
 	textRenderer.Render();
 	tech->EndPass();
