@@ -25,25 +25,19 @@
 #include "graphics/Texture.h"
 #include "lib/ogl.h"
 #include "maths/Matrix3D.h"
+#include "maths/Vector2D.h"
 #include "ps/Overlay.h"
 #include "renderer/VertexBufferManager.h"
 
 class CSimulation2;
+class CFrustum;
 
-struct SWavesVertex {
-	// vertex position
-	CVector3D m_Position;
-	u8 m_UV[2];
-};
-cassert(sizeof(SWavesVertex) == 16);
-
-
+struct CoastalPoint;
+struct WaveObject;
 
 /**
- * Class WaterManager: Maintain water settings and textures.
- *
- * This could be extended to provide more advanced water rendering effects
- * (refractive/reflective water) in the future.
+ * Class WaterManager: Maintain rendering-related water settings and textures
+ * Anything that affects gameplay should go in CcmpWaterManager.cpp and passed to this (possibly as copy).
  */
 
 class WaterManager
@@ -51,18 +45,28 @@ class WaterManager
 public:
 	CTexturePtr m_WaterTexture[60];
 	CTexturePtr m_NormalMap[60];
-	CTexturePtr m_Foam;
-	CTexturePtr m_Wave;
-	float* m_WaveX;
-	float* m_WaveZ;
-	float* m_DistanceToShore;
-	float* m_FoamFactor;
+
+	float* m_WindStrength;	// How strong the waves are at point X. % of waviness.
+	float* m_DistanceHeightmap; // How far from the shore a point is. Manhattan
+	CVector3D* m_BlurredNormalMap;	// Cache a slightly blurred map of the normals of the terrain.
+
+	// Waves vertex buffers
+	std::vector< WaveObject* > m_ShoreWaves;	// TODO: once we get C++11, remove pointer
+	// Waves indices buffer. Only one since All Wave Objects have the same.
+	CVertexBuffer::VBChunk* m_ShoreWaves_VBIndices;
 	
 	size_t m_MapSize;
 	ssize_t m_TexSize;
 
+	CTexturePtr m_WaveTex;
+	CTexturePtr m_FoamTex;
+
 	GLuint m_depthTT;
-	GLuint m_waveTT;
+	GLuint m_FancyTextureNormal;
+	GLuint m_FancyTextureOther;
+	GLuint m_FancyTextureDepth;
+	GLuint m_ReflFboDepthTexture;
+	GLuint m_RefrFboDepthTexture;
 
 	// used to know what to update when updating parts of the terrain only.
 	u32 m_updatei0;
@@ -71,35 +75,30 @@ public:
 	u32 m_updatej1;
 	
 	int m_WaterCurrentTex;
-	CColor m_WaterColor;
 	bool m_RenderWater;
 
+	// Force the use of the fixed function for rendering.
+	bool m_WaterUgly;
 	// Those variables register the current quality level. If there is a change, I have to recompile the shader.
-	bool m_WaterNormal;
+	// Use real depth or use the fake precomputed one.
 	bool m_WaterRealDepth;
-	bool m_WaterFoam;
-	bool m_WaterCoastalWaves;
+	// Use fancy shore effects and show trails behind ships
+	bool m_WaterFancyEffects;
+	// Use refractions instead of simply making the water more or less transparent.
 	bool m_WaterRefraction;
+	// Use complete reflections instead of showing merely the sky.
 	bool m_WaterReflection;
+	// Show shadows on the water.
 	bool m_WaterShadows;
 	
 	bool m_NeedsReloading;
 	// requires also recreating the super fancy information.
 	bool m_NeedInfoUpdate;
 
-	bool m_WaterScroll;
 	float m_WaterHeight;
-	float m_WaterMaxAlpha;
-	float m_WaterFullDepth;
-	float m_WaterAlphaOffset;
 
-	float m_SWaterSpeed;
-	float m_TWaterSpeed;
-	float m_SWaterTrans;
-	float m_TWaterTrans;
-	float m_SWaterScrollCounter;
-	float m_TWaterScrollCounter;
 	double m_WaterTexTimer;
+	float m_RepeatPeriod;
 
 	// Reflection and refraction textures for fancy water
 	GLuint m_ReflectionTexture;
@@ -107,25 +106,24 @@ public:
 	size_t m_ReflectionTextureSize;
 	size_t m_RefractionTextureSize;
 
+	// framebuffer objects
+	GLuint m_RefractionFbo;
+	GLuint m_ReflectionFbo;
+	GLuint m_FancyEffectsFBO;
+
 	// Model-view-projection matrices for reflected & refracted cameras
 	// (used to let the vertex shader do projective texturing)
 	CMatrix3D m_ReflectionMatrix;
 	CMatrix3D m_RefractionMatrix;
 
-	// Shader parameters for fancy water
-	CColor m_WaterTint;
-	float m_RepeatPeriod;
-	float m_SpecularStrength;
-	float m_Waviness;
-	float m_Murkiness;
-	CColor m_ReflectionTint;
-	float m_ReflectionTintStrength;
+	// Water parameters
+	std::wstring m_WaterType;		// Which texture to use.
+	CColor m_WaterColor;	// Color of the water without refractions. This is what you're seeing when the water's deep or murkiness high.
+	CColor m_WaterTint;		// Tint of refraction in the water.
+	float m_Waviness;		// How big the waves are.
+	float m_Murkiness;		// How murky the water is.
+	float m_WindAngle;	// In which direction the water waves go.
 	
-	// Waves
-	// see the struct above.
-	CVertexBuffer::VBChunk* m_VBWaves;
-	CVertexBuffer::VBChunk* m_VBWavesIndices;
-
 public:
 	WaterManager();
 	~WaterManager();
@@ -146,9 +144,24 @@ public:
 	void UnloadWaterTextures();
 
 	/**
-	 * CreateSuperfancyInfo: creates textures and wave vertices for superfancy water
+	 * RecomputeWindStrength: calculates the intensity of waves
 	 */
-	void CreateSuperfancyInfo(CSimulation2* simulation);
+	void RecomputeWindStrength();
+
+	/**
+	 * RecomputeDistanceHeightmap: recalculates (or calculates) the distance heightmap.
+	 */
+	void RecomputeDistanceHeightmap();
+
+	/**
+	 * RecomputeBlurredNormalMap: calculates the blurred normal map of the terrain. Slow.
+	 */
+	void RecomputeBlurredNormalMap();
+	
+	/**
+	 * CreateWaveMeshes: Creates the waves objects (and meshes).
+	 */
+	void CreateWaveMeshes();
 
 	/**
 	 * Updates the map size. Will trigger a complete recalculation of fancy water information the next turn.
@@ -165,6 +178,8 @@ public:
 	 * and it hasn't been configured off)
 	 */
 	bool WillRenderFancyWater();
+	
+	void RenderWaves(const CFrustum& frustrum);
 };
 
 
