@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -38,14 +38,19 @@ MESSAGE(Update_Final)
 MESSAGE(Interpolate) // non-deterministic (use with caution)
 MESSAGE(RenderSubmit) // non-deterministic (use with caution)
 MESSAGE(ProgressiveLoad) // non-deterministic (use with caution)
+MESSAGE(Deserialized) // non-deterministic (use with caution)
 MESSAGE(Create)
 MESSAGE(Destroy)
 MESSAGE(OwnershipChanged)
 MESSAGE(PositionChanged)
+MESSAGE(InterpolatedPositionChanged)
 MESSAGE(TerritoryPositionChanged)
 MESSAGE(MotionChanged)
 MESSAGE(RangeUpdate)
 MESSAGE(TerrainChanged)
+MESSAGE(VisibilityChanged)
+MESSAGE(WaterChanged)
+MESSAGE(ObstructionMapShapeChanged)
 MESSAGE(TerritoriesChanged)
 MESSAGE(PathResult)
 MESSAGE(ValueModification)
@@ -76,6 +81,9 @@ COMPONENT(CommandQueue)
 INTERFACE(Decay)
 COMPONENT(Decay)
 
+INTERFACE(Fogging)
+COMPONENT(FoggingScripted)
+
 // Note: The VisualActor component relies on this component being initialized before itself, in order to support using
 // an entity's footprint shape for the selection boxes. This dependency is not strictly necessary, but it does avoid
 // some extra plumbing code to set up on-demand initialization. If you find yourself forced to break this dependency, 
@@ -92,6 +100,9 @@ COMPONENT(IdentityScripted)
 INTERFACE(Minimap)
 COMPONENT(Minimap)
 
+INTERFACE(Mirage)
+COMPONENT(MirageScripted)
+
 INTERFACE(Motion)
 COMPONENT(MotionBall)
 COMPONENT(MotionScripted)
@@ -160,6 +171,9 @@ INTERFACE(UnitMotion)
 COMPONENT(UnitMotion) // must be after Obstruction
 COMPONENT(UnitMotionScripted)
 
+INTERFACE(UnitRenderer)
+COMPONENT(UnitRenderer)
+
 INTERFACE(Vision)
 COMPONENT(Vision)
 
