@@ -90,15 +90,19 @@ private:
 			m_Commands.clear();
 		}
 
-		bool Initialise(bool callConstructor)
+		bool Initialise()
 		{
 			// LoadScripts will only load each script once even though we call it for each player
 			if (!m_Worker.LoadScripts(m_AIName))
 				return false;
+				
+			JSContext* cx = m_ScriptInterface->GetContext();
+			JSAutoRequest rq(cx);
 
 			OsPath path = L"simulation/ai/" + m_AIName + L"/data.json";
-			CScriptValRooted metadata = m_Worker.LoadMetadata(path);
-			if (metadata.uninitialised())
+			JS::RootedValue metadata(cx);
+			m_Worker.LoadMetadata(path, &metadata);
+			if (metadata.isUndefined())
 			{
 				LOGERROR(L"Failed to create AI player: can't find %ls", path.string().c_str());
 				return false;
@@ -109,87 +113,90 @@ private:
 			// TODO: All AIs should use modules. Remove the condition if this requirement is met.
 			std::string moduleName;
 			std::string constructor;
-			CScriptVal objectWithConstructor; // object that should contain the constructor function
-			CScriptVal ctor;
-			if (!m_ScriptInterface->HasProperty(metadata.get(), "moduleName"))
+			JS::RootedValue objectWithConstructor(cx); // object that should contain the constructor function
+			JS::RootedValue global(cx, m_ScriptInterface->GetGlobalObject());
+			JS::RootedValue ctor(cx);
+			if (!m_ScriptInterface->HasProperty(metadata, "moduleName"))
 			{
-				objectWithConstructor = m_ScriptInterface->GetGlobalObject();
+				objectWithConstructor.set(m_ScriptInterface->GetGlobalObject());
 			}
 			else
 			{
-				m_ScriptInterface->GetProperty(metadata.get(), "moduleName", moduleName);
-				if(!m_ScriptInterface->GetProperty(m_ScriptInterface->GetGlobalObject(), moduleName.c_str(), objectWithConstructor) || objectWithConstructor.undefined())
+				m_ScriptInterface->GetProperty(metadata, "moduleName", moduleName);
+				if(!m_ScriptInterface->GetProperty(global, moduleName.c_str(), &objectWithConstructor) || objectWithConstructor.isUndefined())
 				{
 					LOGERROR(L"Failed to create AI player: %ls: can't find the module that should contain the constructor: '%hs'", path.string().c_str(), moduleName.c_str());
 					return false;
 				}
 			}
-			if (!m_ScriptInterface->GetProperty(metadata.get(), "constructor", constructor))
+			if (!m_ScriptInterface->GetProperty(metadata, "constructor", constructor))
 			{
 				LOGERROR(L"Failed to create AI player: %ls: missing 'constructor'", path.string().c_str());
 				return false;
 			}
 
 			// Get the constructor function from the loaded scripts
-			if (!m_ScriptInterface->GetProperty(objectWithConstructor.get(), constructor.c_str(), ctor)
-				|| ctor.undefined())
+			if (!m_ScriptInterface->GetProperty(objectWithConstructor, constructor.c_str(), &ctor)
+				|| ctor.isNull())
 			{
 				LOGERROR(L"Failed to create AI player: %ls: can't find constructor '%hs'", path.string().c_str(), constructor.c_str());
 				return false;
 			}
 
-			m_ScriptInterface->GetProperty(metadata.get(), "useShared", m_UseSharedComponent);
-			
-			CScriptVal obj;
+			m_ScriptInterface->GetProperty(metadata, "useShared", m_UseSharedComponent);
 
-			if (callConstructor)
-			{
-				// Set up the data to pass as the constructor argument
-				CScriptVal settings;
-				m_ScriptInterface->Eval(L"({})", settings);
-				m_ScriptInterface->SetProperty(settings.get(), "player", m_Player, false);
-				m_ScriptInterface->SetProperty(settings.get(), "difficulty", m_Difficulty, false);
-				ENSURE(m_Worker.m_HasLoadedEntityTemplates);
-				m_ScriptInterface->SetProperty(settings.get(), "templates", m_Worker.m_EntityTemplates, false);
+			JS::RootedValue obj(cx);
 
-				JS::AutoValueVector argv(m_ScriptInterface->GetContext());
-				argv.append(settings.get());
-				obj = m_ScriptInterface->CallConstructor(ctor.get(), argv.length(), argv.handleAt(0));
-			}
-			else
-			{
-				// For deserialization, we want to create the object with the correct prototype
-				// but don't want to actually run the constructor again
-				// XXX: actually we don't currently use this path for deserialization - maybe delete it?
-				obj = m_ScriptInterface->NewObjectFromConstructor(ctor.get());
-			}
+			// Set up the data to pass as the constructor argument
+			JS::RootedValue settings(cx);
+			m_ScriptInterface->Eval(L"({})", &settings);
+			m_ScriptInterface->SetProperty(settings, "player", m_Player, false);
+			m_ScriptInterface->SetProperty(settings, "difficulty", m_Difficulty, false);
+			ENSURE(m_Worker.m_HasLoadedEntityTemplates);
+			m_ScriptInterface->SetProperty(settings, "templates", m_Worker.m_EntityTemplates, false);
 
-			if (obj.undefined())
+			JS::AutoValueVector argv(cx);
+			argv.append(settings.get());
+			m_ScriptInterface->CallConstructor(ctor, argv, &obj);
+
+			if (obj.isNull())
 			{
 				LOGERROR(L"Failed to create AI player: %ls: error calling constructor '%hs'", path.string().c_str(), constructor.c_str());
 				return false;
 			}
 
-			m_Obj = CScriptValRooted(m_ScriptInterface->GetContext(), obj);
+			m_Obj = CScriptValRooted(cx, obj);
 			return true;
 		}
 
-		void Run(CScriptVal state, int playerID)
+		void Run(JS::HandleValue state, int playerID)
 		{
+			JSContext* cx = m_ScriptInterface->GetContext();
+			JSAutoRequest rq(cx);
+			JS::RootedValue tmpObj(cx, m_Obj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade			
+			
 			m_Commands.clear();
-			m_ScriptInterface->CallFunctionVoid(m_Obj.get(), "HandleMessage", state, playerID);
+			m_ScriptInterface->CallFunctionVoid(tmpObj, "HandleMessage", state, playerID);
 		}
 		// overloaded with a sharedAI part.
 		// javascript can handle both natively on the same function.
-		void Run(CScriptVal state, int playerID, CScriptValRooted SharedAI)
+		void Run(JS::HandleValue state, int playerID, CScriptValRooted SharedAI)
 		{
+			JSContext* cx = m_ScriptInterface->GetContext();
+			JSAutoRequest rq(cx);
+			JS::RootedValue tmpObj(cx, m_Obj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade			
+			
 			m_Commands.clear();
-			m_ScriptInterface->CallFunctionVoid(m_Obj.get(), "HandleMessage", state, playerID, SharedAI);
+			m_ScriptInterface->CallFunctionVoid(tmpObj, "HandleMessage", state, playerID, SharedAI);
 		}
-		void InitAI(CScriptVal state, CScriptValRooted SharedAI)
+		void InitAI(JS::HandleValue state, CScriptValRooted SharedAI)
 		{
+			JSContext* cx = m_ScriptInterface->GetContext();
+			JSAutoRequest rq(cx);
+			JS::RootedValue tmpObj(cx, m_Obj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade			
+			
 			m_Commands.clear();
-			m_ScriptInterface->CallFunctionVoid(m_Obj.get(), "Init", state, m_Player, SharedAI);
+			m_ScriptInterface->CallFunctionVoid(tmpObj, "Init", state, m_Player, SharedAI);
 		}
 
 		CAIWorker& m_Worker;
@@ -254,7 +261,12 @@ public:
 
 		// Load and execute *.js
 		VfsPaths pathnames;
-		vfs::GetPathnames(g_VFS, L"simulation/ai/" + moduleName + L"/", L"*.js", pathnames);
+		if (vfs::GetPathnames(g_VFS, L"simulation/ai/" + moduleName + L"/", L"*.js", pathnames) < 0)
+		{
+			LOGERROR(L"Failed to load AI scripts for module %ls", moduleName.c_str());
+			return false;
+		}
+
 		for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
 		{
 			if (!m_ScriptInterface->LoadGlobalScriptFile(*it))
@@ -274,20 +286,26 @@ public:
 		self->LoadScripts(name);
 	}
 
-	static void PostCommand(ScriptInterface::CxPrivate* pCxPrivate, int playerid, CScriptValRooted cmd)
+	static void PostCommand(ScriptInterface::CxPrivate* pCxPrivate, int playerid, CScriptValRooted cmd1)
 	{
 		ENSURE(pCxPrivate->pCBData);
 		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
+		
+		JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+		JSAutoRequest rq(cx);
+		// TODO: Get Handle parameter directly with SpiderMonkey 31
+		JS::RootedValue cmd(cx, cmd1.get());
+		
 		self->PostCommand(playerid, cmd);
 	}
 	
-	void PostCommand(int playerid, CScriptValRooted cmd)
+	void PostCommand(int playerid, JS::HandleValue cmd)
 	{
 		for (size_t i=0; i<m_Players.size(); i++)
 		{
 			if (m_Players[i]->m_Player == playerid)	
 			{
-				m_Players[i]->m_Commands.push_back(m_ScriptInterface->WriteStructuredClone(cmd.get()));
+				m_Players[i]->m_Commands.push_back(m_ScriptInterface->WriteStructuredClone(cmd));
 				return;
 			}
 		}
@@ -345,6 +363,9 @@ public:
 
 	bool TryLoadSharedComponent(bool hasTechs)
 	{
+		JSContext* cx = m_ScriptInterface->GetContext();
+		JSAutoRequest rq(cx);
+
 		// we don't need to load it.
 		if (!m_HasSharedComponent)
 			return false;
@@ -362,56 +383,59 @@ public:
 		
 		// Constructor name is SharedScript, it's in the module API3
 		// TODO: Hardcoding this is bad, we need a smarter way. 
-		CScriptVal AIModule;
-		CScriptVal ctor;
-		if (!m_ScriptInterface->GetProperty(m_ScriptInterface->GetGlobalObject(), "API3", AIModule) || AIModule.undefined())
+		JS::RootedValue AIModule(cx);
+		JS::RootedValue global(cx, m_ScriptInterface->GetGlobalObject());
+		JS::RootedValue ctor(cx);
+		if (!m_ScriptInterface->GetProperty(global, "API3", &AIModule) || AIModule.isUndefined())
 		{
 			LOGERROR(L"Failed to create shared AI component: %ls: can't find module '%hs'", path.string().c_str(), "API3");
 			return false;
 		}
 		
-		if (!m_ScriptInterface->GetProperty(AIModule.get(), "SharedScript", ctor)
-			|| ctor.undefined())
+		if (!m_ScriptInterface->GetProperty(AIModule, "SharedScript", &ctor)
+			|| ctor.isUndefined())
 		{
 			LOGERROR(L"Failed to create shared AI component: %ls: can't find constructor '%hs'", path.string().c_str(), "SharedScript");
 			return false;
 		}
 		
 		// Set up the data to pass as the constructor argument
-		CScriptVal settings;
-		m_ScriptInterface->Eval(L"({})", settings);
-		CScriptVal playersID;
-		m_ScriptInterface->Eval(L"({})", playersID);
+		JS::RootedValue settings(cx);
+		m_ScriptInterface->Eval(L"({})", &settings);
+		JS::RootedValue playersID(cx);
+		m_ScriptInterface->Eval(L"({})", &playersID);
 		
 		for (size_t i = 0; i < m_Players.size(); ++i)
 		{
-			JS::Value val;
-			m_ScriptInterface->ToJSVal(m_ScriptInterface->GetContext(), val, m_Players[i]->m_Player);
-			m_ScriptInterface->SetPropertyInt(playersID.get(), i, CScriptVal(val), true);
+			JS::RootedValue val(cx);
+			m_ScriptInterface->ToJSVal(cx, &val, m_Players[i]->m_Player);
+			m_ScriptInterface->SetPropertyInt(playersID, i, val, true);
 		}
 		
-		m_ScriptInterface->SetProperty(settings.get(), "players", playersID);
+		m_ScriptInterface->SetProperty(settings, "players", playersID);
 		ENSURE(m_HasLoadedEntityTemplates);
-		m_ScriptInterface->SetProperty(settings.get(), "templates", m_EntityTemplates, false);
+		m_ScriptInterface->SetProperty(settings, "templates", m_EntityTemplates, false);
 		
 		if (hasTechs)
 		{
-			m_ScriptInterface->SetProperty(settings.get(), "techTemplates", m_TechTemplates, false);
+			m_ScriptInterface->SetProperty(settings, "techTemplates", m_TechTemplates, false);
 		}
 		else
 		{
 			// won't get the tech templates directly.
-			CScriptVal fakeTech;
-			m_ScriptInterface->Eval("({})", fakeTech);
-			m_ScriptInterface->SetProperty(settings.get(), "techTemplates", fakeTech, false);
+			JS::RootedValue fakeTech(cx);
+			m_ScriptInterface->Eval("({})", &fakeTech);
+			m_ScriptInterface->SetProperty(settings, "techTemplates", fakeTech, false);
 		}
 		
-		JS::AutoValueVector argv(m_ScriptInterface->GetContext());
-		argv.append(settings.get());
-		m_SharedAIObj = CScriptValRooted(m_ScriptInterface->GetContext(),m_ScriptInterface->CallConstructor(ctor.get(), argv.length(), argv.handleAt(0)));
-	
+		JS::AutoValueVector argv(cx);
+		argv.append(settings);
+		// TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+		JS::RootedValue tmpSharedAIObj(cx, m_SharedAIObj.get());
+		m_ScriptInterface->CallConstructor(ctor, argv, &tmpSharedAIObj);
+		m_SharedAIObj = CScriptValRooted(cx, tmpSharedAIObj);
 		
-		if (m_SharedAIObj.undefined())
+		if (tmpSharedAIObj.isNull())
 		{
 			LOGERROR(L"Failed to create shared AI component: %ls: error calling constructor '%hs'", path.string().c_str(), "SharedScript");
 			return false;
@@ -420,10 +444,10 @@ public:
 		return true;
 	}
 
-	bool AddPlayer(const std::wstring& aiName, player_id_t player, uint8_t difficulty, bool callConstructor)
+	bool AddPlayer(const std::wstring& aiName, player_id_t player, uint8_t difficulty)
 	{
 		shared_ptr<CAIPlayer> ai(new CAIPlayer(*this, aiName, player, difficulty, m_ScriptInterface));
-		if (!ai->Initialise(callConstructor))
+		if (!ai->Initialise())
 			return false;
 		
 		// this will be set to true if we need to load the shared Component.
@@ -440,25 +464,32 @@ public:
 		// this will be run last by InitGame.Js, passing the full game representation.
 		// For now it will run for the shared Component.
 		// This is NOT run during deserialization.
-		CScriptVal state = m_ScriptInterface->ReadStructuredClone(gameState);
 		JSContext* cx = m_ScriptInterface->GetContext();
-
-		JS::RootedValue tmpVal(cx);
-		ScriptInterface::ToJSVal(cx, tmpVal.get(), passabilityMap);
+		JSAutoRequest rq(cx);
+		
+		JS::RootedValue state(cx);
+		m_ScriptInterface->ReadStructuredClone(gameState, &state);
+		
+		JS::RootedValue tmpVal(cx); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+		ScriptInterface::ToJSVal(cx, &tmpVal, passabilityMap);
 		m_PassabilityMapVal = CScriptValRooted(cx, tmpVal.get());
-		ScriptInterface::ToJSVal(cx, tmpVal.get(), territoryMap);
+		
+		ScriptInterface::ToJSVal(cx, &tmpVal, territoryMap);
 		m_TerritoryMapVal = CScriptValRooted(cx, tmpVal);
+		
 		if (m_HasSharedComponent)
 		{
-			m_ScriptInterface->SetProperty(state.get(), "passabilityMap", m_PassabilityMapVal, true);
-			m_ScriptInterface->SetProperty(state.get(), "territoryMap", m_TerritoryMapVal, true);
+			m_ScriptInterface->SetProperty(state, "passabilityMap", m_PassabilityMapVal, true);
+			m_ScriptInterface->SetProperty(state, "territoryMap", m_TerritoryMapVal, true);
 
-			m_ScriptInterface->CallFunctionVoid(m_SharedAIObj.get(), "init", state);
+			JS::RootedValue tmpSharedAIObj(cx, m_SharedAIObj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade			
+			
+			m_ScriptInterface->CallFunctionVoid(tmpSharedAIObj, "init", state);
 			
 			for (size_t i = 0; i < m_Players.size(); ++i)
 			{
 				if (m_HasSharedComponent && m_Players[i]->m_UseSharedComponent)
-					m_Players[i]->InitAI(state,m_SharedAIObj);
+					m_Players[i]->InitAI(state, m_SharedAIObj);
 			}
 		}
 		
@@ -476,8 +507,8 @@ public:
 		if (passabilityMap.m_DirtyID != m_PassabilityMap.m_DirtyID)
 		{
 			m_PassabilityMap = passabilityMap;
-			JS::RootedValue tmpVal(cx);
-			ScriptInterface::ToJSVal(cx, tmpVal.get(), m_PassabilityMap);
+			JS::RootedValue tmpVal(cx); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+			ScriptInterface::ToJSVal(cx, &tmpVal, m_PassabilityMap);
 			m_PassabilityMapVal = CScriptValRooted(cx, tmpVal);
 		}
 
@@ -485,7 +516,7 @@ public:
 		{
 			m_TerritoryMap = territoryMap;
 			JS::RootedValue tmpVal(cx);
-			ScriptInterface::ToJSVal(cx, tmpVal.get(), m_TerritoryMap);
+			ScriptInterface::ToJSVal(cx, &tmpVal, m_TerritoryMap);
 			m_TerritoryMapVal = CScriptValRooted(cx, tmpVal);
 		}
 
@@ -516,24 +547,32 @@ public:
 
 	void RegisterTechTemplates(const shared_ptr<ScriptInterface::StructuredClone>& techTemplates) {
 		JSContext* cx = m_ScriptInterface->GetContext();
-		m_TechTemplates = CScriptValRooted(cx, m_ScriptInterface->ReadStructuredClone(techTemplates));
+		JSAutoRequest rq(cx);
+		JS::RootedValue ret(cx); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+		m_ScriptInterface->ReadStructuredClone(techTemplates, &ret);
+		m_TechTemplates = CScriptValRooted(cx, ret);
 	}
 	
 	void LoadEntityTemplates(const std::vector<std::pair<std::string, const CParamNode*> >& templates)
 	{
+		JSContext* cx = m_ScriptInterface->GetContext();
+		JSAutoRequest rq(cx);
+
 		m_HasLoadedEntityTemplates = true;
 
-		m_ScriptInterface->Eval("({})", m_EntityTemplates);
+		JS::RootedValue tmpEntityTemplates(cx); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+		m_ScriptInterface->Eval("({})", &tmpEntityTemplates);
 
 		for (size_t i = 0; i < templates.size(); ++i)
 		{
-			jsval val = templates[i].second->ToJSVal(m_ScriptInterface->GetContext(), false);
-			m_ScriptInterface->SetProperty(m_EntityTemplates.get(), templates[i].first.c_str(), CScriptVal(val), true);
+			JS::RootedValue val(cx, templates[i].second->ToJSVal(cx, false));
+			m_ScriptInterface->SetProperty(tmpEntityTemplates, templates[i].first.c_str(), val, true);
 		}
 
 		// Since the template data is shared between AI players, freeze it
 		// to stop any of them changing it and confusing the other players
-		m_ScriptInterface->FreezeObject(m_EntityTemplates.get(), true);
+		m_EntityTemplates = CScriptValRooted(cx, tmpEntityTemplates);
+		m_ScriptInterface->FreezeObject(tmpEntityTemplates, true);
 	}
 
 	void Serialize(std::ostream& stream, bool isDebug)
@@ -557,6 +596,9 @@ public:
 
 	void SerializeState(ISerializer& serializer)
 	{
+		JSContext* cx = m_ScriptInterface->GetContext();
+		JSAutoRequest rq(cx);
+		
 		std::stringstream rngStream;
 		rngStream << m_RNG;
 		serializer.StringASCII("rng", rngStream.str(), 0, 32);
@@ -568,10 +610,11 @@ public:
 		serializer.Bool("useSharedScript", m_HasSharedComponent);
 		if (m_HasSharedComponent)
 		{
-			CScriptVal sharedData;
-			if (!m_ScriptInterface->CallFunction(m_SharedAIObj.get(), "Serialize", sharedData))
+			JS::RootedValue sharedData(cx);
+			JS::RootedValue tmpSharedAIObj(cx, m_SharedAIObj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade
+			if (!m_ScriptInterface->CallFunction(tmpSharedAIObj, "Serialize", &sharedData))
 				LOGERROR(L"AI shared script Serialize call failed");
-			serializer.ScriptVal("sharedData", sharedData);
+			serializer.ScriptVal("sharedData", &sharedData);
 		}
 		for (size_t i = 0; i < m_Players.size(); ++i)
 		{
@@ -582,27 +625,32 @@ public:
 			serializer.NumberU32_Unbounded("num commands", (u32)m_Players[i]->m_Commands.size());
 			for (size_t j = 0; j < m_Players[i]->m_Commands.size(); ++j)
 			{
-				CScriptVal val = m_ScriptInterface->ReadStructuredClone(m_Players[i]->m_Commands[j]);
-				serializer.ScriptVal("command", val);
+				JS::RootedValue val(cx);
+				m_ScriptInterface->ReadStructuredClone(m_Players[i]->m_Commands[j], &val);
+				serializer.ScriptVal("command", &val);
 			}
 
-			bool hasCustomSerialize = m_ScriptInterface->HasProperty(m_Players[i]->m_Obj.get(), "Serialize");
+			JS::RootedValue tmpPlayerObj(cx, m_Players[i]->m_Obj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+			bool hasCustomSerialize = m_ScriptInterface->HasProperty(tmpPlayerObj, "Serialize");
 			if (hasCustomSerialize)
 			{
-				CScriptVal scriptData;
-				if (!m_ScriptInterface->CallFunction(m_Players[i]->m_Obj.get(), "Serialize", scriptData))
+				JS::RootedValue scriptData(cx);
+				if (!m_ScriptInterface->CallFunction(tmpPlayerObj, "Serialize", &scriptData))
 					LOGERROR(L"AI script Serialize call failed");
-				serializer.ScriptVal("data", scriptData);
+				serializer.ScriptVal("data", &scriptData);
 			}
 			else
 			{
-				serializer.ScriptVal("data", m_Players[i]->m_Obj.get());
+				serializer.ScriptVal("data", &tmpPlayerObj);
 			}
 		}
 	}
 
 	void Deserialize(std::istream& stream)
 	{
+		JSContext* cx = m_ScriptInterface->GetContext();
+		JSAutoRequest rq(cx);
+		
 		ENSURE(m_CommandsComputed); // deserializing while we're still actively computing would be bad
 
 		CStdDeserializer deserializer(*m_ScriptInterface, stream);
@@ -625,9 +673,10 @@ public:
 		TryLoadSharedComponent(false);
 		if (m_HasSharedComponent)
 		{
-			CScriptVal sharedData;
-			deserializer.ScriptVal("sharedData", sharedData);
-			if (!m_ScriptInterface->CallFunctionVoid(m_SharedAIObj.get(), "Deserialize", sharedData))
+			JS::RootedValue sharedData(cx);
+			JS::RootedValue tmpSharedAIObj(cx, m_SharedAIObj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31
+			deserializer.ScriptVal("sharedData", &sharedData);
+			if (!m_ScriptInterface->CallFunctionVoid(tmpSharedAIObj, "Deserialize", sharedData))
 				LOGERROR(L"AI shared script Deserialize call failed");
 		}
 
@@ -639,7 +688,7 @@ public:
 			deserializer.String("name", name, 1, 256);
 			deserializer.NumberI32_Unbounded("player", player);
 			deserializer.NumberU8_Unbounded("difficulty",difficulty);
-			if (!AddPlayer(name, player, difficulty, true))
+			if (!AddPlayer(name, player, difficulty))
 				throw PSERROR_Deserialize_ScriptError();
 
 			uint32_t numCommands;
@@ -647,9 +696,9 @@ public:
 			m_Players.back()->m_Commands.reserve(numCommands);
 			for (size_t j = 0; j < numCommands; ++j)
 			{
-				CScriptVal val;
-				deserializer.ScriptVal("command", val);
-				m_Players.back()->m_Commands.push_back(m_ScriptInterface->WriteStructuredClone(val.get()));
+				JS::RootedValue val(cx);
+				deserializer.ScriptVal("command", &val);
+				m_Players.back()->m_Commands.push_back(m_ScriptInterface->WriteStructuredClone(val));
 			}
 			
 			// TODO: this is yucky but necessary while the AIs are sharing data between contexts;
@@ -658,24 +707,26 @@ public:
 			//	prototypes could be stored in their ScriptInterface
 			deserializer.SetSerializablePrototypes(m_DeserializablePrototypes);
 
-			bool hasCustomDeserialize = m_ScriptInterface->HasProperty(m_Players.back()->m_Obj.get(), "Deserialize");
+			JS::RootedValue tmpPlayerObj(cx, m_Players.back()->m_Obj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+			bool hasCustomDeserialize = m_ScriptInterface->HasProperty(tmpPlayerObj, "Deserialize");
 			if (hasCustomDeserialize)
 			{
-				CScriptVal scriptData;
-				deserializer.ScriptVal("data", scriptData);
+				JS::RootedValue scriptData(cx);
+				deserializer.ScriptVal("data", &scriptData);
 				if (m_Players[i]->m_UseSharedComponent)
 				{
-					if (!m_ScriptInterface->CallFunctionVoid(m_Players.back()->m_Obj.get(), "Deserialize", scriptData, m_SharedAIObj))
+					if (!m_ScriptInterface->CallFunctionVoid(tmpPlayerObj, "Deserialize", scriptData, m_SharedAIObj))
 						LOGERROR(L"AI script Deserialize call failed");
 				}
-				else if (!m_ScriptInterface->CallFunctionVoid(m_Players.back()->m_Obj.get(), "Deserialize", scriptData))
+				else if (!m_ScriptInterface->CallFunctionVoid(tmpPlayerObj, "Deserialize", scriptData))
 				{
 					LOGERROR(L"AI script deserialize() call failed");
 				}
 			}
 			else
 			{
-				deserializer.ScriptVal("data", m_Players.back()->m_Obj);
+				deserializer.ScriptVal("data", &tmpPlayerObj);
+				m_Players.back()->m_Obj = CScriptValRooted(cx, tmpPlayerObj);
 			}
 		}
 	}
@@ -697,26 +748,28 @@ public:
 	}
 
 private:
-	CScriptValRooted LoadMetadata(const VfsPath& path)
+	void LoadMetadata(const VfsPath& path, JS::MutableHandleValue out)
 	{
 		if (m_PlayerMetadata.find(path) == m_PlayerMetadata.end())
 		{
 			// Load and cache the AI player metadata
-			m_PlayerMetadata[path] = m_ScriptInterface->ReadJSONFile(path);
+			m_ScriptInterface->ReadJSONFile(path, out);
+			m_PlayerMetadata[path] = CScriptValRooted(m_ScriptInterface->GetContext(), out);
 		}
-
-		return m_PlayerMetadata[path];
+		out.set(m_PlayerMetadata[path].get());
 	}
 
 	void PerformComputation()
 	{			
 		// Deserialize the game state, to pass to the AI's HandleMessage
-		CScriptVal state;
+		JSContext* cx = m_ScriptInterface->GetContext();
+		JSAutoRequest rq(cx);
+		JS::RootedValue state(cx);
 		{
 			PROFILE3("AI compute read state");
-			state = m_ScriptInterface->ReadStructuredClone(m_GameState);
-			m_ScriptInterface->SetProperty(state.get(), "passabilityMap", m_PassabilityMapVal, true);
-			m_ScriptInterface->SetProperty(state.get(), "territoryMap", m_TerritoryMapVal, true);
+			m_ScriptInterface->ReadStructuredClone(m_GameState, &state);
+			m_ScriptInterface->SetProperty(state, "passabilityMap", m_PassabilityMapVal, true);
+			m_ScriptInterface->SetProperty(state, "territoryMap", m_TerritoryMapVal, true);
 		}
 
 		// It would be nice to do
@@ -729,7 +782,8 @@ private:
 		if (m_HasSharedComponent)
 		{
 			PROFILE3("AI run shared component");
-			m_ScriptInterface->CallFunctionVoid(m_SharedAIObj.get(), "onUpdate", state);
+			JS::RootedValue tmpSharedAIObj(cx, m_SharedAIObj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31
+			m_ScriptInterface->CallFunctionVoid(tmpSharedAIObj, "onUpdate", state);
 		}
 		
 		for (size_t i = 0; i < m_Players.size(); ++i)
@@ -852,7 +906,7 @@ public:
 
 	virtual void AddPlayer(std::wstring id, player_id_t player, uint8_t difficulty)
 	{
-		m_Worker.AddPlayer(id, player, difficulty, true);
+		m_Worker.AddPlayer(id, player, difficulty);
 
 		// AI players can cheat and see through FoW/SoD, since that greatly simplifies
 		// their implementation.
@@ -865,27 +919,32 @@ public:
 	virtual void TryLoadSharedComponent()
 	{
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
+		
 		// load the technology templates
 		CmpPtr<ICmpTechnologyTemplateManager> cmpTechTemplateManager(GetSystemEntity());
 		ENSURE(cmpTechTemplateManager);
 		
 		// Get the game state from AIInterface
-		CScriptVal techTemplates = cmpTechTemplateManager->GetAllTechs();
+		JS::RootedValue techTemplates(cx, cmpTechTemplateManager->GetAllTechs().get());
 		
-		m_Worker.RegisterTechTemplates(scriptInterface.WriteStructuredClone(techTemplates.get()));
+		m_Worker.RegisterTechTemplates(scriptInterface.WriteStructuredClone(techTemplates));
 		m_Worker.TryLoadSharedComponent(true);
 	}
 
 	virtual void RunGamestateInit()
 	{
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
 		
 		CmpPtr<ICmpAIInterface> cmpAIInterface(GetSystemEntity());
 		ENSURE(cmpAIInterface);
 		
 		// Get the game state from AIInterface
 		// We flush events from the initialization so we get a clean state now.
-		CScriptVal state = cmpAIInterface->GetFullRepresentation(true);
+		JS::RootedValue state(cx, cmpAIInterface->GetFullRepresentation(true).get());
 
 		// Get the passability data
 		Grid<u16> dummyGrid;
@@ -906,7 +965,7 @@ public:
 		
 		LoadPathfinderClasses(state);
 
-		m_Worker.RunGamestateInit(scriptInterface.WriteStructuredClone(state.get()), *passabilityMap, *territoryMap);
+		m_Worker.RunGamestateInit(scriptInterface.WriteStructuredClone(state), *passabilityMap, *territoryMap);
 	}
 
 	virtual void StartComputation()
@@ -916,6 +975,8 @@ public:
 		ForceLoadEntityTemplates();
 
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
 
 		if (m_Worker.getPlayerSize() == 0)
 			return;
@@ -924,11 +985,11 @@ public:
 		ENSURE(cmpAIInterface);
 
 		// Get the game state from AIInterface
-		CScriptVal state;
+		JS::RootedValue state(cx);
 		if (m_JustDeserialized)
-			state = cmpAIInterface->GetFullRepresentation(true);
+			state.set(cmpAIInterface->GetFullRepresentation(true).get());
 		else
-			state = cmpAIInterface->GetRepresentation();
+			state.set(cmpAIInterface->GetRepresentation().get());
 
 		// Get the passability data
 		Grid<u16> dummyGrid;
@@ -951,28 +1012,31 @@ public:
 
 		LoadPathfinderClasses(state);
 
-		m_Worker.StartComputation(scriptInterface.WriteStructuredClone(state.get()), *passabilityMap, *territoryMap, territoryMapDirty);
+		m_Worker.StartComputation(scriptInterface.WriteStructuredClone(state), *passabilityMap, *territoryMap, territoryMapDirty);
 		
 		m_JustDeserialized = false;
 	}
 
 	virtual void PushCommands()
 	{
-		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
-
 		std::vector<CAIWorker::SCommandSets> commands;
 		m_Worker.GetCommands(commands);
 
 		CmpPtr<ICmpCommandQueue> cmpCommandQueue(GetSystemEntity());
 		if (!cmpCommandQueue)
 			return;
-
+		
+		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
+		JS::RootedValue clonedCommandVal(cx);
+		
 		for (size_t i = 0; i < commands.size(); ++i)
 		{
 			for (size_t j = 0; j < commands[i].commands.size(); ++j)
 			{
-				cmpCommandQueue->PushLocalCommand(commands[i].player,
-					scriptInterface.ReadStructuredClone(commands[i].commands[j]));
+				scriptInterface.ReadStructuredClone(commands[i].commands[j], &clonedCommandVal);
+				cmpCommandQueue->PushLocalCommand(commands[i].player, CScriptVal(clonedCommandVal));
 			}
 		}
 	}
@@ -1024,22 +1088,24 @@ private:
 		}
 	}
 
-	void LoadPathfinderClasses(CScriptVal state)
+	void LoadPathfinderClasses(JS::HandleValue state)
 	{
 		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (!cmpPathfinder)
 			return;
 
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
 
-		CScriptVal classesVal;
-		scriptInterface.Eval("({ pathfinderObstruction: 1, foundationObstruction: 2 })", classesVal);
+		JS::RootedValue classesVal(cx);
+		scriptInterface.Eval("({ pathfinderObstruction: 1, foundationObstruction: 2 })", &classesVal);
 
 		std::map<std::string, ICmpPathfinder::pass_class_t> classes = cmpPathfinder->GetPassabilityClasses();
 		for (std::map<std::string, ICmpPathfinder::pass_class_t>::iterator it = classes.begin(); it != classes.end(); ++it)
-			scriptInterface.SetProperty(classesVal.get(), it->first.c_str(), it->second, true);
+			scriptInterface.SetProperty(classesVal, it->first.c_str(), it->second, true);
 
-		scriptInterface.SetProperty(state.get(), "passabilityClasses", classesVal, true);
+		scriptInterface.SetProperty(state, "passabilityClasses", classesVal, true);
 	}
 
 	CAIWorker m_Worker;
