@@ -218,6 +218,8 @@ void CCmpPathfinder::HandleMessage(const CMessage& msg, bool UNUSED(global))
 		break;
 	}
 	case MT_TerrainChanged:
+	case MT_WaterChanged:
+	case MT_ObstructionMapShapeChanged:
 	{
 		// TODO: we ought to only bother updating the dirtied region
 		m_TerrainDirty = true;
@@ -306,7 +308,7 @@ const Grid<u16>& CCmpPathfinder::GetPassabilityGrid()
 
 void CCmpPathfinder::UpdateGrid()
 {
-	CmpPtr<ICmpTerrain> cmpTerrain(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
 	if (!cmpTerrain)
 		return; // error
 
@@ -326,7 +328,7 @@ void CCmpPathfinder::UpdateGrid()
 		m_ObstructionGrid = new Grid<u8>(m_MapSize, m_MapSize);
 	}
 
-	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 
 	bool obstructionsDirty = cmpObstructionManager->Rasterise(*m_ObstructionGrid);
 
@@ -338,11 +340,6 @@ void CCmpPathfinder::UpdateGrid()
 		// Since terrain hasn't changed we only need to update the obstruction bits
 		// and can skip the rest of the data
 
-		// TODO: if ObstructionManager::SetPassabilityCircular was called at runtime
-		// (which should probably never happen, but that's not guaranteed),
-		// then TILE_OUTOFBOUNDS will change and we can't use this fast path, but
-		// currently it'll just set obstructionsDirty and we won't notice
-
 		for (u16 j = 0; j < m_MapSize; ++j)
 		{
 			for (u16 i = 0; i < m_MapSize; ++i)
@@ -372,7 +369,7 @@ void CCmpPathfinder::UpdateGrid()
 		// Obstructions or terrain changed - we need to recompute passability
 		// TODO: only bother recomputing the region that has actually changed
 
-		CmpPtr<ICmpWaterManager> cmpWaterManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpWaterManager> cmpWaterManager(GetSystemEntity());
 
 		// TOOD: these bits should come from ICmpTerrain
 		CTerrain& terrain = GetSimContext().GetTerrain();
@@ -666,7 +663,7 @@ ICmpObstruction::EFoundationCheck CCmpPathfinder::CheckUnitPlacement(const IObst
 	entity_pos_t x, entity_pos_t z, entity_pos_t r,	pass_class_t passClass, bool onlyCenterPoint)
 {
 	// Check unit obstruction
-	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 	if (!cmpObstructionManager)
 		return ICmpObstruction::FOUNDATION_CHECK_FAIL_ERROR;
 
@@ -717,7 +714,7 @@ ICmpObstruction::EFoundationCheck CCmpPathfinder::CheckBuildingPlacement(const I
 	entity_pos_t h, entity_id_t id, pass_class_t passClass, bool onlyCenterPoint)
 {
 	// Check unit obstruction
-	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 	if (!cmpObstructionManager)
 		return ICmpObstruction::FOUNDATION_CHECK_FAIL_ERROR;
 
