@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -23,6 +23,8 @@
 #include "simulation2/MessageTypes.h"
 
 #include "ICmpTerrain.h"
+#include "ICmpTerritoryManager.h"
+#include "ICmpVisual.h"
 #include "ICmpWaterManager.h"
 
 #include "graphics/Terrain.h"
@@ -32,7 +34,6 @@
 #include "maths/Vector3D.h"
 #include "maths/Vector2D.h"
 #include "ps/CLogger.h"
-#include "ICmpTerritoryManager.h"
 
 /**
  * Basic ICmpPosition implementation.
@@ -43,7 +44,9 @@ public:
 	static void ClassInit(CComponentManager& componentManager)
 	{
 		componentManager.SubscribeToMessageType(MT_TurnStart);
-		componentManager.SubscribeToMessageType(MT_Interpolate);
+		componentManager.SubscribeToMessageType(MT_TerrainChanged);
+		componentManager.SubscribeToMessageType(MT_WaterChanged);
+		componentManager.SubscribeToMessageType(MT_Deserialized);
 
 		// TODO: if this component turns out to be a performance issue, it should
 		// be optimised by creating a new PositionStatic component that doesn't subscribe
@@ -60,7 +63,7 @@ public:
 		UPRIGHT = 0,
 		PITCH = 1,
 		PITCH_ROLL = 2,
-		ROLL=3,
+		ROLL = 3,
 	} m_AnchorType;
 
 	bool m_Floating;
@@ -72,18 +75,28 @@ public:
 	// m_LastX/Z contain the position from the start of the most recent turn
 	// m_PrevX/Z conatain the position from the turn before that
 	entity_pos_t m_X, m_Z, m_LastX, m_LastZ, m_PrevX, m_PrevZ; // these values contain undefined junk if !InWorld
-	entity_pos_t m_YOffset, m_LastYOffset;
-	bool m_RelativeToGround; // whether m_YOffset is relative to terrain/water plane, or an absolute height
 
+	entity_pos_t m_Y, m_LastYDifference; // either the relative or the absolute Y coordinate
+	bool m_RelativeToGround; // whether m_Y is relative to terrain/water plane, or an absolute height
+
+	fixed m_ConstructionProgress;
+
+	// when the entity is a turret, only m_RotY is used, and this is the rotation
+	// relative to the parent entity
 	entity_angle_t m_RotX, m_RotY, m_RotZ;
 
 	player_id_t m_Territory;
 
-	// not serialized:
-	float m_InterpolatedRotX, m_InterpolatedRotY, m_InterpolatedRotZ;
-	float m_LastInterpolatedRotX, m_LastInterpolatedRotZ; // not serialized
+	entity_id_t m_TurretParent;
+	CFixedVector3D m_TurretPosition;
+	std::set<entity_id_t> m_Turrets;
 
-	bool m_NeedInitialXZRotation;
+	// Not serialized:
+	float m_InterpolatedRotX, m_InterpolatedRotY, m_InterpolatedRotZ;
+	float m_LastInterpolatedRotX, m_LastInterpolatedRotZ;
+	bool m_ActorFloating;
+
+	bool m_EnabledMessageInterpolate;
 
 	static std::string GetSchema()
 	{
@@ -128,7 +141,8 @@ public:
 
 		m_InWorld = false;
 
-		m_LastYOffset = m_YOffset = paramNode.GetChild("Altitude").ToFixed();
+		m_LastYDifference = entity_pos_t::Zero();
+		m_Y = paramNode.GetChild("Altitude").ToFixed();
 		m_RelativeToGround = true;
 		m_Floating = paramNode.GetChild("Floating").ToBool();
 
@@ -139,7 +153,12 @@ public:
 		m_LastInterpolatedRotX = m_LastInterpolatedRotZ = 0.f;
 		m_Territory = INVALID_PLAYER;
 
-		m_NeedInitialXZRotation = false;
+		m_TurretParent = INVALID_ENTITY;
+		m_TurretPosition = CFixedVector3D();
+
+		m_ActorFloating = false;
+
+		m_EnabledMessageInterpolate = false;
 	}
 
 	virtual void Deinit()
@@ -152,16 +171,20 @@ public:
 		if (m_InWorld)
 		{
 			serialize.NumberFixed_Unbounded("x", m_X);
+			serialize.NumberFixed_Unbounded("y", m_Y);
 			serialize.NumberFixed_Unbounded("z", m_Z);
 			serialize.NumberFixed_Unbounded("last x", m_LastX);
+			serialize.NumberFixed_Unbounded("last y diff", m_LastYDifference);
 			serialize.NumberFixed_Unbounded("last z", m_LastZ);
 		}
 		serialize.NumberI32_Unbounded("territory", m_Territory);
 		serialize.NumberFixed_Unbounded("rot x", m_RotX);
 		serialize.NumberFixed_Unbounded("rot y", m_RotY);
 		serialize.NumberFixed_Unbounded("rot z", m_RotZ);
-		serialize.NumberFixed_Unbounded("altitude", m_YOffset);
+		serialize.NumberFixed_Unbounded("altitude", m_Y);
 		serialize.Bool("relative", m_RelativeToGround);
+		serialize.Bool("floating", m_Floating);
+		serialize.NumberFixed_Unbounded("constructionprogress", m_ConstructionProgress);
 
 		if (serialize.IsDebug())
 		{
@@ -186,7 +209,13 @@ public:
 				break;
 			}
 			serialize.StringASCII("anchor", anchor, 0, 16);
-			serialize.Bool("floating", m_Floating);
+		}
+		serialize.NumberU32_Unbounded("turret parent", m_TurretParent);
+		if (m_TurretParent != INVALID_ENTITY)
+		{
+			serialize.NumberFixed_Unbounded("x", m_TurretPosition.X);
+			serialize.NumberFixed_Unbounded("y", m_TurretPosition.Y);
+			serialize.NumberFixed_Unbounded("z", m_TurretPosition.Z);
 		}
 	}
 
@@ -198,23 +227,100 @@ public:
 		if (m_InWorld)
 		{
 			deserialize.NumberFixed_Unbounded("x", m_X);
+			deserialize.NumberFixed_Unbounded("y", m_Y);
 			deserialize.NumberFixed_Unbounded("z", m_Z);
 			deserialize.NumberFixed_Unbounded("last x", m_LastX);
+			deserialize.NumberFixed_Unbounded("last y diff", m_LastYDifference);
 			deserialize.NumberFixed_Unbounded("last z", m_LastZ);
 		}
 		deserialize.NumberI32_Unbounded("territory", m_Territory);
 		deserialize.NumberFixed_Unbounded("rot x", m_RotX);
 		deserialize.NumberFixed_Unbounded("rot y", m_RotY);
 		deserialize.NumberFixed_Unbounded("rot z", m_RotZ);
-		deserialize.NumberFixed_Unbounded("altitude", m_YOffset);
+		deserialize.NumberFixed_Unbounded("altitude", m_Y);
 		deserialize.Bool("relative", m_RelativeToGround);
+		deserialize.Bool("floating", m_Floating);
+		deserialize.NumberFixed_Unbounded("constructionprogress", m_ConstructionProgress);
 		// TODO: should there be range checks on all these values?
 
 		m_InterpolatedRotY = m_RotY.ToFloat();
-		m_LastYOffset = m_YOffset;
+
+		deserialize.NumberU32_Unbounded("turret parent", m_TurretParent);
+		if (m_TurretParent != INVALID_ENTITY)
+		{
+			deserialize.NumberFixed_Unbounded("x", m_TurretPosition.X);
+			deserialize.NumberFixed_Unbounded("y", m_TurretPosition.Y);
+			deserialize.NumberFixed_Unbounded("z", m_TurretPosition.Z);
+		}
 
 		if (m_InWorld)
 			UpdateXZRotation();
+
+		UpdateMessageSubscriptions();
+	}
+
+	void Deserialized()
+	{
+		AdvertiseInterpolatedPositionChanges();
+	}
+
+	virtual void UpdateTurretPosition()
+	{
+		if (m_TurretParent == INVALID_ENTITY)
+			return;
+		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), m_TurretParent);
+		if (!cmpPosition)
+		{
+			LOGERROR(L"Turret with parent without position component");
+			return;
+		}
+		if (!cmpPosition->IsInWorld())
+			MoveOutOfWorld();
+		else
+		{
+			CFixedVector2D rotatedPosition = CFixedVector2D(m_TurretPosition.X, m_TurretPosition.Z);
+			rotatedPosition = rotatedPosition.Rotate(cmpPosition->GetRotation().Y);
+			CFixedVector2D rootPosition = cmpPosition->GetPosition2D();
+			entity_pos_t x = rootPosition.X + rotatedPosition.X;
+			entity_pos_t z = rootPosition.Y + rotatedPosition.Y;
+			if (!m_InWorld || m_X != x || m_Z != z)
+				MoveTo(x, z);
+			entity_pos_t y = cmpPosition->GetHeightOffset() + m_TurretPosition.Y;
+			if (!m_InWorld || GetHeightOffset() != y)
+				SetHeightOffset(y);
+			m_InWorld = true;
+		}
+	}
+
+	virtual std::set<entity_id_t>* GetTurrets()
+	{
+		return &m_Turrets;
+	}
+
+	virtual void SetTurretParent(entity_id_t id, CFixedVector3D offset)
+	{
+		if (m_TurretParent != INVALID_ENTITY)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), m_TurretParent);
+			if (cmpPosition)
+				cmpPosition->GetTurrets()->erase(GetEntityId());
+		}
+
+		m_TurretParent = id;
+		m_TurretPosition = offset;
+
+		if (m_TurretParent != INVALID_ENTITY)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), m_TurretParent);
+			if (cmpPosition)
+				cmpPosition->GetTurrets()->insert(GetEntityId());
+		}
+		UpdateTurretPosition();
+	}
+
+	virtual entity_id_t GetTurretParent()
+	{
+		return m_TurretParent;
 	}
 
 	virtual bool IsInWorld()
@@ -227,6 +333,7 @@ public:
 		m_InWorld = false;
 
 		AdvertisePositionChanges();
+		AdvertiseInterpolatedPositionChanges();
 	}
 
 	virtual void MoveTo(entity_pos_t x, entity_pos_t z)
@@ -239,27 +346,30 @@ public:
 			m_InWorld = true;
 			m_LastX = m_PrevX = m_X;
 			m_LastZ = m_PrevZ = m_Z;
-			m_LastYOffset = m_YOffset;
+			m_LastYDifference = entity_pos_t::Zero();
 		}
 
 		AdvertisePositionChanges();
+		AdvertiseInterpolatedPositionChanges();
 	}
 	
 	virtual void MoveAndTurnTo(entity_pos_t x, entity_pos_t z, entity_angle_t ry)
 	{
 		m_X = x;
 		m_Z = z;
-		m_RotY = ry;
 		
 		if (!m_InWorld)
 		{
 			m_InWorld = true;
 			m_LastX = m_PrevX = m_X;
 			m_LastZ = m_PrevZ = m_Z;
-			m_LastYOffset = m_YOffset;
+			m_LastYDifference = entity_pos_t::Zero();
 		}
 		
-		AdvertisePositionChanges();
+		// TurnTo will advertise the position changes
+		TurnTo(ry);
+
+		AdvertiseInterpolatedPositionChanges();
 	}
 
 	virtual void JumpTo(entity_pos_t x, entity_pos_t z)
@@ -274,41 +384,75 @@ public:
 		m_LastInterpolatedRotZ = m_InterpolatedRotZ;
 
 		AdvertisePositionChanges();
+		AdvertiseInterpolatedPositionChanges();
 	}
 
 	virtual void SetHeightOffset(entity_pos_t dy)
 	{
-		if (m_RelativeToGround)
-		{
-			m_LastYOffset = m_YOffset;
-			m_YOffset = dy;
-		}
-		else 
-		{
-			m_LastYOffset = m_YOffset = dy;
-			m_RelativeToGround = true;
-		}
-
-		AdvertisePositionChanges();
+		// subtract the offset and replace with a new offset
+		m_LastYDifference = dy - GetHeightOffset();
+		m_Y += m_LastYDifference;
+		AdvertiseInterpolatedPositionChanges();
 	}
 
 	virtual entity_pos_t GetHeightOffset()
 	{
-		return m_YOffset;
+		if (m_RelativeToGround)
+			return m_Y;
+		// not relative to the ground, so the height offset is m_Y - ground height
+		entity_pos_t baseY;
+		CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
+		if (cmpTerrain)
+			baseY = cmpTerrain->GetGroundLevel(m_X, m_Z);
+
+		if (m_Floating)
+		{
+			CmpPtr<ICmpWaterManager> cmpWaterManager(GetSystemEntity());
+			if (cmpWaterManager)
+				baseY = std::max(baseY, cmpWaterManager->GetWaterLevel(m_X, m_Z));
+		}
+		return m_Y - baseY;
 	}
 
 	virtual void SetHeightFixed(entity_pos_t y)
 	{
-		if (m_RelativeToGround)
+		// subtract the absolute height and replace it with a new absolute height
+		m_LastYDifference = y - GetHeightFixed();
+		m_Y += m_LastYDifference;
+		AdvertiseInterpolatedPositionChanges();
+	}
+
+	virtual entity_pos_t GetHeightFixed()
+	{
+		if (!m_RelativeToGround)
+			return m_Y;
+		// relative to the ground, so the fixed height = ground height + m_Y
+		entity_pos_t baseY;
+		CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
+		if (cmpTerrain)
+			baseY = cmpTerrain->GetGroundLevel(m_X, m_Z);
+
+		if (m_Floating)
 		{
-			m_LastYOffset = m_YOffset = y;
-			m_RelativeToGround = false;
-		}
-		else
-		{
-			m_LastYOffset = m_YOffset;
-			m_YOffset = y;	
+			CmpPtr<ICmpWaterManager> cmpWaterManager(GetSystemEntity());
+			if (cmpWaterManager)
+				baseY = std::max(baseY, cmpWaterManager->GetWaterLevel(m_X, m_Z));
 		}
+		return m_Y + baseY;
+	}
+
+	virtual bool IsHeightRelative()
+	{
+		return m_RelativeToGround;
+	}
+
+	virtual void SetHeightRelative(bool relative)
+	{
+		// move y to use the right offset (from terrain or from map origin)
+		m_Y = relative ? GetHeightOffset() : GetHeightFixed();
+		m_RelativeToGround = relative;
+		m_LastYDifference = entity_pos_t::Zero();
+		AdvertiseInterpolatedPositionChanges();
 	}
 
 	virtual bool IsFloating()
@@ -316,6 +460,24 @@ public:
 		return m_Floating;
 	}
 
+	virtual void SetFloating(bool flag)
+	{
+		m_Floating = flag;
+		AdvertiseInterpolatedPositionChanges();
+	}
+
+	virtual void SetActorFloating(bool flag)
+	{
+		m_ActorFloating = flag;
+		AdvertiseInterpolatedPositionChanges();
+	}
+
+	virtual void SetConstructionProgress(fixed progress)
+	{
+		m_ConstructionProgress = progress;
+		AdvertiseInterpolatedPositionChanges();
+	}
+
 	virtual CFixedVector3D GetPosition()
 	{
 		if (!m_InWorld)
@@ -324,22 +486,7 @@ public:
 			return CFixedVector3D();
 		}
 
-		entity_pos_t baseY;
-		if (m_RelativeToGround)
-		{
-			CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
-			if (cmpTerrain)
-				baseY = cmpTerrain->GetGroundLevel(m_X, m_Z);
-
-			if (m_Floating)
-			{
-				CmpPtr<ICmpWaterManager> cmpWaterManager(GetSystemEntity());
-				if (cmpWaterManager)
-					baseY = std::max(baseY, cmpWaterManager->GetWaterLevel(m_X, m_Z));
-			}
-		}
-
-		return CFixedVector3D(m_X, baseY + m_YOffset, m_Z);
+		return CFixedVector3D(m_X, GetHeightFixed(), m_Z);
 	}
 
 	virtual CFixedVector2D GetPosition2D()
@@ -361,22 +508,7 @@ public:
 			return CFixedVector3D(); 
 		} 
 
-		entity_pos_t baseY; 
-		if (m_RelativeToGround) 
-		{ 
-			CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
-			if (cmpTerrain) 
-				baseY = cmpTerrain->GetGroundLevel(m_PrevX, m_PrevZ); 
-
-			if (m_Floating) 
-			{ 
-				CmpPtr<ICmpWaterManager> cmpWaterMan(GetSystemEntity());
-				if (cmpWaterMan) 
-					baseY = std::max(baseY, cmpWaterMan->GetWaterLevel(m_PrevX, m_PrevZ)); 
-			} 
-		} 
-
-		return CFixedVector3D(m_PrevX, baseY + m_YOffset, m_PrevZ); 
+		return CFixedVector3D(m_PrevX, GetHeightFixed(), m_PrevZ); 
 	} 
 
 	virtual CFixedVector2D GetPreviousPosition2D() 
@@ -392,13 +524,26 @@ public:
 
 	virtual void TurnTo(entity_angle_t y)
 	{
+		if (m_TurretParent != INVALID_ENTITY)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), m_TurretParent);
+			if (cmpPosition)
+				y -= cmpPosition->GetRotation().Y;
+		}
 		m_RotY = y;
 
 		AdvertisePositionChanges();
+		UpdateMessageSubscriptions();
 	}
 
 	virtual void SetYRotation(entity_angle_t y)
 	{
+		if (m_TurretParent != INVALID_ENTITY)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), m_TurretParent);
+			if (cmpPosition)
+				y -= cmpPosition->GetRotation().Y;
+		}
 		m_RotY = y;
 		m_InterpolatedRotY = m_RotY.ToFloat();
 
@@ -411,6 +556,7 @@ public:
 		}
 
 		AdvertisePositionChanges();
+		UpdateMessageSubscriptions();
 	}
 
 	virtual void SetXZRotation(entity_angle_t x, entity_angle_t z)
@@ -425,13 +571,18 @@ public:
 			m_LastInterpolatedRotX = m_InterpolatedRotX;
 			m_LastInterpolatedRotZ = m_InterpolatedRotZ;
 		}
-
-		AdvertisePositionChanges();
 	}
 
 	virtual CFixedVector3D GetRotation()
 	{
-		return CFixedVector3D(m_RotX, m_RotY, m_RotZ);
+		entity_angle_t y = m_RotY;
+		if (m_TurretParent != INVALID_ENTITY)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), m_TurretParent);
+			if (cmpPosition)
+				y += cmpPosition->GetRotation().Y;
+		}
+		return CFixedVector3D(m_RotX, y, m_RotZ);
 	}
 
 	virtual fixed GetDistanceTravelled()
@@ -445,6 +596,36 @@ public:
 		return CFixedVector2D(m_X - m_LastX, m_Z - m_LastZ).Length();
 	}
 
+	float GetConstructionProgressOffset(const CVector3D& pos)
+	{
+		if (m_ConstructionProgress.IsZero())
+			return 0.0f;
+
+		CmpPtr<ICmpVisual> cmpVisual(GetEntityHandle());
+		if (!cmpVisual)
+			return 0.0f;
+
+		// We use selection boxes to calculate the model size, since the model could be offset
+		// TODO: this annoyingly shows decals, would be nice to hide them
+		CBoundingBoxOriented bounds = cmpVisual->GetSelectionBox();
+		if (bounds.IsEmpty())
+			return 0.0f;
+
+		float dy = 2.0f * bounds.m_HalfSizes.Y;
+
+		// If this is a floating unit, we want it to start all the way under the terrain,
+		// so find the difference between its current position and the terrain
+
+		CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
+		if (cmpTerrain && (m_Floating || m_ActorFloating))
+		{
+			float ground = cmpTerrain->GetExactGroundLevel(pos.X, pos.Z);
+			dy += std::max(0.f, pos.Y - ground);
+		}
+
+		return (m_ConstructionProgress.ToFloat() - 1.0f) * dy;
+	}
+
 	virtual void GetInterpolatedPosition2D(float frameOffset, float& x, float& z, float& rotY)
 	{
 		if (!m_InWorld)
@@ -459,8 +640,34 @@ public:
 		rotY = m_InterpolatedRotY;
 	}
 
-	virtual CMatrix3D GetInterpolatedTransform(float frameOffset, bool forceFloating)
+	virtual CMatrix3D GetInterpolatedTransform(float frameOffset)
 	{
+		if (m_TurretParent != INVALID_ENTITY)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), m_TurretParent);
+			if (!cmpPosition)
+			{
+				LOGERROR(L"Turret with parent without position component");
+				CMatrix3D m;
+				m.SetIdentity();
+				return m;
+			}
+			if (!cmpPosition->IsInWorld())
+			{
+				LOGERROR(L"CCmpPosition::GetInterpolatedTransform called on turret entity when IsInWorld is false");
+				CMatrix3D m;
+				m.SetIdentity();
+				return m;
+			}
+			else
+			{
+				CMatrix3D parentTransformMatrix = cmpPosition->GetInterpolatedTransform(frameOffset);
+				CMatrix3D ownTransformation = CMatrix3D();
+				ownTransformation.SetYRotation(m_InterpolatedRotY);
+				ownTransformation.Translate(-m_TurretPosition.X.ToFloat(), m_TurretPosition.Y.ToFloat(), -m_TurretPosition.Z.ToFloat());
+				return parentTransformMatrix * ownTransformation;
+			}
+		}
 		if (!m_InWorld)
 		{
 			LOGERROR(L"CCmpPosition::GetInterpolatedTransform called on entity when IsInWorld is false");
@@ -472,6 +679,7 @@ public:
 		float x, z, rotY;
 		GetInterpolatedPosition2D(frameOffset, x, z, rotY);
 
+	
 		float baseY = 0;
 		if (m_RelativeToGround)
 		{
@@ -479,7 +687,7 @@ public:
 			if (cmpTerrain)
 				baseY = cmpTerrain->GetExactGroundLevel(x, z);
 
-			if (m_Floating || forceFloating)
+			if (m_Floating || m_ActorFloating)
 			{
 				CmpPtr<ICmpWaterManager> cmpWaterManager(GetSystemEntity());
 				if (cmpWaterManager)
@@ -487,27 +695,71 @@ public:
 			}
 		}
 
-		float y = baseY + Interpolate(m_LastYOffset.ToFloat(), m_YOffset.ToFloat(), frameOffset);
+		float y = baseY + m_Y.ToFloat() + Interpolate(m_LastYDifference.ToFloat(), 0.f, frameOffset);
 
 		CMatrix3D m;
-		
+
 		// linear interpolation is good enough (for RotX/Z). 
 		// As you always stay close to zero angle.	
 		m.SetXRotation(Interpolate(m_LastInterpolatedRotX, m_InterpolatedRotX, frameOffset));
 		m.RotateZ(Interpolate(m_LastInterpolatedRotZ, m_InterpolatedRotZ, frameOffset));
-	
+
+		CVector3D pos(x, y, z);
+
+		pos.Y += GetConstructionProgressOffset(pos);
+
 		m.RotateY(rotY + (float)M_PI);
-		m.Translate(CVector3D(x, y, z));
-		
+		m.Translate(pos);
+
 		return m;
 	}
 
+	void GetInterpolatedPositions(CVector3D& pos0, CVector3D& pos1)
+	{
+		float baseY0 = 0;
+		float baseY1 = 0;
+		float x0 = m_LastX.ToFloat();
+		float z0 = m_LastZ.ToFloat();
+		float x1 = m_X.ToFloat();
+		float z1 = m_Z.ToFloat();
+		if (m_RelativeToGround)
+		{
+			CmpPtr<ICmpTerrain> cmpTerrain(GetSimContext(), SYSTEM_ENTITY);
+			if (cmpTerrain)
+			{
+				baseY0 = cmpTerrain->GetExactGroundLevel(x0, z0);
+				baseY1 = cmpTerrain->GetExactGroundLevel(x1, z1);
+			}
+
+			if (m_Floating || m_ActorFloating)
+			{
+				CmpPtr<ICmpWaterManager> cmpWaterManager(GetSimContext(), SYSTEM_ENTITY);
+				if (cmpWaterManager)
+				{
+					baseY0 = std::max(baseY0, cmpWaterManager->GetExactWaterLevel(x0, z0));
+					baseY1 = std::max(baseY1, cmpWaterManager->GetExactWaterLevel(x1, z1));
+				}
+			}
+		}
+
+		float y0 = baseY0 + m_Y.ToFloat() + m_LastYDifference.ToFloat();
+		float y1 = baseY1 + m_Y.ToFloat();
+
+		pos0 = CVector3D(x0, y0, z0);
+		pos1 = CVector3D(x1, y1, z1);
+
+		pos0.Y += GetConstructionProgressOffset(pos0);
+		pos1.Y += GetConstructionProgressOffset(pos1);
+	}
+
 	virtual void HandleMessage(const CMessage& msg, bool UNUSED(global))
 	{
 		switch (msg.GetType())
 		{
 		case MT_Interpolate:
 		{
+			PROFILE3("Position::Interpolate");
+
 			const CMessageInterpolate& msgData = static_cast<const CMessageInterpolate&> (msg);
 
 			float rotY = m_RotY.ToFloat();
@@ -533,21 +785,15 @@ public:
 
 					UpdateXZRotation();
 				}
-			}
-			
-			if (m_InWorld && m_NeedInitialXZRotation)
-			{
-				// the terrain probably wasn't loaded last time we tried, so update the XZ rotation without interpolation
-				UpdateXZRotation();
 
-				m_LastInterpolatedRotX = m_InterpolatedRotX;
-				m_LastInterpolatedRotZ = m_InterpolatedRotZ;
+				UpdateMessageSubscriptions();
 			}
 
 			break;
 		}
 		case MT_TurnStart:
 		{
+			
 			m_LastInterpolatedRotX = m_InterpolatedRotX;
 			m_LastInterpolatedRotZ = m_InterpolatedRotZ;
 
@@ -560,7 +806,7 @@ public:
 
 			m_LastX = m_X;
 			m_LastZ = m_Z;
-			m_LastYOffset = m_YOffset;
+			m_LastYDifference = entity_pos_t::Zero();
 
 
 			// warn when a position change also causes a territory change under the entity
@@ -586,12 +832,56 @@ public:
 			}
 			break;
 		}
+		case MT_TerrainChanged:
+		case MT_WaterChanged:
+		{
+			AdvertiseInterpolatedPositionChanges();
+			break;
+		}
+		case MT_Deserialized:
+		{
+			Deserialized();
+			break;
+		}
+		}
 	}
-};
 
 private:
+
+	/*
+	 * Must be called whenever m_RotY or m_InterpolatedRotY change,
+	 * to determine whether we need to call Interpolate to make the unit rotate.
+	 */
+	void UpdateMessageSubscriptions()
+	{
+		bool needInterpolate = false;
+
+		float rotY = m_RotY.ToFloat();
+		if (rotY != m_InterpolatedRotY)
+			needInterpolate = true;
+
+		if (needInterpolate != m_EnabledMessageInterpolate)
+		{
+			GetSimContext().GetComponentManager().DynamicSubscriptionNonsync(MT_Interpolate, this, needInterpolate);
+			m_EnabledMessageInterpolate = needInterpolate;
+		}
+	}
+
+	/**
+	 * This must be called after changing anything that will affect the
+	 * return value of GetPosition2D() or GetRotation().Y:
+	 *  - m_InWorld
+	 *  - m_X, m_Z
+	 *  - m_RotY
+	 */
 	void AdvertisePositionChanges()
 	{
+		for (std::set<entity_id_t>::const_iterator it = m_Turrets.begin(); it != m_Turrets.end(); ++it)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), *it);
+			if (cmpPosition)
+				cmpPosition->UpdateTurretPosition();
+		}
 		if (m_InWorld)
 		{
 			CMessagePositionChanged msg(GetEntityId(), true, m_X, m_Z, m_RotY);
@@ -604,6 +894,33 @@ private:
 		}
 	}
 
+	/**
+	 * This must be called after changing anything that will affect the
+	 * return value of GetInterpolatedPositions():
+	 *  - m_InWorld
+	 *  - m_X, m_Z
+	 *  - m_LastX, m_LastZ
+	 *  - m_Y, m_LastYDifference, m_RelativeToGround
+	 *  - If m_RelativeToGround, then the ground under this unit
+	 *  - If m_RelativeToGround && m_Float, then the water level
+	 */
+	void AdvertiseInterpolatedPositionChanges()
+	{
+		if (m_InWorld)
+		{
+			CVector3D pos0, pos1;
+			GetInterpolatedPositions(pos0, pos1);
+
+			CMessageInterpolatedPositionChanged msg(GetEntityId(), true, pos0, pos1);
+			GetSimContext().GetComponentManager().PostMessage(GetEntityId(), msg);
+		}
+		else
+		{
+			CMessageInterpolatedPositionChanged msg(GetEntityId(), false, CVector3D(), CVector3D());
+			GetSimContext().GetComponentManager().PostMessage(GetEntityId(), msg);
+		}
+	}
+
 	void UpdateXZRotation()
 	{
 		if (!m_InWorld)
@@ -623,8 +940,7 @@ private:
 		CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
 		if (!cmpTerrain || !cmpTerrain->IsLoaded())
 		{
-			// try again when terrain is loaded
-			m_NeedInitialXZRotation = true;
+			LOGERROR(L"Terrain not loaded");
 			return;
 		}
 
@@ -644,10 +960,6 @@ private:
 
 		if (m_AnchorType == ROLL || m_AnchorType == PITCH_ROLL)
 			m_InterpolatedRotZ = atan2(normal.X, normal.Y);
-
-		m_NeedInitialXZRotation = false;
-		
-		return;
 	}
 };
 
