@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -23,6 +23,9 @@
 #include "ICmpTerrain.h"
 #include "simulation2/system/EntityMap.h"
 #include "simulation2/MessageTypes.h"
+#include "simulation2/components/ICmpFogging.h"
+#include "simulation2/components/ICmpMirage.h"
+#include "simulation2/components/ICmpOwnership.h"
 #include "simulation2/components/ICmpPosition.h"
 #include "simulation2/components/ICmpTerritoryManager.h"
 #include "simulation2/components/ICmpVision.h"
@@ -39,7 +42,7 @@
 #include "renderer/Scene.h"
 #include "lib/ps_stl.h"
 
-
+#define LOS_TILES_RATIO 8
 #define DEBUG_RANGE_MANAGER_BOUNDS 0
 
 /**
@@ -135,16 +138,17 @@ static std::map<entity_id_t, EntityParabolicRangeOutline> ParabolicRangesOutline
  */
 struct EntityData
 {
-	EntityData() : retainInFog(0), owner(-1), inWorld(0), flags(1) { }
+	EntityData() : visibilities(0), retainInFog(0), owner(-1), inWorld(0), flags(1) { }
 	entity_pos_t x, z;
 	entity_pos_t visionRange;
+	u32 visibilities; // 2-bit visibility, per player
 	u8 retainInFog; // boolean
 	i8 owner;
 	u8 inWorld; // boolean
 	u8 flags; // See GetEntityFlagMask
 };
 
-cassert(sizeof(EntityData) == 16);
+cassert(sizeof(EntityData) == 20);
 
 /**
  * Serialization helper template for Query
@@ -196,6 +200,7 @@ struct SerializeEntityData
 		serialize.NumberFixed_Unbounded("x", value.x);
 		serialize.NumberFixed_Unbounded("z", value.z);
 		serialize.NumberFixed_Unbounded("vision", value.visionRange);
+		serialize.NumberU32_Unbounded("visibilities", value.visibilities);
 		serialize.NumberU8("retain in fog", value.retainInFog, 0, 1);
 		serialize.NumberI8_Unbounded("owner", value.owner);
 		serialize.NumberU8("in world", value.inWorld, 0, 1);
@@ -282,7 +287,14 @@ public:
 	std::vector<bool> m_LosRevealAll;
 	bool m_LosCircular;
 	i32 m_TerrainVerticesPerSide;
-	size_t m_TerritoriesDirtyID;
+	
+	// Cache for visibility tracking (not serialized)
+	i32 m_LosTilesPerSide;
+	bool m_GlobalVisibilityUpdate;
+	std::vector<u8> m_DirtyVisibility;
+	std::vector<std::set<entity_id_t> > m_LosTiles;
+	// List of entities that must be updated, regardless of the status of their tile
+	std::vector<entity_id_t> m_ModifiedEntities;
 
 	// Counts of units seeing vertex, per vertex, per player (starting with player 0).
 	// Use u16 to avoid overflows when we have very large (but not infeasibly large) numbers
@@ -326,12 +338,11 @@ public:
 		// The whole map should be visible to Gaia by default, else e.g. animals
 		// will get confused when trying to run from enemies
 		m_LosRevealAll.resize(MAX_LOS_PLAYER_ID+2,false);
+		m_LosRevealAll[0] = true;
 		m_SharedLosMasks.resize(MAX_LOS_PLAYER_ID+2,0);
 
 		m_LosCircular = false;
 		m_TerrainVerticesPerSide = 0;
-
-		m_TerritoriesDirtyID = 0;
 	}
 
 	virtual void Deinit()
@@ -354,7 +365,9 @@ public:
 		serialize.Bool("los circular", m_LosCircular);
 		serialize.NumberI32_Unbounded("terrain verts per side", m_TerrainVerticesPerSide);
 
-		// We don't serialize m_Subdivision or m_LosPlayerCounts
+		SerializeVector<SerializeU32_Unbounded>()(serialize, "modified entities", m_ModifiedEntities);
+
+		// We don't serialize m_Subdivision, m_LosPlayerCounts or m_LosTiles
 		// since they can be recomputed from the entity data when deserializing;
 		// m_LosState must be serialized since it depends on the history of exploration
 
@@ -431,12 +444,20 @@ public:
 					CFixedVector2D to(msgData.x, msgData.z);
 					m_Subdivision.Move(ent, from, to);
 					LosMove(it->second.owner, it->second.visionRange, from, to);
+					i32 oldLosTile = PosToLosTilesHelper(it->second.x, it->second.z);
+					i32 newLosTile = PosToLosTilesHelper(msgData.x, msgData.z);
+					if (oldLosTile != newLosTile)
+					{
+						RemoveFromTile(oldLosTile, ent);
+						AddToTile(newLosTile, ent);
+					}
 				}
 				else
 				{
 					CFixedVector2D to(msgData.x, msgData.z);
 					m_Subdivision.Add(ent, to);
 					LosAdd(it->second.owner, it->second.visionRange, to);
+					AddToTile(PosToLosTilesHelper(msgData.x, msgData.z), ent);
 				}
 
 				it->second.inWorld = 1;
@@ -450,6 +471,7 @@ public:
 					CFixedVector2D from(it->second.x, it->second.z);
 					m_Subdivision.Remove(ent, from);
 					LosRemove(it->second.owner, it->second.visionRange, from);
+					RemoveFromTile(PosToLosTilesHelper(it->second.x, it->second.z), ent);
 				}
 
 				it->second.inWorld = 0;
@@ -457,6 +479,8 @@ public:
 				it->second.z = entity_pos_t::Zero();
 			}
 
+			m_ModifiedEntities.push_back(ent);
+
 			break;
 		}
 		case MT_OwnershipChanged:
@@ -494,7 +518,10 @@ public:
 				break;
 
 			if (it->second.inWorld)
+			{
 				m_Subdivision.Remove(ent, CFixedVector2D(it->second.x, it->second.z));
+				RemoveFromTile(PosToLosTilesHelper(it->second.x, it->second.z), ent);
+			}
 
 			// This will be called after Ownership's OnDestroy, so ownership will be set
 			// to -1 already and we don't have to do a LosRemove here
@@ -538,8 +565,8 @@ public:
 		case MT_Update:
 		{
 			m_DebugOverlayDirty = true;
-			UpdateTerritoriesLos();
 			ExecuteActiveQueries();
+			UpdateVisibilityData();
 			break;
 		}
 		case MT_RenderSubmit:
@@ -574,6 +601,7 @@ public:
 		std::vector<std::vector<u16> > oldPlayerCounts = m_LosPlayerCounts;
 		std::vector<u32> oldStateRevealed = m_LosStateRevealed;
 		SpatialSubdivision oldSubdivision = m_Subdivision;
+		std::vector<std::set<entity_id_t> > oldLosTiles = m_LosTiles;
 
 		ResetDerivedData(true);
 
@@ -599,6 +627,8 @@ public:
 			debug_warn(L"inconsistent revealed");
 		if (oldSubdivision != m_Subdivision)
 			debug_warn(L"inconsistent subdivs");
+		if (oldLosTiles != m_LosTiles)
+			debug_warn(L"inconsistent los tiles");
 	}
 
 	SpatialSubdivision* GetSubdivision()
@@ -612,6 +642,8 @@ public:
 		ENSURE(m_WorldX0.IsZero() && m_WorldZ0.IsZero()); // don't bother implementing non-zero offsets yet
 		ResetSubdivisions(m_WorldX1, m_WorldZ1);
 
+		m_LosTilesPerSide = (m_TerrainVerticesPerSide - 1)/LOS_TILES_RATIO;
+
 		m_LosPlayerCounts.clear();
 		m_LosPlayerCounts.resize(MAX_LOS_PLAYER_ID+1);
 		m_ExploredVertices.clear();
@@ -638,11 +670,20 @@ public:
 		}
 		m_LosStateRevealed.clear();
 		m_LosStateRevealed.resize(m_TerrainVerticesPerSide*m_TerrainVerticesPerSide);
+		
+		m_DirtyVisibility.clear();
+		m_DirtyVisibility.resize(m_LosTilesPerSide*m_LosTilesPerSide);
+		m_LosTiles.clear();
+		m_LosTiles.resize(m_LosTilesPerSide*m_LosTilesPerSide);
+		m_GlobalVisibilityUpdate = true;
 
 		for (EntityMap<EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
 		{
 			if (it->second.inWorld)
+			{
 				LosAdd(it->second.owner, it->second.visionRange, CFixedVector2D(it->second.x, it->second.z));
+				AddToTile(PosToLosTilesHelper(it->second.x, it->second.z), it->first);
+			}
 		}
 
 		m_TotalInworldVertices = 0;
@@ -939,10 +980,10 @@ public:
 			CFixedVector3D pos3d = cmpSourcePosition->GetPosition()+
 			    CFixedVector3D(entity_pos_t::Zero(), q.elevationBonus, entity_pos_t::Zero()) ;
 			// Get a quick list of entities that are potentially in range, with a cutoff of 2*maxRange
-			SpatialQueryArray ents;
+			std::vector<entity_id_t> ents;
 			m_Subdivision.GetNear(ents, pos, q.maxRange*2);
 
-			for (int i = 0; i < ents.size(); ++i)
+			for (size_t i = 0; i < ents.size(); ++i)
 			{
 				EntityMap<EntityData>::const_iterator it = m_EntityData.find(ents[i]);
 				ENSURE(it != m_EntityData.end());
@@ -976,10 +1017,10 @@ public:
 		else
 		{
 			// Get a quick list of entities that are potentially in range
-			SpatialQueryArray ents;
+			std::vector<entity_id_t> ents;
 			m_Subdivision.GetNear(ents, pos, q.maxRange);
 
-			for (int i = 0; i < ents.size(); ++i)
+			for (size_t i = 0; i < ents.size(); ++i)
 			{
 				EntityMap<EntityData>::const_iterator it = m_EntityData.find(ents[i]);
 				ENSURE(it != m_EntityData.end());
@@ -1336,7 +1377,9 @@ public:
 
 	virtual ELosVisibility GetLosVisibility(CEntityHandle ent, player_id_t player, bool forceRetainInFog)
 	{
-		// (We can't use m_EntityData since this needs to handle LOCAL entities too)
+		// This function provides the real visibility of any entity (even local ones) at any time.
+		// The m_EntityData visibility is updated at most once per turn with this function's return value
+		// and must not be used for rendering
 
 		// Entities not with positions in the world are never visible
 		if (ent.GetId() == INVALID_ENTITY)
@@ -1345,8 +1388,12 @@ public:
 		if (!cmpPosition || !cmpPosition->IsInWorld())
 			return VIS_HIDDEN;
 
-		CFixedVector2D pos = cmpPosition->GetPosition2D();
+		// Mirage entities, whatever the situation, are visible for one specific player
+		CmpPtr<ICmpMirage> cmpMirage(ent);
+		if (cmpMirage && cmpMirage->GetPlayer() != player)
+			return VIS_HIDDEN;
 
+		CFixedVector2D pos = cmpPosition->GetPosition2D();
 		int i = (pos.X / (int)TERRAIN_TILE_SIZE).ToInt_RoundToNearest();
 		int j = (pos.Y / (int)TERRAIN_TILE_SIZE).ToInt_RoundToNearest();
 
@@ -1365,15 +1412,39 @@ public:
 		if (los.IsVisible(i, j))
 			return VIS_VISIBLE;
 
+		if (!los.IsExplored(i, j))			
+			return VIS_HIDDEN;
+			
 		// Fogged if the 'retain in fog' flag is set, and in a non-visible explored region
-		if (los.IsExplored(i, j))
-		{
-			CmpPtr<ICmpVision> cmpVision(ent);
-			if (forceRetainInFog || (cmpVision && cmpVision->GetRetainInFog()))
-				return VIS_FOGGED;
-		}
+		CmpPtr<ICmpVision> cmpVision(ent);
+		if (!forceRetainInFog && !(cmpVision && cmpVision->GetRetainInFog()))
+			return VIS_HIDDEN;
 
-		// Otherwise not visible
+		if (cmpMirage && cmpMirage->GetPlayer() == player)
+			return VIS_FOGGED;
+
+		CmpPtr<ICmpOwnership> cmpOwnership(ent);
+		if (!cmpOwnership)
+			return VIS_VISIBLE;
+		
+		if (cmpOwnership->GetOwner() == player)
+		{
+			CmpPtr<ICmpFogging> cmpFogging(ent);
+			if (!cmpFogging)
+				return VIS_VISIBLE;
+			
+			// Fogged entities must not disappear while the mirage is not ready
+			if (!cmpFogging->IsMiraged(player))
+				return VIS_FOGGED;
+
+			return VIS_HIDDEN;
+		}
+	
+		// Fogged entities must not disappear while the mirage is not ready
+		CmpPtr<ICmpFogging> cmpFogging(ent);
+		if (cmpFogging && cmpFogging->WasSeen(player) && !cmpFogging->IsMiraged(player))
+			return VIS_FOGGED;
+				
 		return VIS_HIDDEN;
 	}
 
@@ -1383,6 +1454,104 @@ public:
 		return GetLosVisibility(handle, player, forceRetainInFog);
 	}
 
+	i32 PosToLosTilesHelper(entity_pos_t x, entity_pos_t z)
+	{
+		i32 i = Clamp(
+			(x/(entity_pos_t::FromInt(TERRAIN_TILE_SIZE * LOS_TILES_RATIO))).ToInt_RoundToZero(),
+			0,
+			m_LosTilesPerSide - 1);
+		i32 j = Clamp(
+			(z/(entity_pos_t::FromInt(TERRAIN_TILE_SIZE * LOS_TILES_RATIO))).ToInt_RoundToZero(),
+			0,
+			m_LosTilesPerSide - 1);
+		return j*m_LosTilesPerSide + i;
+	}
+
+	void AddToTile(i32 tile, entity_id_t ent)
+	{
+		m_LosTiles[tile].insert(ent);
+	}
+
+	void RemoveFromTile(i32 tile, entity_id_t ent)
+	{
+		for (std::set<entity_id_t>::iterator tileIt = m_LosTiles[tile].begin();
+			tileIt != m_LosTiles[tile].end();
+			++tileIt)
+		{
+			if (*tileIt == ent)
+			{
+				m_LosTiles[tile].erase(tileIt);
+				return;
+			}
+		}
+	}
+
+	void UpdateVisibilityData()
+	{
+		PROFILE("UpdateVisibilityData");
+		
+		for (i32 n = 0; n < m_LosTilesPerSide*m_LosTilesPerSide; ++n)
+		{
+			if (m_DirtyVisibility[n] == 1 || m_GlobalVisibilityUpdate)
+			{
+				for (std::set<entity_id_t>::iterator it = m_LosTiles[n].begin();
+					it != m_LosTiles[n].end();
+					++it)
+				{
+					UpdateVisibility(*it);
+				}
+				m_DirtyVisibility[n] = 0;
+			}
+		}
+
+		for (std::vector<entity_id_t>::iterator it = m_ModifiedEntities.begin(); it != m_ModifiedEntities.end(); ++it)
+		{
+			UpdateVisibility(*it);
+		}
+		m_ModifiedEntities.clear();
+
+		m_GlobalVisibilityUpdate = false;
+	}
+
+	void UpdateVisibility(entity_id_t ent)
+	{
+		// Warning: Code related to fogging (like posting VisibilityChanged messages) 
+		// shouldn't be invoked while keeping an iterator to an element of m_EntityData.
+		// Otherwise, by deleting mirage entities and so on, 
+		// that code will change the indexes in the map, leading to segfaults. 
+		EntityMap<EntityData>::iterator itEnts = m_EntityData.find(ent);
+		if (itEnts == m_EntityData.end())
+			return;
+
+		// So we just remember what visibilities to update and do that later.
+		std::vector<u8> oldVisibilities;
+		std::vector<u8> newVisibilities;
+		
+		for (player_id_t player = 1; player < MAX_LOS_PLAYER_ID+1; ++player)
+		{
+			u8 oldVis = (itEnts->second.visibilities >> (2*(player-1))) & 0x3;
+			u8 newVis = GetLosVisibility(itEnts->first, player, false);
+			
+			oldVisibilities.push_back(oldVis);
+			newVisibilities.push_back(newVis);
+
+			if (oldVis != newVis)
+				itEnts->second.visibilities = (itEnts->second.visibilities & ~(0x3 << 2*(player-1))) | (newVis << 2*(player-1));
+		}
+
+		for (player_id_t player = 1; player < MAX_LOS_PLAYER_ID+1; ++player)
+		{
+			if (oldVisibilities[player-1] == newVisibilities[player-1])
+				continue;
+			
+			// Another visibility update can be necessary to take in account new mirages
+			if (std::find(m_ModifiedEntities.begin(), m_ModifiedEntities.end(), ent) == m_ModifiedEntities.end())
+				m_ModifiedEntities.push_back(ent);
+
+			CMessageVisibilityChanged msg(player, ent, oldVisibilities[player-1], newVisibilities[player-1]);
+			GetSimContext().GetComponentManager().PostMessage(ent, msg);
+		}
+	}
 
 	virtual void SetLosRevealAll(player_id_t player, bool enabled)
 	{
@@ -1393,6 +1562,9 @@ public:
 			ENSURE(player >= 0 && player <= MAX_LOS_PLAYER_ID);
 			m_LosRevealAll[player] = enabled;
 		}
+
+		// On next update, update the visibility of every entity in the world
+		m_GlobalVisibilityUpdate = true;
 	}
 
 	virtual bool GetLosRevealAll(player_id_t player)
@@ -1443,13 +1615,13 @@ public:
 				m_LosState[i + j*m_TerrainVerticesPerSide] |= (LOS_EXPLORED << (2*(p-1)));
 			}
 		}
+
+		SeeExploredEntities(p);
 	}
 
-	void UpdateTerritoriesLos()
+	virtual void ExploreTerritories()
 	{
 		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSystemEntity());
-		if (!cmpTerritoryManager || !cmpTerritoryManager->NeedUpdate(&m_TerritoriesDirtyID))
-			return;
 
 		const Grid<u8>& grid = cmpTerritoryManager->GetTerritoryGrid();
 		ENSURE(grid.m_W == m_TerrainVerticesPerSide-1 && grid.m_H == m_TerrainVerticesPerSide-1);
@@ -1476,6 +1648,50 @@ public:
 				}
 			}
 		}
+
+		for (player_id_t p = 1; p < MAX_LOS_PLAYER_ID+1; ++p)
+			SeeExploredEntities(p);
+	}
+
+	/**
+	 * Force any entity in explored territory to appear for player p.
+	 * This is useful for miraging entities inside the territory borders at the beginning of a game,
+	 * or if the "Explore Map" option has been set.
+	 */
+	void SeeExploredEntities(player_id_t p)
+	{
+		// Warning: Code related to fogging (like ForceMiraging) shouldn't be
+		// invoked while iterating through m_EntityData.
+		// Otherwise, by deleting mirage entities and so on, that code will 
+		// change the indexes in the map, leading to segfaults. 
+		// So we just remember what entities to mirage and do that later.
+		std::vector<entity_id_t> miragableEntities;
+		
+		for (EntityMap<EntityData>::iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), it->first);
+			if (!cmpPosition || !cmpPosition->IsInWorld())
+				continue;
+
+			CFixedVector2D pos = cmpPosition->GetPosition2D();
+			int i = (pos.X / (int)TERRAIN_TILE_SIZE).ToInt_RoundToNearest();
+			int j = (pos.Y / (int)TERRAIN_TILE_SIZE).ToInt_RoundToNearest();
+
+			CLosQuerier los(GetSharedLosMask(p), m_LosState, m_TerrainVerticesPerSide);
+			if (!los.IsExplored(i,j) || los.IsVisible(i,j))
+				continue;
+
+			CmpPtr<ICmpFogging> cmpFogging(GetSimContext(), it->first);
+			if (cmpFogging)
+				miragableEntities.push_back(it->first);
+		}
+
+		for (std::vector<entity_id_t>::iterator it = miragableEntities.begin(); it != miragableEntities.end(); ++it)
+		{
+			CmpPtr<ICmpFogging> cmpFogging(GetSimContext(), *it);
+			ENSURE(cmpFogging && "Impossible to retrieve Fogging component, previously achieved");
+			cmpFogging->ForceMiraging(p);
+		}
 	}
 
 	/**
@@ -1531,6 +1747,22 @@ public:
 					explored += !(m_LosState[idx] & (LOS_EXPLORED << (2*(owner-1))));
 					m_LosState[idx] |= ((LOS_VISIBLE | LOS_EXPLORED) << (2*(owner-1)));
 				}
+
+				// Mark the LoS tiles around the updated vertex
+				// 1: left-up, 2: right-up, 3: left-down, 4: right-down
+				int n1 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
+				int n2 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
+				int n3 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
+				int n4 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
+
+				if (j > 0 && i > 0)
+					m_DirtyVisibility[n1] = 1;
+				if (n2 != n1 && j > 0 && i < m_TerrainVerticesPerSide)
+					m_DirtyVisibility[n2] = 1;
+				if (n3 != n1 && j < m_TerrainVerticesPerSide && i > 0)
+					m_DirtyVisibility[n3] = 1;
+				if (n4 != n1 && j < m_TerrainVerticesPerSide && i < m_TerrainVerticesPerSide)
+					m_DirtyVisibility[n4] = 1;
 			}
 
 			ASSERT(counts[idx] < 65535);
@@ -1558,6 +1790,24 @@ public:
 			{
 				// (If LosIsOffWorld then this is a no-op, so don't bother doing the check)
 				m_LosState[idx] &= ~(LOS_VISIBLE << (2*(owner-1)));
+
+				i32 i = i0 + idx - idx0;
+
+				// Mark the LoS tiles around the updated vertex
+				// 1: left-up, 2: right-up, 3: left-down, 4: right-down
+				int n1 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
+				int n2 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
+				int n3 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
+				int n4 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
+
+				if (j > 0 && i > 0)
+					m_DirtyVisibility[n1] = 1;
+				if (n2 != n1 && j > 0 && i < m_TerrainVerticesPerSide)
+					m_DirtyVisibility[n2] = 1;
+				if (n3 != n1 && j < m_TerrainVerticesPerSide && i > 0)
+					m_DirtyVisibility[n3] = 1;
+				if (n4 != n1 && j < m_TerrainVerticesPerSide && i < m_TerrainVerticesPerSide)
+					m_DirtyVisibility[n4] = 1;
 			}
 		}
 	}
