@@ -136,15 +136,20 @@ public:
 		}
 		
 		if (ReloadWater && CRenderer::IsInitialised())
+		{
 			g_Renderer.GetWaterManager()->SetMapSize(vertices);
-
+			g_Renderer.GetWaterManager()->RecomputeBlurredNormalMap();
+			g_Renderer.GetWaterManager()->RecomputeDistanceHeightmap();
+			g_Renderer.GetWaterManager()->RecomputeWindStrength();
+			g_Renderer.GetWaterManager()->CreateWaveMeshes();
+		}
 		MakeDirty(0, 0, tiles+1, tiles+1);
 	}
 
 	virtual void MakeDirty(i32 i0, i32 j0, i32 i1, i32 j1)
 	{
 		CMessageTerrainChanged msg(i0, j0, i1, j1);
-		GetSimContext().GetComponentManager().PostMessage(GetEntityId(), msg);
+		GetSimContext().GetComponentManager().BroadcastMessage(msg);
 	}
 };
 
