@@ -68,6 +68,7 @@ public:
 		componentManager.SubscribeGloballyToMessageType(MT_PositionChanged);
 		componentManager.SubscribeGloballyToMessageType(MT_ValueModification);
 		componentManager.SubscribeToMessageType(MT_TerrainChanged);
+		componentManager.SubscribeToMessageType(MT_WaterChanged);
 		componentManager.SubscribeToMessageType(MT_Update);
 		componentManager.SubscribeToMessageType(MT_Interpolate);
 		componentManager.SubscribeToMessageType(MT_RenderSubmit);
@@ -173,6 +174,7 @@ public:
 			break;
 		}
 		case MT_TerrainChanged:
+		case MT_WaterChanged:
 		{
 			MakeDirty();
 			break;
