@@ -116,7 +116,6 @@ public:
 	{
 		componentManager.SubscribeToMessageType(MT_Update_MotionFormation);
 		componentManager.SubscribeToMessageType(MT_Update_MotionUnit);
-		componentManager.SubscribeToMessageType(MT_RenderSubmit); // for debug overlays
 		componentManager.SubscribeToMessageType(MT_PathResult);
 		componentManager.SubscribeToMessageType(MT_ValueModification);
 	}
@@ -133,6 +132,7 @@ public:
 	fixed m_WalkSpeed, m_OriginalWalkSpeed; // in metres per second
 	fixed m_RunSpeed, m_OriginalRunSpeed;
 	ICmpPathfinder::pass_class_t m_PassClass;
+	std::string m_PassClassName;
 	ICmpPathfinder::cost_class_t m_CostClass;
 
 	// Dynamic state:
@@ -306,7 +306,8 @@ public:
 		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (cmpPathfinder)
 		{
-			m_PassClass = cmpPathfinder->GetPassabilityClass(paramNode.GetChild("PassabilityClass").ToUTF8());
+			m_PassClassName = paramNode.GetChild("PassabilityClass").ToUTF8();
+			m_PassClass = cmpPathfinder->GetPassabilityClass(m_PassClassName);
 			m_CostClass = cmpPathfinder->GetCostClass(paramNode.GetChild("CostClass").ToUTF8());
 		}
 
@@ -338,6 +339,8 @@ public:
 		serialize.NumberU8("state", m_State, 0, STATE_MAX-1);
 		serialize.NumberU8("path state", m_PathState, 0, PATHSTATE_MAX-1);
 
+		serialize.StringASCII("pass class", m_PassClassName, 0, 64);
+
 		serialize.NumberU32_Unbounded("ticket", m_ExpectedPathTicket);
 
 		serialize.NumberU32_Unbounded("target entity", m_TargetEntity);
@@ -369,6 +372,10 @@ public:
 		Init(paramNode);
 
 		SerializeCommon(deserialize);
+
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
+		if (cmpPathfinder)
+			m_PassClass = cmpPathfinder->GetPassabilityClass(m_PassClassName);
 	}
 
 	virtual void HandleMessage(const CMessage& msg, bool UNUSED(global))
@@ -395,6 +402,7 @@ public:
 		}
 		case MT_RenderSubmit:
 		{
+			PROFILE3("UnitMotion::RenderSubmit");
 			const CMessageRenderSubmit& msgData = static_cast<const CMessageRenderSubmit&> (msg);
 			RenderSubmit(msgData.collector);
 			break;
@@ -428,6 +436,12 @@ public:
 		}
 	}
 
+	void UpdateMessageSubscriptions()
+	{
+		bool needRender = m_DebugOverlayEnabled;
+		GetSimContext().GetComponentManager().DynamicSubscriptionNonsync(MT_RenderSubmit, this, needRender);
+	}
+
 	virtual bool IsMoving()
 	{
 		return m_Moving;
@@ -448,6 +462,19 @@ public:
 		return m_PassClass;
 	}
 
+	virtual std::string GetPassabilityClassName()
+	{
+		return m_PassClassName;
+	}
+
+	virtual void SetPassabilityClassName(std::string passClassName)
+	{
+		m_PassClassName = passClassName;
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
+		if (cmpPathfinder)
+			m_PassClass = cmpPathfinder->GetPassabilityClass(passClassName);
+	}
+
 	virtual fixed GetCurrentSpeed()
 	{
 		return m_CurSpeed;
@@ -466,6 +493,7 @@ public:
 	virtual void SetDebugOverlay(bool enabled)
 	{
 		m_DebugOverlayEnabled = enabled;
+		UpdateMessageSubscriptions();
 	}
 
 	virtual bool MoveToPointRange(entity_pos_t x, entity_pos_t z, entity_pos_t minRange, entity_pos_t maxRange);
