@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -60,7 +60,7 @@ CBinarySerializerScriptImpl::CBinarySerializerScriptImpl(ScriptInterface& script
 {
 }
 
-void CBinarySerializerScriptImpl::HandleScriptVal(jsval val)
+void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 {
 	JSContext* cx = m_ScriptInterface.GetContext();
 	JSAutoRequest rq(cx);
@@ -119,7 +119,8 @@ void CBinarySerializerScriptImpl::HandleScriptVal(jsval val)
 
 			// Now handle its array buffer
 			// this may be a backref, since ArrayBuffers can be shared by multiple views
-			HandleScriptVal(JS::ObjectValue(*JS_GetArrayBufferViewBuffer(obj)));
+			JS::RootedValue bufferVal(cx, JS::ObjectValue(*JS_GetArrayBufferViewBuffer(obj)));
+			HandleScriptVal(bufferVal);
 			break;
 		}
 		else if (JS_IsArrayBufferObject(obj))
@@ -182,10 +183,10 @@ void CBinarySerializerScriptImpl::HandleScriptVal(jsval val)
 						// If serialize is null, so don't serialize anything more
 						if (!serialize.isNull())
 						{
-							CScriptValRooted data;
-							if (!m_ScriptInterface.CallFunction(val, "Serialize", data))
+							JS::RootedValue data(cx);
+							if (!m_ScriptInterface.CallFunction(val, "Serialize", &data))
 								throw PSERROR_Serialize_ScriptError("Prototype Serialize function failed");
-							HandleScriptVal(data.get());
+							HandleScriptVal(data);
 						}
 						break;
 					}
@@ -222,6 +223,38 @@ void CBinarySerializerScriptImpl::HandleScriptVal(jsval val)
 				m_Serializer.Bool("value", b);
 				break;
 			}
+			else if (protokey == JSProto_Map)
+			{
+				// TODO: There's no C++ API (yet) to work with maps. This code relies on the internal 
+				// structure of the Iterator object returned by Map.entries(). This is not ideal
+				// because the structure could change in the future (and actually does change with v31).
+				// Change this code if SpiderMonkey gets such an API.
+				u32 mapSize;
+				m_ScriptInterface.GetProperty(val, "size", mapSize);
+				
+				m_Serializer.NumberU8_Unbounded("type", SCRIPT_TYPE_OBJECT_MAP);
+				m_Serializer.NumberU32_Unbounded("map size", mapSize);
+				
+				JS::RootedValue keyValueIterator(cx);
+				m_ScriptInterface.CallFunction(val, "entries", &keyValueIterator);
+				for (u32 i=0; i<mapSize; ++i)
+				{
+					JS::RootedValue keyValuePair(cx);
+					ENSURE(m_ScriptInterface.CallFunction(keyValueIterator, "next", &keyValuePair));
+					
+					JS::RootedObject keyValuePairObj(cx, &keyValuePair.toObject());
+					
+					JS::RootedValue key(cx);
+					JS::RootedValue value(cx);
+					ENSURE(JS_GetElement(cx, keyValuePairObj, 0, key.address()));
+					ENSURE(JS_GetElement(cx, keyValuePairObj, 1, value.address()));
+					
+					HandleScriptVal(key);
+					HandleScriptVal(value);
+				}
+				
+				break;
+			}
 			else
 			{
 				// Unrecognized class
