@@ -58,17 +58,17 @@ public:
 	CBinarySerializerScriptImpl(ScriptInterface& scriptInterface, ISerializer& serializer);
 
 	void ScriptString(const char* name, JSString* string);
-	void HandleScriptVal(jsval val);
+	void HandleScriptVal(JS::HandleValue val);
 	void SetSerializablePrototypes(std::map<JSObject*, std::wstring>& prototypes);
 private:
 	ScriptInterface& m_ScriptInterface;
 	ISerializer& m_Serializer;
 
 	// Pooling helps since we do a lot of short-lived allocations
-	typedef ProxyAllocator<std::pair<JSObject* const, u32>, Allocators::Arena<> > ScriptBackrefsAlloc;
+	typedef ProxyAllocator<std::pair<JSObject* const, u32>, Allocators::DynamicArena> ScriptBackrefsAlloc;
 	typedef std::map<JSObject*, u32, std::less<JSObject*>, ScriptBackrefsAlloc> backrefs_t;
 
-	Allocators::Arena<> m_ScriptBackrefsArena;
+	Allocators::DynamicArena m_ScriptBackrefsArena;
 	backrefs_t m_ScriptBackrefs;
 	u32 m_ScriptBackrefsNext;
 	u32 GetScriptBackrefTag(JSObject* obj);
@@ -185,7 +185,7 @@ protected:
 		m_Impl.Put(name, (u8*)value.data(), value.length());
 	}
 
-	virtual void PutScriptVal(const char* UNUSED(name), jsval value)
+	virtual void PutScriptVal(const char* UNUSED(name), JS::MutableHandleValue value)
 	{
 		m_ScriptImpl->HandleScriptVal(value);
 	}
