@@ -31,10 +31,8 @@ public:
 	CStdDeserializer(ScriptInterface& scriptInterface, std::istream& stream);
 	virtual ~CStdDeserializer();
 
-	virtual void ScriptVal(const char* name, jsval& out);
-	virtual void ScriptVal(const char* name, CScriptVal& out);
-	virtual void ScriptVal(const char* name, CScriptValRooted& out);
-	virtual void ScriptObjectAppend(const char* name, jsval& obj);
+	virtual void ScriptVal(const char* name, JS::MutableHandleValue out);
+	virtual void ScriptObjectAppend(const char* name, JS::HandleValue objVal);
 	virtual void ScriptString(const char* name, JSString*& out);
 
 	virtual std::istream& GetStream();
@@ -46,7 +44,7 @@ protected:
 	virtual void Get(const char* name, u8* data, size_t len);
 
 private:
-	jsval ReadScriptVal(const char* name, JSObject* appendParent);
+	jsval ReadScriptVal(const char* name, JS::HandleObject appendParent);
 	void ReadStringUTF16(const char* name, utf16string& str);
 
 	virtual void AddScriptBackref(JSObject* obj);
