@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -19,6 +19,7 @@
 
 #include "ComponentManager.h"
 
+#include "DynamicSubscription.h"
 #include "IComponent.h"
 #include "ParamNode.h"
 #include "SimContext.h"
@@ -41,8 +42,8 @@ public:
 	virtual const char* GetScriptGlobalHandlerName() const { return globalHandlerName.c_str(); }
 	virtual jsval ToJSVal(ScriptInterface& UNUSED(scriptInterface)) const { return msg.get(); }
 
-	CMessageScripted(int mtid, const std::string& name, const CScriptValRooted& msg) :
-		mtid(mtid), handlerName("On" + name), globalHandlerName("OnGlobal" + name), msg(msg)
+	CMessageScripted(ScriptInterface& scriptInterface, int mtid, const std::string& name, JS::HandleValue msg) :
+		mtid(mtid), handlerName("On" + name), globalHandlerName("OnGlobal" + name), msg(scriptInterface.GetContext(), msg)
 	{
 	}
 
@@ -70,6 +71,8 @@ CComponentManager::CComponentManager(CSimContext& context, shared_ptr<ScriptRunt
 	if (!skipScriptFunctions)
 	{
 		m_ScriptInterface.RegisterFunction<void, int, std::string, CScriptVal, CComponentManager::Script_RegisterComponentType> ("RegisterComponentType");
+		m_ScriptInterface.RegisterFunction<void, int, std::string, CScriptVal, CComponentManager::Script_RegisterSystemComponentType> ("RegisterSystemComponentType");
+		m_ScriptInterface.RegisterFunction<void, int, std::string, CScriptVal, CComponentManager::Script_ReRegisterComponentType> ("ReRegisterComponentType");
 		m_ScriptInterface.RegisterFunction<void, std::string, CComponentManager::Script_RegisterInterface> ("RegisterInterface");
 		m_ScriptInterface.RegisterFunction<void, std::string, CComponentManager::Script_RegisterMessageType> ("RegisterMessageType");
 		m_ScriptInterface.RegisterFunction<void, std::string, CScriptVal, CComponentManager::Script_RegisterGlobal> ("RegisterGlobal");
@@ -137,17 +140,20 @@ bool CComponentManager::LoadScript(const VfsPath& filename, bool hotload)
 	m_CurrentlyHotloading = hotload;
 	CVFSFile file;
 	PSRETURN loadOk = file.Load(g_VFS, filename);
-	ENSURE(loadOk == PSRETURN_OK); // TODO
+	if (loadOk != PSRETURN_OK) // VFS will log the failed file and the reason
+		return false;
 	std::string content = file.DecodeUTF8(); // assume it's UTF-8
 	bool ok = m_ScriptInterface.LoadScript(filename, content);
 	return ok;
 }
 
-void CComponentManager::Script_RegisterComponentType(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor)
+void CComponentManager::Script_RegisterComponentType_Common(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor1, bool reRegister, bool systemComponent)
 {
 	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 	JSContext* cx = componentManager->m_ScriptInterface.GetContext();
 	JSAutoRequest rq(cx);
+	
+	JS::RootedValue ctor(cx, ctor1.get()); // TODO: Get Handle parameter directly with SpiderMonkey 31
 
 	// Find the C++ component that wraps the interface
 	int cidWrapper = componentManager->GetScriptWrapper(iid);
@@ -163,15 +169,22 @@ void CComponentManager::Script_RegisterComponentType(ScriptInterface::CxPrivate*
 	ComponentTypeId cid = componentManager->LookupCID(cname);
 	if (cid == CID__Invalid)
 	{
+		if (reRegister)
+		{
+			componentManager->m_ScriptInterface.ReportError("ReRegistering component type that was not registered before"); // TODO: report the actual name
+			return;
+		}
 		// Allocate a new cid number
 		cid = componentManager->m_NextScriptComponentTypeId++;
 		componentManager->m_ComponentTypeIdsByName[cname] = cid;
+		if (systemComponent)
+			componentManager->MarkScriptedComponentForSystemEntity(cid);
 	}
 	else
 	{
 		// Component type is already loaded, so do hotloading:
 
-		if (!componentManager->m_CurrentlyHotloading)
+		if (!componentManager->m_CurrentlyHotloading && !reRegister)
 		{
 			componentManager->m_ScriptInterface.ReportError("Registering component type with already-registered name"); // TODO: report the actual name
 			return;
@@ -182,7 +195,7 @@ void CComponentManager::Script_RegisterComponentType(ScriptInterface::CxPrivate*
 		// We can only replace scripted component types, not native ones
 		if (ctPrevious.type != CT_Script)
 		{
-			componentManager->m_ScriptInterface.ReportError("Hotloading script component type with same name as native component");
+			componentManager->m_ScriptInterface.ReportError("Loading script component type with same name as native component");
 			return;
 		}
 
@@ -220,11 +233,11 @@ void CComponentManager::Script_RegisterComponentType(ScriptInterface::CxPrivate*
 
 	std::string schema = "<empty/>";
 	{
-		CScriptValRooted prototype;
-		if (componentManager->m_ScriptInterface.GetProperty(ctor.get(), "prototype", prototype) &&
-			componentManager->m_ScriptInterface.HasProperty(prototype.get(), "Schema"))
+		JS::RootedValue prototype(cx);
+		if (componentManager->m_ScriptInterface.GetProperty(ctor, "prototype", &prototype) &&
+			componentManager->m_ScriptInterface.HasProperty(prototype, "Schema"))
 		{
-			componentManager->m_ScriptInterface.GetProperty(prototype.get(), "Schema", schema);
+			componentManager->m_ScriptInterface.GetProperty(prototype, "Schema", schema);
 		}
 	}
 
@@ -245,7 +258,7 @@ void CComponentManager::Script_RegisterComponentType(ScriptInterface::CxPrivate*
 
 	// Find all the ctor prototype's On* methods, and subscribe to the appropriate messages:
 	JS::RootedValue protoVal(cx);
-	if (!componentManager->m_ScriptInterface.GetPropertyJS(ctor.get(), "prototype", &protoVal))
+	if (!componentManager->m_ScriptInterface.GetProperty(ctor, "prototype", &protoVal))
 		return; // error
 
 	std::vector<std::string> methods;
@@ -303,6 +316,25 @@ void CComponentManager::Script_RegisterComponentType(ScriptInterface::CxPrivate*
 	}
 }
 
+void CComponentManager::Script_RegisterComponentType(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor)
+{
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
+	componentManager->Script_RegisterComponentType_Common(pCxPrivate, iid, cname, ctor, false, false);
+	componentManager->m_ScriptInterface.SetGlobal(cname.c_str(), ctor, componentManager->m_CurrentlyHotloading);
+}
+
+void CComponentManager::Script_RegisterSystemComponentType(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor)
+{
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
+	componentManager->Script_RegisterComponentType_Common(pCxPrivate, iid, cname, ctor, false, true);
+	componentManager->m_ScriptInterface.SetGlobal(cname.c_str(), ctor, componentManager->m_CurrentlyHotloading);
+}
+
+void CComponentManager::Script_ReRegisterComponentType(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor)
+{
+	Script_RegisterComponentType_Common(pCxPrivate, iid, cname, ctor, true, false);
+}
+
 void CComponentManager::Script_RegisterInterface(ScriptInterface::CxPrivate* pCxPrivate, std::string name)
 {
 	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
@@ -384,25 +416,29 @@ std::vector<IComponent*> CComponentManager::Script_GetComponentsWithInterface(Sc
 	return ret;
 }
 
-CMessage* CComponentManager::ConstructMessage(int mtid, CScriptVal data)
+CMessage* CComponentManager::ConstructMessage(int mtid, JS::HandleValue data)
 {
 	if (mtid == MT__Invalid || mtid > (int)m_MessageTypeIdsByName.size()) // (IDs start at 1 so use '>' here)
 		LOGERROR(L"PostMessage with invalid message type ID '%d'", mtid);
 
 	if (mtid < MT__LastNative)
 	{
-		return CMessageFromJSVal(mtid, m_ScriptInterface, data.get());
+		return CMessageFromJSVal(mtid, m_ScriptInterface, data);
 	}
 	else
 	{
-		return new CMessageScripted(mtid, m_MessageTypeNamesById[mtid],
-				CScriptValRooted(m_ScriptInterface.GetContext(), data));
+		return new CMessageScripted(m_ScriptInterface, mtid, m_MessageTypeNamesById[mtid], data);
 	}
 }
 
-void CComponentManager::Script_PostMessage(ScriptInterface::CxPrivate* pCxPrivate, int ent, int mtid, CScriptVal data)
+void CComponentManager::Script_PostMessage(ScriptInterface::CxPrivate* pCxPrivate, int ent, int mtid, CScriptVal data1)
 {
 	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
+	
+	JSContext* cx = componentManager->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+	// TODO: With ESR31 we should be able to take JS::HandleValue directly
+	JS::RootedValue data(cx, data1.get());
 
 	CMessage* msg = componentManager->ConstructMessage(mtid, data);
 	if (!msg)
@@ -413,9 +449,14 @@ void CComponentManager::Script_PostMessage(ScriptInterface::CxPrivate* pCxPrivat
 	delete msg;
 }
 
-void CComponentManager::Script_BroadcastMessage(ScriptInterface::CxPrivate* pCxPrivate, int mtid, CScriptVal data)
+void CComponentManager::Script_BroadcastMessage(ScriptInterface::CxPrivate* pCxPrivate, int mtid, CScriptVal data1)
 {
 	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
+	
+	JSContext* cx = componentManager->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
+	// TODO: With ESR31 we should be able to take JS::HandleValue directly
+	JS::RootedValue data(cx, data1.get());
 
 	CMessage* msg = componentManager->ConstructMessage(mtid, data);
 	if (!msg)
@@ -459,6 +500,10 @@ void CComponentManager::Script_DestroyEntity(ScriptInterface::CxPrivate* pCxPriv
 
 void CComponentManager::ResetState()
 {
+	// Delete all dynamic message subscriptions
+	m_DynamicMessageSubscriptionsNonsync.clear();
+	m_DynamicMessageSubscriptionsNonsyncByComponent.clear();
+
 	// Delete all IComponents
 	std::map<ComponentTypeId, std::map<entity_id_t, IComponent*> >::iterator iit = m_ComponentsByTypeId.begin();
 	for (; iit != m_ComponentsByTypeId.end(); ++iit)
@@ -508,6 +553,11 @@ void CComponentManager::RegisterComponentTypeScriptWrapper(InterfaceId iid, Comp
 	// TODO: merge with RegisterComponentType
 }
 
+void CComponentManager::MarkScriptedComponentForSystemEntity(CComponentManager::ComponentTypeId cid)
+{
+	m_ScriptedSystemComponents.push_back(cid);
+}
+
 void CComponentManager::RegisterMessageType(MessageTypeId mtid, const char* name)
 {
 	m_MessageTypeIdsByName[name] = mtid;
@@ -532,6 +582,50 @@ void CComponentManager::SubscribeGloballyToMessageType(MessageTypeId mtid)
 	std::sort(types.begin(), types.end()); // TODO: just sort once at the end of LoadComponents
 }
 
+void CComponentManager::FlattenDynamicSubscriptions()
+{
+	std::map<MessageTypeId, CDynamicSubscription>::iterator it;
+	for (it = m_DynamicMessageSubscriptionsNonsync.begin();
+	     it != m_DynamicMessageSubscriptionsNonsync.end(); ++it)
+	{
+		it->second.Flatten();
+	}
+}
+
+void CComponentManager::DynamicSubscriptionNonsync(MessageTypeId mtid, IComponent* component, bool enable)
+{
+	if (enable)
+	{
+		bool newlyInserted = m_DynamicMessageSubscriptionsNonsyncByComponent[component].insert(mtid).second;
+		if (newlyInserted)
+			m_DynamicMessageSubscriptionsNonsync[mtid].Add(component);
+	}
+	else
+	{
+		size_t numRemoved = m_DynamicMessageSubscriptionsNonsyncByComponent[component].erase(mtid);
+		if (numRemoved)
+			m_DynamicMessageSubscriptionsNonsync[mtid].Remove(component);
+	}
+}
+
+void CComponentManager::RemoveComponentDynamicSubscriptions(IComponent* component)
+{
+	std::map<IComponent*, std::set<MessageTypeId> >::iterator it = m_DynamicMessageSubscriptionsNonsyncByComponent.find(component);
+	if (it == m_DynamicMessageSubscriptionsNonsyncByComponent.end())
+		return;
+
+	std::set<MessageTypeId>::iterator mtit;
+	for (mtit = it->second.begin(); mtit != it->second.end(); ++mtit)
+	{
+		m_DynamicMessageSubscriptionsNonsync[*mtit].Remove(component);
+
+		// Need to flatten the subscription lists immediately to avoid dangling IComponent* references
+		m_DynamicMessageSubscriptionsNonsync[*mtit].Flatten();
+	}
+
+	m_DynamicMessageSubscriptionsNonsyncByComponent.erase(it);
+}
+
 CComponentManager::ComponentTypeId CComponentManager::LookupCID(const std::string& cname) const
 {
 	std::map<std::string, ComponentTypeId>::const_iterator it = m_ComponentTypeIdsByName.find(cname);
@@ -599,8 +693,37 @@ bool CComponentManager::AddComponent(CEntityHandle ent, ComponentTypeId cid, con
 	return true;
 }
 
+void CComponentManager::AddSystemComponents(bool skipScriptedComponents, bool skipAI)
+{
+	CParamNode noParam;
+	AddComponent(m_SystemEntity, CID_TemplateManager, noParam);
+	AddComponent(m_SystemEntity, CID_CommandQueue, noParam);
+	AddComponent(m_SystemEntity, CID_ObstructionManager, noParam);
+	AddComponent(m_SystemEntity, CID_ParticleManager, noParam);
+	AddComponent(m_SystemEntity, CID_Pathfinder, noParam);
+	AddComponent(m_SystemEntity, CID_ProjectileManager, noParam);
+	AddComponent(m_SystemEntity, CID_RangeManager, noParam);
+	AddComponent(m_SystemEntity, CID_SoundManager, noParam);
+	AddComponent(m_SystemEntity, CID_Terrain, noParam);
+	AddComponent(m_SystemEntity, CID_TerritoryManager, noParam);
+	AddComponent(m_SystemEntity, CID_UnitRenderer, noParam);
+	AddComponent(m_SystemEntity, CID_WaterManager, noParam);
+
+	// Add scripted system components:
+	if (!skipScriptedComponents)
+	{
+		for (uint32_t i = 0; i < m_ScriptedSystemComponents.size(); ++i)
+			AddComponent(m_SystemEntity, m_ScriptedSystemComponents[i], noParam);
+		if (!skipAI)
+			AddComponent(m_SystemEntity, CID_AIManager, noParam);
+	}
+}
+
 IComponent* CComponentManager::ConstructComponent(CEntityHandle ent, ComponentTypeId cid)
 {
+	JSContext* cx = m_ScriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+	
 	std::map<ComponentTypeId, ComponentType>::const_iterator it = m_ComponentTypesById.find(cid);
 	if (it == m_ComponentTypesById.end())
 	{
@@ -622,11 +745,14 @@ IComponent* CComponentManager::ConstructComponent(CEntityHandle ent, ComponentTy
 	std::map<entity_id_t, IComponent*>& emap2 = m_ComponentsByTypeId[cid];
 
 	// If this is a scripted component, construct the appropriate JS object first
-	jsval obj = JSVAL_NULL;
+	JS::RootedValue obj(cx);
+	// TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
+	JS::RootedValue tmpCtor(cx, ct.ctor.get());
 	if (ct.type == CT_Script)
 	{
-		obj = m_ScriptInterface.CallConstructor(ct.ctor.get(), 0, JSVAL_VOID);
-		if (JSVAL_IS_VOID(obj))
+		JS::AutoValueVector argv(cx); // TODO: With SpiderMonkey 31, we can pass JS::HandleValueArray::empty()
+		m_ScriptInterface.CallConstructor(tmpCtor, argv, &obj);
+		if (obj.isNull())
 		{
 			LOGERROR(L"Script component constructor failed");
 			return NULL;
@@ -768,6 +894,10 @@ void CComponentManager::FlushDestroyedComponents()
 		std::vector<entity_id_t> queue;
 		queue.swap(m_DestructionQueue);
 
+		// Flatten all the dynamic subscriptions to ensure there are no dangling
+		// references in the 'removed' lists to components we're going to delete
+		FlattenDynamicSubscriptions();
+
 		for (std::vector<entity_id_t>::iterator it = queue.begin(); it != queue.end(); ++it)
 		{
 			entity_id_t ent = *it;
@@ -784,6 +914,7 @@ void CComponentManager::FlushDestroyedComponents()
 				if (eit != iit->second.end())
 				{
 					eit->second->Deinit();
+					RemoveComponentDynamicSubscriptions(eit->second);
 					m_ComponentTypesById[iit->first].dealloc(eit->second);
 					iit->second.erase(ent);
 					handle.GetComponentCache()->interfaces[m_ComponentTypesById[iit->first].iid] = NULL;
@@ -854,7 +985,7 @@ const CComponentManager::InterfaceListUnordered& CComponentManager::GetEntitiesW
 	return m_ComponentsByInterface[iid];
 }
 
-void CComponentManager::PostMessage(entity_id_t ent, const CMessage& msg) const
+void CComponentManager::PostMessage(entity_id_t ent, const CMessage& msg)
 {
 	// Send the message to components of ent, that subscribed locally to this message
 	std::map<MessageTypeId, std::vector<ComponentTypeId> >::const_iterator it;
@@ -879,7 +1010,7 @@ void CComponentManager::PostMessage(entity_id_t ent, const CMessage& msg) const
 	SendGlobalMessage(ent, msg);
 }
 
-void CComponentManager::BroadcastMessage(const CMessage& msg) const
+void CComponentManager::BroadcastMessage(const CMessage& msg)
 {
 	// Send the message to components of all entities that subscribed locally to this message
 	std::map<MessageTypeId, std::vector<ComponentTypeId> >::const_iterator it;
@@ -904,7 +1035,7 @@ void CComponentManager::BroadcastMessage(const CMessage& msg) const
 	SendGlobalMessage(INVALID_ENTITY, msg);
 }
 
-void CComponentManager::SendGlobalMessage(entity_id_t ent, const CMessage& msg) const
+void CComponentManager::SendGlobalMessage(entity_id_t ent, const CMessage& msg)
 {
 	// (Common functionality for PostMessage and BroadcastMessage)
 
@@ -937,8 +1068,17 @@ void CComponentManager::SendGlobalMessage(entity_id_t ent, const CMessage& msg)
 				eit->second->HandleMessage(msg, true);
 		}
 	}
-}
 
+	// Send the message to component instances that dynamically subscribed to this message
+	std::map<MessageTypeId, CDynamicSubscription>::iterator dit = m_DynamicMessageSubscriptionsNonsync.find(msg.GetType());
+	if (dit != m_DynamicMessageSubscriptionsNonsync.end())
+	{
+		dit->second.Flatten();
+		const std::vector<IComponent*>& dynamic = dit->second.GetComponents();
+		for (size_t i = 0; i < dynamic.size(); i++)
+			dynamic[i]->HandleMessage(msg, false);
+	}
+}
 
 std::string CComponentManager::GenerateSchema()
 {
@@ -1020,13 +1160,16 @@ CScriptVal CComponentManager::Script_ReadCivJSONFile(ScriptInterface::CxPrivate*
 	return ReadJSONFile(pCxPrivate, L"civs", fileName);
 }
 
-CScriptVal CComponentManager::ReadJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring filePath, std::wstring fileName)
+JS::Value CComponentManager::ReadJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring filePath, std::wstring fileName)
 {
 	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
+	JSContext* cx = pCxPrivate->pScriptInterface->GetContext();
+	JSAutoRequest rq(cx);
 
 	VfsPath path = VfsPath(filePath) / fileName;
-
-	return componentManager->GetScriptInterface().ReadJSONFile(path).get();
+	JS::RootedValue out(cx);
+	componentManager->GetScriptInterface().ReadJSONFile(path, &out);
+	return out.get();
 }
 	
 Status CComponentManager::FindJSONFilesCallback(const VfsPath& pathname, const CFileInfo& UNUSED(fileInfo), const uintptr_t cbData)
