@@ -33,6 +33,7 @@ class IComponent;
 class CParamNode;
 class CMessage;
 class CSimContext;
+class CDynamicSubscription;
 
 class CComponentManager
 {
@@ -72,7 +73,8 @@ private:
 		CScriptValRooted ctor; // only valid if type == CT_Script
 	};
 	
-	struct FindJSONFilesCallbackData {
+	struct FindJSONFilesCallbackData
+	{
 		VfsPath path;
 		std::vector<std::string> templates;
 	};
@@ -95,6 +97,8 @@ public:
 
 	void RegisterComponentType(InterfaceId, ComponentTypeId, AllocFunc, DeallocFunc, const char*, const std::string& schema);
 	void RegisterComponentTypeScriptWrapper(InterfaceId, ComponentTypeId, AllocFunc, DeallocFunc, const char*, const std::string& schema);
+	
+	void MarkScriptedComponentForSystemEntity(CComponentManager::ComponentTypeId cid);
 
 	/**
 	 * Subscribe the current component type to the given message type.
@@ -112,6 +116,23 @@ public:
 	 */
 	void SubscribeGloballyToMessageType(MessageTypeId mtid);
 
+	/**
+	 * Subscribe the given component instance to all messages of the given message type.
+	 * The component's HandleMessage will be called on any BroadcastMessage or PostMessage of
+	 * this message type, regardless of the entity.
+	 *
+	 * This can be called at any time (including inside the HandleMessage callback for this message type).
+	 *
+	 * The component type must not have statically subscribed to this message type in its ClassInit.
+	 *
+	 * The subscription status is not saved or network-synchronised. Components must remember to
+	 * resubscribe in their Deserialize methods if they still want the message.
+	 *
+	 * This is primarily intended for Interpolate and RenderSubmit messages, to avoid the cost of
+	 * sending the message to components that do not currently need to do any rendering.
+	 */
+	void DynamicSubscriptionNonsync(MessageTypeId mtid, IComponent* component, bool enabled);
+
 	/**
 	 * @param cname Requested component type name (not including any "CID" or "CCmp" prefix)
 	 * @return The component type id, or CID__Invalid if not found
@@ -166,6 +187,11 @@ public:
 	 */
 	bool AddComponent(CEntityHandle ent, ComponentTypeId cid, const CParamNode& paramNode);
 
+	/**
+	 * Add all system components to the system entity (skip the scripted components or the AI components on demand)
+	 */
+	void AddSystemComponents(bool skipScriptedComponents, bool skipAI);
+
 	/**
 	 * Adds an externally-created component, so that it is returned by QueryInterface
 	 * but does not get destroyed and does not receive messages from the component manager.
@@ -214,13 +240,13 @@ public:
 	 * components of that entity which subscribed to the message type, and by any other components
 	 * that subscribed globally to the message type.
 	 */
-	void PostMessage(entity_id_t ent, const CMessage& msg) const;
+	void PostMessage(entity_id_t ent, const CMessage& msg);
 
 	/**
 	 * Send a message, not targeted at any particular entity. The message will be received by any
 	 * components that subscribed (either globally or not) to the message type.
 	 */
-	void BroadcastMessage(const CMessage& msg) const;
+	void BroadcastMessage(const CMessage& msg);
 
 	/**
 	 * Resets the dynamic simulation state (deletes all entities, resets entity ID counters;
@@ -242,7 +268,10 @@ public:
 
 private:
 	// Implementations of functions exposed to scripts
+	static void Script_RegisterComponentType_Common(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor, bool reRegister, bool systemComponent);
 	static void Script_RegisterComponentType(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor);
+	static void Script_RegisterSystemComponentType(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor);
+	static void Script_ReRegisterComponentType(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor);
 	static void Script_RegisterInterface(ScriptInterface::CxPrivate* pCxPrivate, std::string name);
 	static void Script_RegisterMessageType(ScriptInterface::CxPrivate* pCxPrivate, std::string name);
 	static void Script_RegisterGlobal(ScriptInterface::CxPrivate* pCxPrivate, std::string name, CScriptVal value);
@@ -257,13 +286,16 @@ private:
 	static CScriptVal Script_ReadJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring fileName);
 	static CScriptVal Script_ReadCivJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring fileName);
 	static std::vector<std::string> Script_FindJSONFiles(ScriptInterface::CxPrivate* pCxPrivate, std::wstring subPath, bool recursive);
-	static CScriptVal ReadJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring filePath, std::wstring fileName);
+	static JS::Value ReadJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring filePath, std::wstring fileName);
 	
 	// callback function to handle recursively finding files in a directory
 	static Status FindJSONFilesCallback(const VfsPath&, const CFileInfo&, const uintptr_t);
 
-	CMessage* ConstructMessage(int mtid, CScriptVal data);
-	void SendGlobalMessage(entity_id_t ent, const CMessage& msg) const;
+	CMessage* ConstructMessage(int mtid, JS::HandleValue data);
+	void SendGlobalMessage(entity_id_t ent, const CMessage& msg);
+
+	void FlattenDynamicSubscriptions();
+	void RemoveComponentDynamicSubscriptions(IComponent* component);
 
 	ComponentTypeId GetScriptWrapper(InterfaceId iid);
 
@@ -279,6 +311,7 @@ private:
 
 	// TODO: some of these should be vectors
 	std::map<ComponentTypeId, ComponentType> m_ComponentTypesById;
+	std::vector<CComponentManager::ComponentTypeId> m_ScriptedSystemComponents;
 	std::vector<boost::unordered_map<entity_id_t, IComponent*> > m_ComponentsByInterface; // indexed by InterfaceId
 	std::map<ComponentTypeId, std::map<entity_id_t, IComponent*> > m_ComponentsByTypeId;
 	std::map<MessageTypeId, std::vector<ComponentTypeId> > m_LocalMessageSubscriptions;
@@ -288,6 +321,9 @@ private:
 	std::map<MessageTypeId, std::string> m_MessageTypeNamesById;
 	std::map<std::string, InterfaceId> m_InterfaceIdsByName;
 
+	std::map<MessageTypeId, CDynamicSubscription> m_DynamicMessageSubscriptionsNonsync;
+	std::map<IComponent*, std::set<MessageTypeId> > m_DynamicMessageSubscriptionsNonsyncByComponent;
+
 	std::map<entity_id_t, SEntityComponentCache*> m_ComponentCaches;
 
 	// TODO: maintaining both ComponentsBy* is nasty; can we get rid of one,
