@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -87,8 +87,8 @@ private:
 		case 'D': case WXK_RIGHT: dir = eScrollConstantDir::RIGHT; break;
 		case 'W': case WXK_UP:    dir = eScrollConstantDir::FORWARDS; break;
 		case 'S': case WXK_DOWN:  dir = eScrollConstantDir::BACKWARDS; break;
-		case 'E': case ']':       dir = eScrollConstantDir::CLOCKWISE; break;
-		case 'Q': case '[':       dir = eScrollConstantDir::ANTICLOCKWISE; break;
+		case 'Q': case '[':       dir = eScrollConstantDir::CLOCKWISE; break;
+		case 'E': case ']':       dir = eScrollConstantDir::ANTICLOCKWISE; break;
 		case WXK_SHIFT: case WXK_CONTROL: dir = -1; break;
 		default: return false;
 		}
@@ -101,8 +101,8 @@ private:
 			if (wxGetKeyState(WXK_RIGHT))      POST_MESSAGE(ScrollConstant, (eRenderView::GAME, eScrollConstantDir::RIGHT, speed));
 			if (wxGetKeyState(WXK_UP))         POST_MESSAGE(ScrollConstant, (eRenderView::GAME, eScrollConstantDir::FORWARDS, speed));
 			if (wxGetKeyState(WXK_DOWN))       POST_MESSAGE(ScrollConstant, (eRenderView::GAME, eScrollConstantDir::BACKWARDS, speed));
-			if (wxGetKeyState((wxKeyCode)']')) POST_MESSAGE(ScrollConstant, (eRenderView::GAME, eScrollConstantDir::CLOCKWISE, speed));
-			if (wxGetKeyState((wxKeyCode)'[')) POST_MESSAGE(ScrollConstant, (eRenderView::GAME, eScrollConstantDir::ANTICLOCKWISE, speed));
+			if (wxGetKeyState((wxKeyCode)'[')) POST_MESSAGE(ScrollConstant, (eRenderView::GAME, eScrollConstantDir::CLOCKWISE, speed));
+			if (wxGetKeyState((wxKeyCode)']')) POST_MESSAGE(ScrollConstant, (eRenderView::GAME, eScrollConstantDir::ANTICLOCKWISE, speed));
 			return false;
 		}
 		else
