@@ -84,22 +84,26 @@ namespace AtlasMessage {
 
 QUERYHANDLER(GenerateMap)
 {
-	InitGame();
-
-	// Random map
-	ScriptInterface& scriptInterface = g_Game->GetSimulation2()->GetScriptInterface();
-	
-	CScriptValRooted settings = scriptInterface.ParseJSON(*msg->settings);
-
-	CScriptValRooted attrs;
-	scriptInterface.Eval("({})", attrs);
-	scriptInterface.SetProperty(attrs.get(), "mapType", std::string("random"));
-	scriptInterface.SetProperty(attrs.get(), "script", std::wstring(*msg->filename));
-	scriptInterface.SetProperty(attrs.get(), "settings", settings, false);
-
 	try
 	{
-		StartGame(attrs);
+		InitGame();
+
+		// Random map
+		ScriptInterface& scriptInterface = g_Game->GetSimulation2()->GetScriptInterface();
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
+	
+		JS::RootedValue settings(cx);
+		scriptInterface.ParseJSON(*msg->settings, &settings);
+		scriptInterface.SetProperty(settings, "mapType", std::string("random"));
+
+		JS::RootedValue attrs(cx);
+		scriptInterface.Eval("({})", &attrs);
+		scriptInterface.SetProperty(attrs, "mapType", std::string("random"));
+		scriptInterface.SetProperty(attrs, "script", std::wstring(*msg->filename));
+		scriptInterface.SetProperty(attrs, "settings", settings);
+
+		StartGame(CScriptValRooted(cx, attrs));
 
 		msg->status = 0;
 	}
@@ -112,12 +116,30 @@ QUERYHANDLER(GenerateMap)
 
 		InitGame();
 
-		ScriptInterface& si = g_Game->GetSimulation2()->GetScriptInterface();
-		CScriptValRooted atts;
-		si.Eval("({})", atts);
-		si.SetProperty(atts.get(), "mapType", std::string("scenario"));
-		si.SetProperty(atts.get(), "map", std::wstring(L"maps/scenarios/_default"));
-		StartGame(atts);
+		ScriptInterface& scriptInterface = g_Game->GetSimulation2()->GetScriptInterface();
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
+
+		JS::RootedValue settings(cx);
+		scriptInterface.Eval("({})", &settings);
+		// Set up 8-element array of empty objects to satisfy init
+		JS::RootedValue playerData(cx);
+		scriptInterface.Eval("([])", &playerData);
+		for (int i = 0; i < 8; ++i)
+		{
+			JS::RootedValue player(cx);
+			scriptInterface.Eval("({})", &player);
+			scriptInterface.SetPropertyInt(playerData, i, player);
+		}
+		scriptInterface.SetProperty(settings, "mapType", std::string("scenario"));
+		scriptInterface.SetProperty(settings, "PlayerData", playerData);
+
+		JS::RootedValue atts(cx);
+		scriptInterface.Eval("({})", &atts);
+		scriptInterface.SetProperty(atts, "mapType", std::string("scenario"));
+		scriptInterface.SetProperty(atts, "map", std::wstring(L"maps/scenarios/_default"));
+		scriptInterface.SetProperty(atts, "settings", settings);
+		StartGame(CScriptValRooted(cx, atts));
 
 		msg->status = -1;
 	}
@@ -126,19 +148,21 @@ QUERYHANDLER(GenerateMap)
 MESSAGEHANDLER(LoadMap)
 {
 	InitGame();
+	
+	ScriptInterface& scriptInterface = g_Game->GetSimulation2()->GetScriptInterface();
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
 
 	// Scenario
 	CStrW map = *msg->filename;
 	CStrW mapBase = map.BeforeLast(L".pmp"); // strip the file extension, if any
-
-	ScriptInterface& scriptInterface = g_Game->GetSimulation2()->GetScriptInterface();
 	
-	CScriptValRooted attrs;
-	scriptInterface.Eval("({})", attrs);
-	scriptInterface.SetProperty(attrs.get(), "mapType", std::string("scenario"));
-	scriptInterface.SetProperty(attrs.get(), "map", std::wstring(mapBase));
+	JS::RootedValue attrs(cx);
+	scriptInterface.Eval("({})", &attrs);
+	scriptInterface.SetProperty(attrs, "mapType", std::string("scenario"));
+	scriptInterface.SetProperty(attrs, "map", std::wstring(mapBase));
 
-	StartGame(attrs);
+	StartGame(CScriptValRooted(cx, attrs));
 }
 
 MESSAGEHANDLER(ImportHeightmap)
