@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -47,8 +47,9 @@
 #include "simulation2/components/ICmpPlayerManager.h"
 #include "simulation2/components/ICmpSelectable.h"
 #include "simulation2/components/ICmpTemplateManager.h"
+#include "simulation2/components/ICmpVisual.h"
 #include "simulation2/helpers/Selection.h"
-
+#include "ps/XML/XMLWriter.h"
 
 namespace AtlasMessage {
 
@@ -67,7 +68,7 @@ QUERYHANDLER(GetObjectsList)
 	CmpPtr<ICmpTemplateManager> cmpTemplateManager(*g_Game->GetSimulation2(), SYSTEM_ENTITY);
 	if (cmpTemplateManager)
 	{
-		std::vector<std::string> names = cmpTemplateManager->FindAllTemplates(true);
+		std::vector<std::string> names = cmpTemplateManager->FindAllPlaceableTemplates(true);
 
 		for (std::vector<std::string>::iterator it = names.begin(); it != names.end(); ++it)
 		{
@@ -217,6 +218,64 @@ QUERYHANDLER(GetObjectSettings)
 	msg->settings = settings;
 }
 
+QUERYHANDLER(GetObjectMapSettings)
+{
+	std::vector<entity_id_t> ids = *msg->ids;
+	
+	CmpPtr<ICmpTemplateManager> cmpTemplateManager(*g_Game->GetSimulation2(), SYSTEM_ENTITY);
+	ENSURE(cmpTemplateManager);
+	
+	XML_Start();
+	{
+		XML_Element("Entities");
+		{
+			for (size_t i = 0; i < ids.size(); i++)
+			{
+				entity_id_t id = (entity_id_t)ids[i];
+				XML_Element("Entity");
+				{
+					//Template name
+					XML_Setting("Template", cmpTemplateManager->GetCurrentTemplateName(id));
+					
+					//Player
+					CmpPtr<ICmpOwnership> cmpOwnership(*g_Game->GetSimulation2(), id);
+					if (cmpOwnership)
+						XML_Setting("Player", (int)cmpOwnership->GetOwner());
+					
+					//Adding position to make some relative position later
+					CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), id);
+					if (cmpPosition)
+					{
+						CFixedVector3D pos = cmpPosition->GetPosition();
+						CFixedVector3D rot = cmpPosition->GetRotation();
+						{
+							XML_Element("Position");
+							XML_Attribute("x", pos.X);
+							XML_Attribute("z", pos.Z);
+							// TODO: height offset etc
+						}
+						{
+							XML_Element("Orientation");
+							XML_Attribute("y", rot.Y);
+														// TODO: X, Z maybe
+						}
+					}
+
+					// Adding actor seed
+					CmpPtr<ICmpVisual> cmpVisual(*g_Game->GetSimulation2(), id);
+					if (cmpVisual)
+						XML_Setting("ActorSeed", (unsigned int)cmpVisual->GetActorSeed());
+
+				}
+			}
+		}
+	}
+	
+	const CStr& data = XML_GetOutput();
+	msg->xmldata = std::wstring(data.begin(), data.end());
+}
+
+
 BEGIN_COMMAND(SetObjectSettings)
 {
 	player_id_t m_PlayerOld, m_PlayerNew;
@@ -282,6 +341,7 @@ END_COMMAND(SetObjectSettings);
 
 static CStrW g_PreviewUnitName;
 static entity_id_t g_PreviewEntityID = INVALID_ENTITY;
+static std::vector<entity_id_t> g_PreviewEntitiesID;
 
 static CVector3D GetUnitPos(const Position& pos, bool floating)
 {
@@ -310,20 +370,141 @@ static CVector3D GetUnitPos(const Position& pos, bool floating)
 	return vec;
 }
 
+QUERYHANDLER(GetCurrentSelection)
+{
+	msg->ids = g_Selection;
+}
+
+MESSAGEHANDLER(ObjectPreviewToEntity)
+{
+	UNUSED2(msg);
+
+	if (g_PreviewEntitiesID.size() == 0)
+		return;
+
+	CmpPtr<ICmpTemplateManager> cmpTemplateManager(*g_Game->GetSimulation2(), SYSTEM_ENTITY);
+	ENSURE(cmpTemplateManager);
+
+	PlayerColorMap playerColor;
+
+	//I need to re create the objects finally delete preview objects
+	std::vector<entity_id_t>::const_iterator i;
+	for (i = g_PreviewEntitiesID.begin(); i != g_PreviewEntitiesID.end(); i++)
+	{
+		//Get template
+		entity_id_t ent = *i;
+		std::string templateName = cmpTemplateManager->GetCurrentTemplateName(ent);
+		std::wstring wTemplateName(templateName.begin() + 8, templateName.end());
+		//Create new entity
+		entity_id_t new_ent = g_Game->GetSimulation2()->AddEntity(wTemplateName);
+		if (new_ent == INVALID_ENTITY)
+			continue;
+
+		//get position, get rotation
+		CmpPtr<ICmpPosition> cmpPositionNew(*g_Game->GetSimulation2(), new_ent);
+		CmpPtr<ICmpPosition> cmpPositionOld(*g_Game->GetSimulation2(), ent);
+
+		if (cmpPositionNew && cmpPositionOld)
+		{
+			CVector3D pos = cmpPositionOld->GetPosition();
+			cmpPositionNew->JumpTo(entity_pos_t::FromFloat(pos.X), entity_pos_t::FromFloat(pos.Z));
+
+			//now rotate
+			CFixedVector3D rotation = cmpPositionOld->GetRotation();
+			cmpPositionNew->SetYRotation(rotation.Y);
+		}
+
+		//get owner
+		CmpPtr<ICmpOwnership> cmpOwnershipNew(*g_Game->GetSimulation2(), new_ent);
+		CmpPtr<ICmpOwnership> cmpOwnershipOld(*g_Game->GetSimulation2(), ent);
+		if (cmpOwnershipNew && cmpOwnershipOld)
+			cmpOwnershipNew->SetOwner(cmpOwnershipOld->GetOwner());
+
+		//getVisual
+		CmpPtr<ICmpVisual> cmpVisualNew(*g_Game->GetSimulation2(), new_ent);
+		CmpPtr<ICmpVisual> cmpVisualOld(*g_Game->GetSimulation2(), ent);
+		if (cmpVisualNew && cmpVisualOld)
+			cmpVisualNew->SetActorSeed(cmpVisualOld->GetActorSeed());
+
+		//Update g_selectedObject and higligth
+		g_Selection.push_back(new_ent);
+		CmpPtr<ICmpSelectable> cmpSelectable(*g_Game->GetSimulation2(), new_ent);
+		if (cmpSelectable)
+			cmpSelectable->SetSelectionHighlight(GetOwnerPlayerColor(playerColor, new_ent), true);
+
+		g_Game->GetSimulation2()->DestroyEntity(ent);
+	}
+	g_PreviewEntitiesID.clear();
+
+}
+
+MESSAGEHANDLER(MoveObjectPreview)
+{
+	if (g_PreviewEntitiesID.size()==0)
+		return;
+
+	//TODO:Change pivot 
+	entity_id_t referenceEntity = *g_PreviewEntitiesID.begin();
+
+	// All selected objects move relative to a pivot object,
+	//	so get its position and whether it's floating
+	CFixedVector3D referencePos;
+	
+	CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), referenceEntity);
+	if (cmpPosition && cmpPosition->IsInWorld())
+		referencePos = cmpPosition->GetPosition();
+		
+
+	// Calculate directional vector of movement for pivot object,
+	//	we apply the same movement to all objects
+	CVector3D targetPos = GetUnitPos(msg->pos, true);
+	CFixedVector3D fTargetPos(entity_pos_t::FromFloat(targetPos.X), entity_pos_t::FromFloat(targetPos.Y), entity_pos_t::FromFloat(targetPos.Z));
+	CFixedVector3D dir = fTargetPos - referencePos;
+
+	for (size_t i = 0; i < g_PreviewEntitiesID.size(); ++i)
+	{
+		entity_id_t id = (entity_id_t)g_PreviewEntitiesID[i];
+		CFixedVector3D posFinal;
+		CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), id);
+		if (cmpPosition && cmpPosition->IsInWorld())
+		{
+			// Calculate this object's position
+			CFixedVector3D posFixed = cmpPosition->GetPosition();
+			posFinal = posFixed + dir;
+		}
+		cmpPosition->JumpTo(posFinal.X, posFinal.Z);
+	}
+}
+
 MESSAGEHANDLER(ObjectPreview)
 {
 	// If the selection has changed...
-	if (*msg->id != g_PreviewUnitName)
+	if (*msg->id != g_PreviewUnitName || (!msg->cleanObjectPreviews))
 	{
 		// Delete old entity
-		if (g_PreviewEntityID != INVALID_ENTITY)
-			g_Game->GetSimulation2()->DestroyEntity(g_PreviewEntityID);
+		if (g_PreviewEntityID != INVALID_ENTITY && msg->cleanObjectPreviews)
+		{
+			//Time to delete all preview objects
+			if (g_PreviewEntitiesID.size() > 0)
+			{
+				std::vector<entity_id_t>::const_iterator i;
+				for (i = g_PreviewEntitiesID.begin(); i != g_PreviewEntitiesID.end(); i++)
+				{
+					g_Game->GetSimulation2()->DestroyEntity(*i);
+				}
+				g_PreviewEntitiesID.clear();
+			}
+		}
 
 		// Create the new entity
 		if ((*msg->id).empty())
 			g_PreviewEntityID = INVALID_ENTITY;
 		else
+		{
 			g_PreviewEntityID = g_Game->GetSimulation2()->AddLocalEntity(L"preview|" + *msg->id);
+			g_PreviewEntitiesID.push_back(g_PreviewEntityID);
+		}
+		
 
 		g_PreviewUnitName = *msg->id;
 	}
@@ -355,6 +536,10 @@ MESSAGEHANDLER(ObjectPreview)
 
 		// TODO: handle random variations somehow
 
+		CmpPtr<ICmpVisual> cmpVisual(*g_Game->GetSimulation2(), g_PreviewEntityID);
+		if (cmpVisual)
+			cmpVisual->SetActorSeed(msg->actorseed);
+
 		CmpPtr<ICmpOwnership> cmpOwnership(*g_Game->GetSimulation2(), g_PreviewEntityID);
 		if (cmpOwnership)
 			cmpOwnership->SetOwner((player_id_t)msg->settings->player);
@@ -365,9 +550,9 @@ BEGIN_COMMAND(CreateObject)
 {
 	CVector3D m_Pos;
 	float m_Angle;
-	size_t m_ID; // old simulation system
 	player_id_t m_Player;
-	entity_id_t m_EntityID; // new simulation system
+	entity_id_t m_EntityID;
+	u32 m_ActorSeed;
 
 	void Do()
 	{
@@ -386,8 +571,9 @@ BEGIN_COMMAND(CreateObject)
 			m_Angle = msg->angle;
 		}
 
-		// TODO: variations too
 		m_Player = (player_id_t)msg->settings->player;
+		m_ActorSeed = msg->actorseed;
+		// TODO: variation/selection strings
 
 		Redo();
 	}
@@ -409,7 +595,12 @@ BEGIN_COMMAND(CreateObject)
 		if (cmpOwnership)
 			cmpOwnership->SetOwner(m_Player);
 
-		// TODO: handle random variations somehow
+		CmpPtr<ICmpVisual> cmpVisual(*g_Game->GetSimulation2(), m_EntityID);
+		if (cmpVisual)
+		{
+			cmpVisual->SetActorSeed(m_ActorSeed);
+			// TODO: variation/selection strings
+		}
 	}
 
 	void Undo()
@@ -594,7 +785,7 @@ BEGIN_COMMAND(RotateObject)
 		m_AngleOld = cmpPosition->GetRotation().Y.ToFloat();
 		if (msg->usetarget)
 		{
-			CMatrix3D transform = cmpPosition->GetInterpolatedTransform(0.f, false);
+			CMatrix3D transform = cmpPosition->GetInterpolatedTransform(0.f);
 			CVector3D pos = transform.GetTranslation();
 			CVector3D target = msg->target->GetWorldSpace(pos.Y);
 			m_AngleNew = atan2(target.X-pos.X, target.Z-pos.Z);
