@@ -334,6 +334,20 @@ struct sObjectSettings
 SHAREABLE_STRUCT(sObjectSettings);
 #endif
 
+// transform de local entity to a real entity
+MESSAGE(ObjectPreviewToEntity,);
+
+//Query for get selected objects
+QUERY(GetCurrentSelection,
+	, //No inputs
+	((std::vector<ObjectID>, ids))
+	);
+
+// Moving Preview(s) object together, default is using the firs element in vector
+MESSAGE(MoveObjectPreview,
+		((Position,pos))
+		);
+
 // Preview object in the game world - creates a temporary unit at the given
 // position, and removes it when the preview is next changed
 MESSAGE(ObjectPreview,
@@ -344,6 +358,7 @@ MESSAGE(ObjectPreview,
 		((Position, target))
 		((float, angle))
 		((unsigned int, actorseed))
+		((bool, cleanObjectPreviews))
 		);
 
 COMMAND(CreateObject, NOMERGE,
@@ -421,14 +436,14 @@ MESSAGE(SetView,
 #ifndef MESSAGES_SKIP_STRUCTS
 struct sEnvironmentSettings
 {
+	Shareable<std::wstring> watertype; // range 0..1 corresponds to min..max terrain height; out-of-bounds values allowed
 	Shareable<float> waterheight; // range 0..1 corresponds to min..max terrain height; out-of-bounds values allowed
 	Shareable<float> waterwaviness; // range ???
 	Shareable<float> watermurkiness; // range ???
-	
+	Shareable<float> windangle;
+
 	Shareable<Colour> watercolour;
 	Shareable<Colour> watertint;
-	Shareable<Colour> waterreflectiontint;
-	Shareable<float> waterreflectiontintstrength; // range ???
 
 	Shareable<float> sunrotation; // range -pi..+pi
 	Shareable<float> sunelevation; // range -pi/2 .. +pi/2
@@ -590,6 +605,13 @@ COMMAND(SetObjectSettings, NOMERGE,
 		((sObjectSettings, settings))
 		);
 
+QUERY(GetObjectMapSettings,
+		((std::vector<ObjectID>, ids))
+		,
+		((std::wstring, xmldata))
+		);
+
+
 QUERY(GetPlayerObjects,
 		((int, player))
 		,
